#include <mpcl/util/logging/log.hh>
#include "odbc-3.x.hh"

#line 1 "../test-001-sql_types.uesql.cc"
/*
*  Name:         test-001-sql_types.uesql.cc
*  Author:       Rafael Jesus Alcantara Perez
*  Contributor:  Francisco Vides Fernandez
*  Summary:      Test for UESQL
*  Date:         $Date: 2003/10/06 13:21:36 $
*  Revision:     $Revision: 1.6 $
*
*  Copyright (C) 1996-2003  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/test.h>
#include <mpcl/text/string.hh>
#include <mpcl/util/prefs/config_processor.hh>


//
//  L O C A L   V A R I A B L E S
//

static uesqlc::TSqlstateDescriptor   _tSqlstateDescriptor;
  

//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::text::TString _GetErrorMessage (void)
{

  using mpcl::text::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesqlc::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


static int _TestSqlTypes (void)
{

  TEST_INIT ("test for SQL types");

  using std::cerr;
  using std::endl;
  using std::size_t;
  using uesqlc::TDate;
  using uesqlc::TInteger;
  using uesqlc::TReal;
  using uesqlc::TString;
  using uesqlc::TTime;

  TDate      tDate;
  TInteger   tInteger;
  TReal      tReal;
  TTime      tTime;
  TString    yString;
  TString    yString4;
  TString    yString1;
  TString    yString2;
  TString    yString3;
  int        iRecordCount = 0;

  //
  //  Checks for quote escaping.
  //
  TEST_STRINGS ("'hello'",        TString ("hello").get().c_str());
  TEST_STRINGS ("'hello''bye'",   TString ("hello'bye").get().c_str());
  TEST_STRINGS ("'hello''''bye'", TString ("hello''bye").get().c_str());
  TEST_STRINGS ("'hello\\\\bye'", TString ("hello\\bye").get().c_str());

  //
  //  Other checks.
  //
                                     TEST_NUMBERS (1,     TInteger (true));
                                     TEST_NUMBERS (0,     TInteger (false));
  tInteger.set ("1");                TEST_NUMBERS (1,     tInteger);
  tInteger.set ("0");                TEST_NUMBERS (0,     tInteger);
  tInteger.set ("10000");            TEST_NUMBERS (10000, tInteger);
                                     TEST_NUMBERS (10001, TInteger (10001));
  tDate.set    ((const char*) NULL); TEST_NUMBERS (true,  tDate.isNull());
  tInteger.set ((const char*) NULL); TEST_NUMBERS (true,  tInteger.isNull());
  tReal.set    ((const char*) NULL); TEST_NUMBERS (true,  tReal.isNull());
  yString.set  ((const char*) NULL); TEST_NUMBERS (true,  yString.isNull());
  tDate.set    ("");                 TEST_NUMBERS (true,  tDate.isNull());
  tInteger.set ("");                 TEST_NUMBERS (true,  tInteger.isNull());
  tReal.set    ("");                 TEST_NUMBERS (true,  tReal.isNull());
  yString.set  ("");                 TEST_NUMBERS (false, yString.isNull());
                                     TEST_NUMBERS (true,  TDate().isNull());
                                     TEST_NUMBERS (true,  TInteger().isNull());
                                     TEST_NUMBERS (true,  TReal().isNull());
                                     TEST_NUMBERS (true,  TString().isNull());
  tInteger.getBuffer (12);
  tInteger.getBuffer (12);
  tInteger.getBuffer (12);
  tInteger.set ("75000");
  TEST_NUMBERS (75000, tInteger);
#line 122 "../test-001-sql_types.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 123 "../test-001-sql_types.uesql.cc"

#line 124 "../test-001-sql_types.uesql.cc"

#line 132 "../test-001-sql_types.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("CREATE TABLE TABLE_1 (CODE CHARACTER (3) NOT NULL, BIRTH_DATE DATE, BIRTH_TIME TIME, SALARY INTEGER, PERCENTAGE DOUBLE PRECISION);");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 133 "../test-001-sql_types.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("INSERT INTO TABLE_1 VALUES ('000', '1999-12-31', '01:00', 12000, 25.75);");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 134 "../test-001-sql_types.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("INSERT INTO TABLE_1 VALUES ('001', '1970-01-01', '02:00', 75000, 30);");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 135 "../test-001-sql_types.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("INSERT INTO TABLE_1 VALUES ('002', '2000-01-01', '03:00', 11500, 99.99);");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 140 "../test-001-sql_types.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnInputQueue (TDate (2000, 12, 31));
uesqlc::htOdbc3xManager.pushOnInputQueue (TTime ("12:00:00"));
uesqlc::htOdbc3xManager.pushOnInputQueue (TInteger (7500));
uesqlc::htOdbc3xManager.pushOnInputQueue (TReal (33.33));

htResultTriple = uesqlc::htOdbc3xManager.execute ("INSERT INTO TABLE_1 VALUES ('003', $z, $z, $z, $z);");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 145 "../test-001-sql_types.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnInputQueue (TDate (2001, 1, 1));
uesqlc::htOdbc3xManager.pushOnInputQueue (TTime (23, 59, 59));
uesqlc::htOdbc3xManager.pushOnInputQueue (TInteger (6500));
uesqlc::htOdbc3xManager.pushOnInputQueue (TReal (0.0));

htResultTriple = uesqlc::htOdbc3xManager.execute ("INSERT INTO TABLE_1 VALUES ('004', $z, $z, $z, $z);");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 149 "../test-001-sql_types.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;
int                              hI = 0;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnOutputQueue (&yString1);

htResultTriple = uesqlc::htOdbc3xManager.execute ("SELECT 'hello\rbye' FROM TABLE_1 WHERE ( CODE = '000' );");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
if ( uesqlc::htOdbc3xManager.isNotFoundCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto loop_end;
}
else if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  bool   gIsSingleRow;
  int    hiLength;

SQLColAttribute (htResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( htResultTriple.tSqlhstmt                                                                ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
  gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_SUCCESS );
  if ( gIsSingleRow )
  {
    gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_NO_DATA );
  }
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  if ( gIsSingleRow )
  {
    uesqlc::htOdbc3xManager.updateOutputVariables();
  }
  else
  {
    // 
    //  Condition:  cardinality violation
    // 
    uesqlc::SQLCODE    = -1;
    uesqlc::SQLMESSAGE = "";
    uesqlc::SQLSTATE   = "21000";
    goto error;
  }
}
}
#line 150 "../test-001-sql_types.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;
int                              hI = 0;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnOutputQueue (&yString2);

htResultTriple = uesqlc::htOdbc3xManager.execute ("SELECT 'hello\"bye' FROM TABLE_1 WHERE ( CODE = '000' );");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
if ( uesqlc::htOdbc3xManager.isNotFoundCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto loop_end;
}
else if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  bool   gIsSingleRow;
  int    hiLength;

SQLColAttribute (htResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( htResultTriple.tSqlhstmt                                                                ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
  gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_SUCCESS );
  if ( gIsSingleRow )
  {
    gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_NO_DATA );
  }
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  if ( gIsSingleRow )
  {
    uesqlc::htOdbc3xManager.updateOutputVariables();
  }
  else
  {
    // 
    //  Condition:  cardinality violation
    // 
    uesqlc::SQLCODE    = -1;
    uesqlc::SQLMESSAGE = "";
    uesqlc::SQLSTATE   = "21000";
    goto error;
  }
}
}
#line 151 "../test-001-sql_types.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;
int                              hI = 0;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnOutputQueue (&yString3);

htResultTriple = uesqlc::htOdbc3xManager.execute ("SELECT 'hello''bye' FROM TABLE_1 WHERE ( CODE = '000' );");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
if ( uesqlc::htOdbc3xManager.isNotFoundCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto loop_end;
}
else if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  bool   gIsSingleRow;
  int    hiLength;

SQLColAttribute (htResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( htResultTriple.tSqlhstmt                                                                ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
  gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_SUCCESS );
  if ( gIsSingleRow )
  {
    gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_NO_DATA );
  }
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  if ( gIsSingleRow )
  {
    uesqlc::htOdbc3xManager.updateOutputVariables();
  }
  else
  {
    // 
    //  Condition:  cardinality violation
    // 
    uesqlc::SQLCODE    = -1;
    uesqlc::SQLMESSAGE = "";
    uesqlc::SQLSTATE   = "21000";
    goto error;
  }
}
}
#line 152 "../test-001-sql_types.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;
int                              hI = 0;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnInputQueue (TString ("\\"));
uesqlc::htOdbc3xManager.pushOnOutputQueue (&yString4);

htResultTriple = uesqlc::htOdbc3xManager.execute ("SELECT $z FROM TABLE_1 WHERE ( CODE = '000' );");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
if ( uesqlc::htOdbc3xManager.isNotFoundCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto loop_end;
}
else if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  bool   gIsSingleRow;
  int    hiLength;

SQLColAttribute (htResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( htResultTriple.tSqlhstmt                                                                ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
  gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_SUCCESS );
  if ( gIsSingleRow )
  {
    gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_NO_DATA );
  }
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  if ( gIsSingleRow )
  {
    uesqlc::htOdbc3xManager.updateOutputVariables();
  }
  else
  {
    // 
    //  Condition:  cardinality violation
    // 
    uesqlc::SQLCODE    = -1;
    uesqlc::SQLMESSAGE = "";
    uesqlc::SQLSTATE   = "21000";
    goto error;
  }
}
}
#line 158 "../test-001-sql_types.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;
mpcl::text::TString              hyCursorName;

uesqlc::htOdbc3xManager.clearQueues();

//
//  First, it instantiates <cursor name> to follow the input order.
//
hyCursorName   = uesqlc::htOdbc3xManager.instantiate ("MYCURSOR");
htResultTriple = uesqlc::htOdbc3xManager.execute ("SELECT CODE, BIRTH_DATE, BIRTH_TIME, SALARY, PERCENTAGE FROM TABLE_1;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  SQLSetCursorName (htResultTriple.tSqlhstmt, (SQLCHAR*) hyCursorName.c_str(), SQL_NTS);
  uesqlc::htOdbc3xManager.addCursor (hyCursorName, htResultTriple);
}
}
#line 159 "../test-001-sql_types.uesql.cc"
uesqlc::htOdbc3xManager.clearQueues();


}  // not ODBC 3.x
#line 160 "../test-001-sql_types.uesql.cc"

  TEST_STRINGS ("'hello\rbye'", yString1.get().c_str());
  TEST_STRINGS ("'hello\"bye'", yString2.get().c_str());
  TEST_STRINGS ("'hello''bye'", yString3.get().c_str());
  TEST_STRINGS ("'\\\\'",       yString4.get().c_str());
  TEST_NUMBERS ("hello\rbye",   yString1);
  TEST_NUMBERS ("hello\"bye",   yString2);
  TEST_NUMBERS ("hello'bye",    yString3);
  TEST_NUMBERS ("\\",           yString4);

  //
  //  Writes the example table.
  //
  if ( hptTestTargetFile )
  {
    fprintf (hptTestTargetFile, "%s\n", "CODE  BIRTH_DATE   BIRTH_DATE SALARY PERCENTAGE");
    fprintf (hptTestTargetFile, "%s\n", "===== ============ ========== ====== ==========");
  }
  while ( true )
  {
#line 181 "../test-001-sql_types.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 182 "../test-001-sql_types.uesql.cc"
{
mpcl::text::TString   hyCursorName;
int                   hI = 0;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnOutputQueue (&yString);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&tDate);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&tTime);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&tInteger);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&tReal);

hyCursorName = uesqlc::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesqlc::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "34000";
  goto error;
}
else
{
int                                   hiLength;
uesqlc::odbc_3_x::TResultTriple&   hrtResultTriple (uesqlc::htOdbc3xManager.getCursor (hyCursorName));

if ( !hrtResultTriple.gIsBound )
{
hrtResultTriple.gIsBound = true;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
}
hrtResultTriple.tSqlreturn = SQLFetch (hrtResultTriple.tSqlhstmt);
uesqlc::htOdbc3xManager.diagnose (hrtResultTriple);
if ( uesqlc::htOdbc3xManager.isNotFoundCondition() )
{
  goto loop_end;
}
else if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
else
{
  uesqlc::htOdbc3xManager.updateOutputVariables();
}
}
}
}  // not ODBC 3.x
#line 183 "../test-001-sql_types.uesql.cc"

    if ( hptTestTargetFile )
    {
      fprintf ( hptTestTargetFile              ,
                "%5s %12s %10s %6ld %10.03f\n" ,
                yString.get().c_str()          ,
                tDate.get().c_str()            ,
                tTime.get().c_str()            ,
                (long int) tInteger            ,
                (double) tReal                 );
    }
    ++iRecordCount;
  }

loop_end:
#line 199 "../test-001-sql_types.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 200 "../test-001-sql_types.uesql.cc"
{
mpcl::text::TString   hyCursorName;

uesqlc::htOdbc3xManager.clearQueues();

hyCursorName = uesqlc::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesqlc::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "34000";
  goto error;
}
else
{
uesqlc::odbc_3_x::TResultTriple&   hrtResultTriple (uesqlc::htOdbc3xManager.getCursor (hyCursorName));

uesqlc::htOdbc3xManager.clearResult (hrtResultTriple);
uesqlc::htOdbc3xManager.removeCursor (hyCursorName);
}
}
#line 201 "../test-001-sql_types.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("DROP TABLE TABLE_1 CASCADE;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not ODBC 3.x
#line 202 "../test-001-sql_types.uesql.cc"

  if ( iRecordCount == 5 )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesqlc::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesqlc::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);  

end:  
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // _TestSqlTypes()


/**
*  Entry  point.  It uses the  user login 'test/test'  and the schema  location
*  'localhost/test'.
*
*  Parameters:
*
*    {-o, --dbms-server-name}=DBMS-SERVER-NAME
*/
int main (int argc, const char* argv[])
{

  using mpcl::text::Format;
  using mpcl::util::prefs::TConfigProcessor;
  using std::cerr;
  using std::endl;
  using uesqlc::ComposeSchemaLocation;
  using uesqlc::ComposeUserLogin;
  using uesqlc::TIdentifier;
  using uesqlc::TInteger;
  using uesqlc::TSearchCondition;
  using uesqlc::TString;
  
  TEST_INIT ("test-1");
  TEST_START_WATCH;

  TInteger           tHours;
  TInteger           tCount;
  TSearchCondition   tCondition;
  TString            yVar0;
  TString            yVar1;
  TString            yVar2;
  TIdentifier        yCursorName;
  TString            yDbmsUserLogin;
  TString            yDbmsSchemaLocation;
  TString            yDbmsServerName;
  TConfigProcessor   tConfigProcessor (argc, argv);
  TString            yCodeValue ("002");
  const char*        pcDbmsServerName = getenv ("DBMS_SERVER_NAME");

  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }
    
    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name", "o", pcDbmsServerName);
    tConfigProcessor.processOptions();
    yDbmsServerName     = tConfigProcessor ["dbms-server-name"].getValue();
    yDbmsUserLogin      = ComposeUserLogin      ("test",          "test");
    yDbmsSchemaLocation = ComposeSchemaLocation (yDbmsServerName, "test");
  }
  catch (const mpcl::TException& rktEXCEPTION)
  {
    cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }

  TEST_WRITE_INFO ("DBMS-SCHEMA-LOCATION = %s", yDbmsSchemaLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN      = %s", yDbmsUserLogin.c_str());
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.also (cerr);
  }
  if ( !yDbmsServerName.empty() )
  {
#line 299 "../test-001-sql_types.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 303 "../test-001-sql_types.uesql.cc"

#line 308 "../test-001-sql_types.uesql.cc"
{
mpcl::text::TString              hySqlSchemaLocation;
mpcl::text::TString              hyDatabaseName;
mpcl::text::TString              hyConnectionName;
mpcl::text::TString              hyUserLogin;
mpcl::text::TString::size_type   zSolidusPos;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnInputQueue (yDbmsSchemaLocation);
uesqlc::htOdbc3xManager.pushOnInputQueue (yDbmsUserLogin);

//
//  '$z' has been omitted.
//
hySqlSchemaLocation = uesqlc::htOdbc3xManager.instantiate ("$z");
hyConnectionName    = "DEFAULT";
hyUserLogin         = uesqlc::htOdbc3xManager.instantiate ("$z");
hySqlSchemaLocation.suppress ('\'');
zSolidusPos = hySqlSchemaLocation.find ('/');
if ( zSolidusPos == mpcl::text::TString::npos )
{
  hyDatabaseName = hySqlSchemaLocation;
}
else
{
  hyDatabaseName = hySqlSchemaLocation.substr (0, zSolidusPos);
}
if ( uesqlc::htOdbc3xManager.isConnected (hyConnectionName) )
{
  uesqlc::htOdbc3xManager.unbindConnection (hyConnectionName);
}
if ( uesqlc::htOdbc3xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "08002";
  goto error;
}
else
{
  uesqlc::htOdbc3xManager.connect ( hyConnectionName ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
  {
    goto error;
  }
  else
  {
    uesqlc::htOdbc3xManager.setCurrentConnection (hyConnectionName);
  }
}
}
}  // not ODBC 3.x
#line 309 "../test-001-sql_types.uesql.cc"

    TEST_FUNCTION (_TestSqlTypes());
#line 312 "../test-001-sql_types.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 316 "../test-001-sql_types.uesql.cc"
{
uesqlc::htOdbc3xManager.clearQueues();

if ( uesqlc::htOdbc3xManager.existsDefaultConnection() )
{
  uesqlc::htOdbc3xManager.resetStatus();
  uesqlc::htOdbc3xManager.unbindCurrentConnection();
}
else
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection does not exist
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "08003";
  goto error;
}
}
}  // not ODBC 3.x
#line 317 "../test-001-sql_types.uesql.cc"

    goto end;

  error:
    cerr << "SQLSTATE="    << uesqlc::SQLSTATE    << endl;
    cerr << "SQLCODE="     << uesqlc::SQLCODE     << endl;
    cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
    TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);
    goto end;
  }

end:
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.erase (cerr);
  }
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH ("UESQL benchmark");
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
