#include <mpcl/util/logging/log.hh>
#include "postgresql-libpq-6.2.x.hh"

#line 1 "../begin.uesql.cc"
/*
*  Name:         begin.uesql.cc
*  Author:       Francisco Vides Fernandez
*  Contributor:  Rafael Jesus Alcantara Perez
*  Summary:      Database creation of iterators test
*  Date:         $Date: 2003/10/06 13:21:36 $
*  Revision:     $Revision: 1.2 $
*
*  Copyright (C) 2001-2003  Francisco Vides Fernandez <pax@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <iostream>
#include <mpcl/exceptions.hh>
#include <mpcl/test.h>
#include <mpcl/text/string.hh>
#include <mpcl/util/logging/log.hh>
#include <mpcl/util/prefs/config_processor.hh>

using uesqlc::TSqlerrorException;


//
//  L O C A L   V A R I A B L E S
//

static uesqlc::TSqlstateDescriptor   _tSqlstateDescriptor;


//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::text::TString _GetErrorMessage (void)
{

  using mpcl::text::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesqlc::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


static void _CreateTestSchema (const mpcl::text::TString& rkyDBMS_SERVER_NAME)
{

  using uesqlc::ComposeSchemaLocation;
  using uesqlc::ComposeUserLogin;
  using uesqlc::TString;

  TString   yDbmsSchemaLocation;
  TString   yDbmsTestSchemaLocation;
  TString   yDbmsUserLogin;

  yDbmsSchemaLocation     = ComposeSchemaLocation (rkyDBMS_SERVER_NAME, "test");
  yDbmsTestSchemaLocation = ComposeSchemaLocation (rkyDBMS_SERVER_NAME, "TEST_ITERATORS_SCHEMA");
  yDbmsUserLogin          = ComposeUserLogin ("test", "test");
  TEST_WRITE_INFO ("DBMS-SCHEMA-LOCATION      = %s", yDbmsSchemaLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN           = %s", yDbmsUserLogin.c_str());
  TEST_WRITE_INFO ("DBMS-TEST-SCHEMA-LOCATION = %s", yDbmsTestSchemaLocation.c_str());
#line 83 "../begin.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 87 "../begin.uesql.cc"
{
mpcl::text::TString   hySqlServerDescriptor;
mpcl::text::TString   hySqlServerName;
mpcl::text::TString   hyDatabaseName;
mpcl::text::TString   hyConnectionName;
mpcl::text::TString   hyUserLogin;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnInputQueue (yDbmsSchemaLocation);
uesqlc::htPostgresql62xManager.pushOnInputQueue (yDbmsUserLogin);

{
  mpcl::text::TString::size_type   hzBarIndex;

  hySqlServerDescriptor = uesqlc::htPostgresql62xManager.instantiate ("$z");
  hyConnectionName      = "DEFAULT";
  hyUserLogin           = uesqlc::htPostgresql62xManager.instantiate ("$z");
  hySqlServerDescriptor.suppress ('\'');
  hzBarIndex = hySqlServerDescriptor.find ('/');
  if ( hzBarIndex == mpcl::text::TString::npos )
  {
    hySqlServerName = hySqlServerDescriptor;
  }
  else
  {
    hySqlServerName = hySqlServerDescriptor.substr (0, hzBarIndex);
    hyDatabaseName  = hySqlServerDescriptor.substr (hzBarIndex + 1);
  }
  if ( uesqlc::htPostgresql62xManager.isConnected (hyConnectionName) )
  {
    uesqlc::htPostgresql62xManager.unbindConnection (hyConnectionName);
  }
}
if ( uesqlc::htPostgresql62xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "08002";
  
throw uesqlc::TSqlerrorException (__FILE__, __LINE__);
}
else
{
  if ( hyDatabaseName.empty() )
  {
    //
    //  If there is no PostgreSQL database then connect to 'template1'
    //  (that should always exists).
    //
    hyDatabaseName = "template1";
  }
  uesqlc::htPostgresql62xManager.connect ( hyConnectionName ,
                                     hySqlServerName  ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
  {
    
throw uesqlc::TSqlerrorException (__FILE__, __LINE__);
  }
  else
  {
    uesqlc::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
  }
}
}
#line 99 "../begin.uesql.cc"
{
PGresult*             ptPgresult;
mpcl::text::TString   hyConnectionName;
mpcl::text::TString   hyLastConnectionName = uesqlc::htPostgresql62xManager.currentConnectionName();

uesqlc::htPostgresql62xManager.clearQueues();

hyConnectionName = uesqlc::htPostgresql62xManager.instantiate ("TEST_ITERATORS_SCHEMA");
hyConnectionName.lowercase();
ptPgresult = uesqlc::htPostgresql62xManager.execute (mpcl::text::Format ("CREATE DATABASE %s;", hyConnectionName.c_str()));
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  
throw uesqlc::TSqlerrorException (__FILE__, __LINE__);
}
uesqlc::htPostgresql62xManager.connect (hyConnectionName, hyConnectionName);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  
throw uesqlc::TSqlerrorException (__FILE__, __LINE__);
}
else
{
  uesqlc::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
  
{
PGresult*   ptPgresult;

ptPgresult = uesqlc::htPostgresql62xManager.execute ("CREATE TABLE INSTANCE (ATTRIB0 DATE, ATTRIB1 INT4, ATTRIB2 FLOAT8, ATTRIB3 VARCHAR (20));");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  
throw uesqlc::TSqlerrorException (__FILE__, __LINE__);
}
}
  uesqlc::htPostgresql62xManager.unbindCurrentConnection();
  uesqlc::htPostgresql62xManager.setCurrentConnection (hyLastConnectionName);
}
}
#line 103 "../begin.uesql.cc"
{
mpcl::text::TString   hySqlServerDescriptor;
mpcl::text::TString   hySqlServerName;
mpcl::text::TString   hyDatabaseName;
mpcl::text::TString   hyConnectionName;
mpcl::text::TString   hyUserLogin;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnInputQueue (yDbmsTestSchemaLocation);
uesqlc::htPostgresql62xManager.pushOnInputQueue (yDbmsUserLogin);

{
  mpcl::text::TString::size_type   hzBarIndex;

  hySqlServerDescriptor = uesqlc::htPostgresql62xManager.instantiate ("$z");
  hyConnectionName      = uesqlc::htPostgresql62xManager.instantiate ("_'test_connection'");
  hyUserLogin           = uesqlc::htPostgresql62xManager.instantiate ("$z");
  hySqlServerDescriptor.suppress ('\'');
  hyConnectionName.suppress ('\'');
  hzBarIndex = hySqlServerDescriptor.find ('/');
  if ( hzBarIndex == mpcl::text::TString::npos )
  {
    hySqlServerName = hySqlServerDescriptor;
  }
  else
  {
    hySqlServerName = hySqlServerDescriptor.substr (0, hzBarIndex);
    hyDatabaseName  = hySqlServerDescriptor.substr (hzBarIndex + 1);
  }
}
if ( uesqlc::htPostgresql62xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "08002";
  
throw uesqlc::TSqlerrorException (__FILE__, __LINE__);
}
else
{
  if ( hyDatabaseName.empty() )
  {
    //
    //  If there is no PostgreSQL database then connect to 'template1'
    //  (that should always exists).
    //
    hyDatabaseName = "template1";
  }
  uesqlc::htPostgresql62xManager.connect ( hyConnectionName ,
                                     hySqlServerName  ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
  {
    
throw uesqlc::TSqlerrorException (__FILE__, __LINE__);
  }
  else
  {
    uesqlc::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
  }
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 104 "../begin.uesql.cc"


}  // _CreateTestSchema()


static void _FillSchema (void)
{

#line 113 "../begin.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 114 "../begin.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO INSTANCE VALUES ('20-12-2000', 0, 0.0, 'valor 0');");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  
throw uesqlc::TSqlerrorException (__FILE__, __LINE__);
}
}
#line 115 "../begin.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO INSTANCE VALUES ('20-12-2001', 1, 1.1, 'valor 1');");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  
throw uesqlc::TSqlerrorException (__FILE__, __LINE__);
}
}
#line 116 "../begin.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO INSTANCE VALUES ('20-12-2002', 2, 2.2, 'valor 2');");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  
throw uesqlc::TSqlerrorException (__FILE__, __LINE__);
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 117 "../begin.uesql.cc"


}   // _FillSchema()


int main (int argc, const char** argv)
{

  mpcl::util::prefs::TConfigProcessor   tConfigProcessor (argc, argv);
  const char*                           pcDbmsServerName = getenv ("DBMS_SERVER_NAME");

  TEST_INIT ("Database creation for iterators test");
  TEST_START_WATCH;
  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }

    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name", "o", pcDbmsServerName);
    tConfigProcessor["dbms-server-name"].setUsageString ("Server where database resides");
    tConfigProcessor.processOptions();
    
    if ( !TEST_IN_SILENT_MODE )
    {
      mpcl::util::logging::tLog.also (std::cerr);
    }
#line 152 "../begin.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 156 "../begin.uesql.cc"

}  // not PostgreSQL LIBPQ 6.2.x
#line 158 "../begin.uesql.cc"

    _CreateTestSchema (tConfigProcessor ["dbms-server-name"].getValue());
    _FillSchema();
  }
  catch (const uesqlc::TSqlerrorException& rktEXCEPTION)
  {
    std::cerr << "SQLSTATE="    << uesqlc::SQLSTATE    << std::endl;
    std::cerr << "SQLCODE="     << uesqlc::SQLCODE     << std::endl;
    std::cerr << "DESCRIPTION=" << _GetErrorMessage() << std::endl;
    TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);
  }
  catch (const mpcl::TException& rktEXCEPTION)
  {
    std::cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }  
  catch (...)
  {
    std::cerr << "Unhandled exception" << std::endl;
    TEST_FAIL_WITH_CODE (3);
  }
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.erase (std::cerr);
  }
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
