
#include "postgresql-libpq-6.2.x.hh"


//
//  C O N S T R U C T O R S
//

void uesqlc::TPostgresql62xManager::
connect ( const mpcl::text::TString& rkyCONNECTION_NAME ,
          const mpcl::text::TString& rkyHOST_NAME       ,
          const mpcl::text::TString& rkyDATABASE_NAME   ,
          const mpcl::text::TString& rkyUSER_NAME       ,
          const mpcl::text::TString& rkyUSER_PASSWORD   )
{

  if ( isConnected (rkyCONNECTION_NAME) )
  {
    //
    //  Condition:           connection exception
    //  Class subcondition:  connection name in use
    //
    SQLCODE    = -1;
    SQLMESSAGE = "";
    SQLSTATE   = "08002";
  }
  else
  {
    using postgresql_6_2_x::TConnectionData;

    mpcl::text::TString   yDatabaseName (mpcl::text::Lowercase (rkyDATABASE_NAME));
    TConnectionData       tConnectionData ( rkyHOST_NAME                              ,
                                            yDatabaseName                             ,
                                            rkyUSER_NAME                              ,
                                            rkyUSER_PASSWORD                          ,
                                            PQsetdbLogin ( rkyHOST_NAME.c_str()     ,
                                                           NULL                     ,
                                                           NULL                     ,
                                                           NULL                     ,
                                                           yDatabaseName.c_str()    ,
                                                           rkyUSER_NAME.c_str()     ,
                                                           rkyUSER_PASSWORD.c_str() ) );

    diagnoseConnection (tConnectionData);
    if ( !isSqlerrorCondition() )
    {
      PGresult*   ptPgresult;

      bindConnection (rkyCONNECTION_NAME, tConnectionData);
#     ifdef _UESQL_DEBUG__
      mpcl::util::logging::tLog << mpcl::util::logging::header 
                                << "CONNECT TO '" << rkyHOST_NAME << "/" << yDatabaseName    << "' "
                                <<  "USER '"      << rkyUSER_NAME << "/" << rkyUSER_PASSWORD << "' "
                                <<  "AS '"        << rkyCONNECTION_NAME  << "';\n";
#     endif  // not _UESQL_DEBUG__

      //
      //  Set date style to ISO SQL.
      //
      ptPgresult = execute ("SET DATESTYLE TO 'ISO';", rkyCONNECTION_NAME.c_str());
      diagnose (ptPgresult);
      clearResult (ptPgresult);
    }
  }

}  // connect()


void uesqlc::TPostgresql62xManager::diagnose (PGresult* ptPGRESULT)
{

  resetStatus();
  if ( !ptPGRESULT )
  {
    //
    //  Condition:           transaction rollback
    //  Class subcondition:  statement completion unknown
    //
    SQLCODE    = -1;
    SQLMESSAGE = PQerrorMessage (getCurrentConnectionData().ptPgconn);
    SQLSTATE   = "40003";
  }
  else
  {
    switch (PQresultStatus (ptPGRESULT))
    {
      case PGRES_COMMAND_OK:
      {
        //
        //  Condition:  successful completion
        // 
        SQLCODE  = 0;
        SQLSTATE = "00000";
        break;
      }
      case PGRES_TUPLES_OK:
      {
        if ( PQntuples (ptPGRESULT) )
        {
          //
          //  Condition:  successful completion
          // 
          SQLCODE  = 0;
          SQLSTATE = "00000";
        }
        else
        {
          //
          //  Condition:  no data
          // 
          SQLCODE  = 101;
          SQLSTATE = "02000";
        }
        break;
      }
      case PGRES_EMPTY_QUERY:
      {
        //
        //  Condition:  no data
        // 
        SQLCODE  = 101;
        SQLSTATE = "02000";
        break;
      }
      default:
      {
        //
        //  Condition:           transaction rollback
        //  Class subcondition:  statement completion unknown
        //
        SQLCODE    = -1;
        SQLMESSAGE = PQerrorMessage (getCurrentConnectionData().ptPgconn);
        SQLSTATE   = "40003";
      }
    }
  }

}  // diagnose()


void uesqlc::TPostgresql62xManager::
diagnoseConnection (postgresql_6_2_x::TConnectionData& rtCONNECTION_DATA)
{

  resetStatus();
  if ( !rtCONNECTION_DATA.ptPgconn )
  {
    //
    //  Condition:           connection exception
    //  Class subcondition:  connection failure
    //
    SQLCODE  = -1;
    SQLSTATE = "08006";
  }
  else
  {
    switch (PQstatus (rtCONNECTION_DATA.ptPgconn))
    {
      case CONNECTION_OK:
      {
        break;
      }
      case CONNECTION_BAD:
      default:
      {
        //
        //  Condition:           connection exception
        //  Class subcondition:  SQL-client unable to establish SQL-connection
        //
        SQLCODE    = -1;
        SQLMESSAGE = PQerrorMessage (rtCONNECTION_DATA.ptPgconn);
        SQLSTATE   = "08001";
// <DELETE>
//         PQfinish (rtCONNECTION_DATA.ptPgconn);
// </DELETE>
      }
    }
  }

}  // diagnoseConnection()


void uesqlc::TPostgresql62xManager::diagnoseSingleRow (PGresult* ptPGRESULT)
{

  diagnose (ptPGRESULT);
  if ( !SQLCODE )
  {
    if ( PQntuples (ptPGRESULT) != 1 )
    {
      // 
      //  Condition:  cardinality violation
      // 
      SQLCODE    = -1;
      SQLMESSAGE = "";
      SQLSTATE   = "21000";
    }
  }

}  // diagnoseSingleRow()


// <DELETE>
// void uesqlc::TPostgresql62xManager::
// disconnect (const mpcl::text::TString kyCONNECTION_NAME)
// {
// 
//   using postgresql_6_2_x::TConnectionData;
// 
//   PQfinish (tConnectionDataMap [kyCONNECTION_NAME].ptPgconn);
//   unbind (kyCONNECTION_NAME);
// # ifdef _UESQL_DEBUG__
//   mpcl::util::logging::tLog << mpcl::util::logging::header << "DISCONNECT '" << kyCONNECTION_NAME << "';\n";
// # endif  // not _UESQL_DEBUG__
// 
// }  // disconnect()
// </DELETE>


PGresult* uesqlc::TPostgresql62xManager::execute (const char* pkcSTATEMENT)
{

  mpcl::text::TString   yStatementInstance (instantiate (pkcSTATEMENT));

# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << mpcl::util::logging::header << yStatementInstance << std::endl;
# endif  // not _UESQL_DEBUG__
  return PQexec (getCurrentConnectionData().ptPgconn, yStatementInstance.c_str());

}  // execute()


PGresult* uesqlc::TPostgresql62xManager::execute (const mpcl::text::TString& rkySTATEMENT)
{

  mpcl::text::TString   yStatementInstance (instantiate (rkySTATEMENT));

# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << mpcl::util::logging::header << yStatementInstance << std::endl;
# endif  // not _UESQL_DEBUG__
  return PQexec (getCurrentConnectionData().ptPgconn, yStatementInstance.c_str());

}  // execute()


PGresult* uesqlc::TPostgresql62xManager::execute ( const char* pkcSTATEMENT       ,
                                                  const char* pkcCONNECTION_NAME )
{

  PGresult*   ptPgresult = NULL;

  if ( !tConnectionDataMap.isBound (pkcCONNECTION_NAME) )
  {
    //
    //  Condition:           connection exception
    //  Class subcondition:  connection does not exist
    //
    SQLCODE    = -1;
    SQLMESSAGE = "";
    SQLSTATE   = "08003";
  }
  else
  {
    mpcl::text::TString   yStatementInstance (instantiate (pkcSTATEMENT));

#   ifdef _UESQL_DEBUG__
    mpcl::util::logging::tLog << mpcl::util::logging::header << yStatementInstance << std::endl;
#   endif  // not _UESQL_DEBUG__
    ptPgresult = PQexec ( tConnectionDataMap [pkcCONNECTION_NAME].ptPgconn ,
                          yStatementInstance.c_str()                       );
  }
  return ptPgresult;

}  // execute()


void uesqlc::TPostgresql62xManager::initialize (void)
{

  connect ("DEFAULT");
  if ( !isSqlerrorCondition() )
  {
    setCurrentConnection ("DEFAULT");
  }

}  // initialize()


//
//  E X P O R T E D   V A R I A B L E S
//

uesqlc::TPostgresql62xManager   uesqlc::htPostgresql62xManager;
