#include <mpcl/util/logging/log.hh>
#include "postgresql-libpq-6.2.x.hh"

#line 1 "../test-001-sql_types.uesql.cc"
/*
*  Name:         test-001-sql_types.uesql.cc
*  Author:       Rafael Jesus Alcantara Perez
*  Contributor:  Francisco Vides Fernandez
*  Summary:      Test for UESQL
*  Date:         $Date: 2003/10/06 13:21:36 $
*  Revision:     $Revision: 1.6 $
*
*  Copyright (C) 1996-2003  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/test.h>
#include <mpcl/text/string.hh>
#include <mpcl/util/prefs/config_processor.hh>


//
//  L O C A L   V A R I A B L E S
//

static uesqlc::TSqlstateDescriptor   _tSqlstateDescriptor;
  

//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::text::TString _GetErrorMessage (void)
{

  using mpcl::text::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesqlc::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


static int _TestSqlTypes (void)
{

  TEST_INIT ("test for SQL types");

  using std::cerr;
  using std::endl;
  using std::size_t;
  using uesqlc::TDate;
  using uesqlc::TInteger;
  using uesqlc::TReal;
  using uesqlc::TString;
  using uesqlc::TTime;

  TDate      tDate;
  TInteger   tInteger;
  TReal      tReal;
  TTime      tTime;
  TString    yString;
  TString    yString4;
  TString    yString1;
  TString    yString2;
  TString    yString3;
  int        iRecordCount = 0;

  //
  //  Checks for quote escaping.
  //
  TEST_STRINGS ("'hello'",        TString ("hello").get().c_str());
  TEST_STRINGS ("'hello''bye'",   TString ("hello'bye").get().c_str());
  TEST_STRINGS ("'hello''''bye'", TString ("hello''bye").get().c_str());
  TEST_STRINGS ("'hello\\\\bye'", TString ("hello\\bye").get().c_str());

  //
  //  Other checks.
  //
                                     TEST_NUMBERS (1,     TInteger (true));
                                     TEST_NUMBERS (0,     TInteger (false));
  tInteger.set ("1");                TEST_NUMBERS (1,     tInteger);
  tInteger.set ("0");                TEST_NUMBERS (0,     tInteger);
  tInteger.set ("10000");            TEST_NUMBERS (10000, tInteger);
                                     TEST_NUMBERS (10001, TInteger (10001));
  tDate.set    ((const char*) NULL); TEST_NUMBERS (true,  tDate.isNull());
  tInteger.set ((const char*) NULL); TEST_NUMBERS (true,  tInteger.isNull());
  tReal.set    ((const char*) NULL); TEST_NUMBERS (true,  tReal.isNull());
  yString.set  ((const char*) NULL); TEST_NUMBERS (true,  yString.isNull());
  tDate.set    ("");                 TEST_NUMBERS (true,  tDate.isNull());
  tInteger.set ("");                 TEST_NUMBERS (true,  tInteger.isNull());
  tReal.set    ("");                 TEST_NUMBERS (true,  tReal.isNull());
  yString.set  ("");                 TEST_NUMBERS (false, yString.isNull());
                                     TEST_NUMBERS (true,  TDate().isNull());
                                     TEST_NUMBERS (true,  TInteger().isNull());
                                     TEST_NUMBERS (true,  TReal().isNull());
                                     TEST_NUMBERS (true,  TString().isNull());
  tInteger.getBuffer (12);
  tInteger.getBuffer (12);
  tInteger.getBuffer (12);
  tInteger.set ("75000");
  TEST_NUMBERS (75000, tInteger);
#line 122 "../test-001-sql_types.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 123 "../test-001-sql_types.uesql.cc"

#line 124 "../test-001-sql_types.uesql.cc"

#line 132 "../test-001-sql_types.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("CREATE TABLE TABLE_1 (CODE CHAR (3) NOT NULL, BIRTH_DATE DATE, BIRTH_TIME TIME, SALARY INT4, PERCENTAGE FLOAT8);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 133 "../test-001-sql_types.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO TABLE_1 VALUES ('000', '1999-12-31', '01:00', 12000, 25.75);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 134 "../test-001-sql_types.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO TABLE_1 VALUES ('001', '1970-01-01', '02:00', 75000, 30);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 135 "../test-001-sql_types.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO TABLE_1 VALUES ('002', '2000-01-01', '03:00', 11500, 99.99);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 140 "../test-001-sql_types.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnInputQueue (TDate (2000, 12, 31));
uesqlc::htPostgresql62xManager.pushOnInputQueue (TTime ("12:00:00"));
uesqlc::htPostgresql62xManager.pushOnInputQueue (TInteger (7500));
uesqlc::htPostgresql62xManager.pushOnInputQueue (TReal (33.33));

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO TABLE_1 VALUES ('003', $z, $z, $z, $z);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 145 "../test-001-sql_types.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnInputQueue (TDate (2001, 1, 1));
uesqlc::htPostgresql62xManager.pushOnInputQueue (TTime (23, 59, 59));
uesqlc::htPostgresql62xManager.pushOnInputQueue (TInteger (6500));
uesqlc::htPostgresql62xManager.pushOnInputQueue (TReal (0.0));

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO TABLE_1 VALUES ('004', $z, $z, $z, $z);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 149 "../test-001-sql_types.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&yString1);

uesqlc::htPostgresql62xManager.begin();
ptPgresult = uesqlc::htPostgresql62xManager.execute ("DECLARE AUX_CURSOR CURSOR FOR SELECT 'hello\rbye' FROM TABLE_1 WHERE ( CODE = '000' );");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( !uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  ptPgresult = uesqlc::htPostgresql62xManager.execute ("FETCH ALL IN AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
}
if ( uesqlc::htPostgresql62xManager.isNotFoundCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
  goto loop_end;
}
else if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << mpcl::util::logging::header << "SELECTED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ");\n";
#endif  // not _UESQL_DEBUG__
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
}
}
#line 150 "../test-001-sql_types.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&yString2);

uesqlc::htPostgresql62xManager.begin();
ptPgresult = uesqlc::htPostgresql62xManager.execute ("DECLARE AUX_CURSOR CURSOR FOR SELECT 'hello\"bye' FROM TABLE_1 WHERE ( CODE = '000' );");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( !uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  ptPgresult = uesqlc::htPostgresql62xManager.execute ("FETCH ALL IN AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
}
if ( uesqlc::htPostgresql62xManager.isNotFoundCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
  goto loop_end;
}
else if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << mpcl::util::logging::header << "SELECTED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ");\n";
#endif  // not _UESQL_DEBUG__
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
}
}
#line 151 "../test-001-sql_types.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&yString3);

uesqlc::htPostgresql62xManager.begin();
ptPgresult = uesqlc::htPostgresql62xManager.execute ("DECLARE AUX_CURSOR CURSOR FOR SELECT 'hello''bye' FROM TABLE_1 WHERE ( CODE = '000' );");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( !uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  ptPgresult = uesqlc::htPostgresql62xManager.execute ("FETCH ALL IN AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
}
if ( uesqlc::htPostgresql62xManager.isNotFoundCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
  goto loop_end;
}
else if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << mpcl::util::logging::header << "SELECTED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ");\n";
#endif  // not _UESQL_DEBUG__
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
}
}
#line 152 "../test-001-sql_types.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnInputQueue (TString ("\\"));
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&yString4);

uesqlc::htPostgresql62xManager.begin();
ptPgresult = uesqlc::htPostgresql62xManager.execute ("DECLARE AUX_CURSOR CURSOR FOR SELECT $z FROM TABLE_1 WHERE ( CODE = '000' );");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( !uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  ptPgresult = uesqlc::htPostgresql62xManager.execute ("FETCH ALL IN AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
}
if ( uesqlc::htPostgresql62xManager.isNotFoundCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
  goto loop_end;
}
else if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << mpcl::util::logging::header << "SELECTED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ");\n";
#endif  // not _UESQL_DEBUG__
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
}
}
#line 158 "../test-001-sql_types.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

uesqlc::htPostgresql62xManager.begin();
ptPgresult = uesqlc::htPostgresql62xManager.execute ("DECLARE MYCURSOR CURSOR FOR SELECT CODE, BIRTH_DATE, BIRTH_TIME, SALARY, PERCENTAGE FROM TABLE_1;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 159 "../test-001-sql_types.uesql.cc"
uesqlc::htPostgresql62xManager.clearQueues();


}  // not PostgreSQL LIBPQ 6.2.x
#line 160 "../test-001-sql_types.uesql.cc"

  TEST_STRINGS ("'hello\rbye'", yString1.get().c_str());
  TEST_STRINGS ("'hello\"bye'", yString2.get().c_str());
  TEST_STRINGS ("'hello''bye'", yString3.get().c_str());
  TEST_STRINGS ("'\\\\'",       yString4.get().c_str());
  TEST_NUMBERS ("hello\rbye",   yString1);
  TEST_NUMBERS ("hello\"bye",   yString2);
  TEST_NUMBERS ("hello'bye",    yString3);
  TEST_NUMBERS ("\\",           yString4);

  //
  //  Writes the example table.
  //
  if ( hptTestTargetFile )
  {
    fprintf (hptTestTargetFile, "%s\n", "CODE  BIRTH_DATE   BIRTH_DATE SALARY PERCENTAGE");
    fprintf (hptTestTargetFile, "%s\n", "===== ============ ========== ====== ==========");
  }
  while ( true )
  {
#line 181 "../test-001-sql_types.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 182 "../test-001-sql_types.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&yString);
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&tDate);
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&tTime);
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&tInteger);
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&tReal);

ptPgresult = uesqlc::htPostgresql62xManager.execute ("FETCH IN MYCURSOR;");
uesqlc::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isNotFoundCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  goto loop_end;
}
else if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << mpcl::util::logging::header << "FETCHED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ", " << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ", " << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ", " << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ", " << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ");\n";
#endif  // not _UESQL_DEBUG__
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 183 "../test-001-sql_types.uesql.cc"

    if ( hptTestTargetFile )
    {
      fprintf ( hptTestTargetFile              ,
                "%5s %12s %10s %6ld %10.03f\n" ,
                yString.get().c_str()          ,
                tDate.get().c_str()            ,
                tTime.get().c_str()            ,
                (long int) tInteger            ,
                (double) tReal                 );
    }
    ++iRecordCount;
  }

loop_end:
#line 199 "../test-001-sql_types.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 200 "../test-001-sql_types.uesql.cc"
uesqlc::htPostgresql62xManager.clearQueues();

uesqlc::htPostgresql62xManager.executeClean ("CLOSE MYCURSOR;");
uesqlc::htPostgresql62xManager.end();

#line 201 "../test-001-sql_types.uesql.cc"
{
PGresult*   ptPgresult;

//
//  'CASCADE' has been omitted.
//
uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("DROP TABLE TABLE_1;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 202 "../test-001-sql_types.uesql.cc"

  if ( iRecordCount == 5 )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesqlc::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesqlc::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);  

end:  
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // _TestSqlTypes()


/**
*  Entry  point.  It uses the  user login 'test/test'  and the schema  location
*  'localhost/test'.
*
*  Parameters:
*
*    {-o, --dbms-server-name}=DBMS-SERVER-NAME
*/
int main (int argc, const char* argv[])
{

  using mpcl::text::Format;
  using mpcl::util::prefs::TConfigProcessor;
  using std::cerr;
  using std::endl;
  using uesqlc::ComposeSchemaLocation;
  using uesqlc::ComposeUserLogin;
  using uesqlc::TIdentifier;
  using uesqlc::TInteger;
  using uesqlc::TSearchCondition;
  using uesqlc::TString;
  
  TEST_INIT ("test-1");
  TEST_START_WATCH;

  TInteger           tHours;
  TInteger           tCount;
  TSearchCondition   tCondition;
  TString            yVar0;
  TString            yVar1;
  TString            yVar2;
  TIdentifier        yCursorName;
  TString            yDbmsUserLogin;
  TString            yDbmsSchemaLocation;
  TString            yDbmsServerName;
  TConfigProcessor   tConfigProcessor (argc, argv);
  TString            yCodeValue ("002");
  const char*        pcDbmsServerName = getenv ("DBMS_SERVER_NAME");

  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }
    
    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name", "o", pcDbmsServerName);
    tConfigProcessor.processOptions();
    yDbmsServerName     = tConfigProcessor ["dbms-server-name"].getValue();
    yDbmsUserLogin      = ComposeUserLogin      ("test",          "test");
    yDbmsSchemaLocation = ComposeSchemaLocation (yDbmsServerName, "test");
  }
  catch (const mpcl::TException& rktEXCEPTION)
  {
    cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }

  TEST_WRITE_INFO ("DBMS-SCHEMA-LOCATION = %s", yDbmsSchemaLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN      = %s", yDbmsUserLogin.c_str());
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.also (cerr);
  }
  if ( !yDbmsServerName.empty() )
  {
#line 299 "../test-001-sql_types.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 303 "../test-001-sql_types.uesql.cc"

#line 308 "../test-001-sql_types.uesql.cc"
{
mpcl::text::TString   hySqlServerDescriptor;
mpcl::text::TString   hySqlServerName;
mpcl::text::TString   hyDatabaseName;
mpcl::text::TString   hyConnectionName;
mpcl::text::TString   hyUserLogin;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnInputQueue (yDbmsSchemaLocation);
uesqlc::htPostgresql62xManager.pushOnInputQueue (yDbmsUserLogin);

{
  mpcl::text::TString::size_type   hzBarIndex;

  hySqlServerDescriptor = uesqlc::htPostgresql62xManager.instantiate ("$z");
  hyConnectionName      = "DEFAULT";
  hyUserLogin           = uesqlc::htPostgresql62xManager.instantiate ("$z");
  hySqlServerDescriptor.suppress ('\'');
  hzBarIndex = hySqlServerDescriptor.find ('/');
  if ( hzBarIndex == mpcl::text::TString::npos )
  {
    hySqlServerName = hySqlServerDescriptor;
  }
  else
  {
    hySqlServerName = hySqlServerDescriptor.substr (0, hzBarIndex);
    hyDatabaseName  = hySqlServerDescriptor.substr (hzBarIndex + 1);
  }
  if ( uesqlc::htPostgresql62xManager.isConnected (hyConnectionName) )
  {
    uesqlc::htPostgresql62xManager.unbindConnection (hyConnectionName);
  }
}
if ( uesqlc::htPostgresql62xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "08002";
  goto error;
}
else
{
  if ( hyDatabaseName.empty() )
  {
    //
    //  If there is no PostgreSQL database then connect to 'template1'
    //  (that should always exists).
    //
    hyDatabaseName = "template1";
  }
  uesqlc::htPostgresql62xManager.connect ( hyConnectionName ,
                                     hySqlServerName  ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
  {
    goto error;
  }
  else
  {
    uesqlc::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
  }
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 309 "../test-001-sql_types.uesql.cc"

    TEST_FUNCTION (_TestSqlTypes());
#line 312 "../test-001-sql_types.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 316 "../test-001-sql_types.uesql.cc"
{
uesqlc::htPostgresql62xManager.clearQueues();

if ( uesqlc::htPostgresql62xManager.existsDefaultConnection() )
{
  uesqlc::htPostgresql62xManager.resetStatus();
  uesqlc::htPostgresql62xManager.unbindCurrentConnection();
}
else
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection does not exist
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "08003";
  goto error;
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 317 "../test-001-sql_types.uesql.cc"

    goto end;

  error:
    cerr << "SQLSTATE="    << uesqlc::SQLSTATE    << endl;
    cerr << "SQLCODE="     << uesqlc::SQLCODE     << endl;
    cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
    TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);
    goto end;
  }

end:
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.erase (cerr);
  }
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH ("UESQL benchmark");
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
