#include <mpcl/util/logging/log.hh>
#include "postgresql-libpq-6.2.x.hh"

#line 1 "../test-002-schema_creation.uesql.cc"
/*
*  Name:      test-002-schema_creation.uesql.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Test for UESQL
*  Date:      $Date: 2003/10/06 13:21:36 $
*  Revision:  $Revision: 1.2 $
*
*  Copyright (C) 2000-2003  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/system/all.hh>
#include <mpcl/test.h>
#include <mpcl/text/string.hh>
#include <mpcl/util/logging/log.hh>
#include <mpcl/util/prefs/config_processor.hh>


//
//  L O C A L   V A R I A B L E S
//

static uesqlc::TSqlstateDescriptor   _tSqlstateDescriptor;
  

//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::text::TString _GetErrorMessage (void)
{

  using mpcl::text::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesqlc::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


/**
*  Entry  point.  It uses the  user login 'test/test'  and the schema  location
*  'localhost/test'.
*
*  Parameters:
*
*    {-o, --dbms-server-name}=DBMS-SERVER-NAME
*/
int main (int argc, const char* argv[])
{

  using mpcl::text::Format;
  using mpcl::util::prefs::TConfigProcessor;
  using std::cerr;
  using std::endl;
  using uesqlc::ComposeSchemaLocation;
  using uesqlc::ComposeUserLogin;
  using uesqlc::TString;
  
  TEST_INIT ("Test for SCHEMA related statements");
  TEST_START_WATCH;

  TString            yVar0;
  TString            yVar1;
  TString            yVar2;
  TString            yCursorName;
  TString            yDbmsUserLogin;
  TString            yDbmsSchemaLocation;
  TString            yDbmsServerName;
  TConfigProcessor   tConfigProcessor (argc, argv);
  const char*        pcDbmsServerName = getenv ("DBMS_SERVER_NAME");

  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }
    
    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name", "o", pcDbmsServerName);
    tConfigProcessor.processOptions();
    yDbmsServerName     = tConfigProcessor ["dbms-server-name"].getValue();
    yDbmsUserLogin      = ComposeUserLogin      ("test",          "test");
    yDbmsSchemaLocation = ComposeSchemaLocation (yDbmsServerName, "test");
  }
  catch (const mpcl::TException& rktEXCEPTION)
  {
    cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }

  TEST_WRITE_INFO ("DBMS-SCHEMA-LOCATION = %s", yDbmsSchemaLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN      = %s", yDbmsUserLogin.c_str());
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.also (cerr);
  }
  if ( !yDbmsServerName.empty() )
  {
#line 129 "../test-002-schema_creation.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 133 "../test-002-schema_creation.uesql.cc"

#line 138 "../test-002-schema_creation.uesql.cc"
{
mpcl::text::TString   hySqlServerDescriptor;
mpcl::text::TString   hySqlServerName;
mpcl::text::TString   hyDatabaseName;
mpcl::text::TString   hyConnectionName;
mpcl::text::TString   hyUserLogin;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnInputQueue (yDbmsServerName);
uesqlc::htPostgresql62xManager.pushOnInputQueue (yDbmsUserLogin);

{
  mpcl::text::TString::size_type   hzBarIndex;

  hySqlServerDescriptor = uesqlc::htPostgresql62xManager.instantiate ("$z");
  hyConnectionName      = "DEFAULT";
  hyUserLogin           = uesqlc::htPostgresql62xManager.instantiate ("$z");
  hySqlServerDescriptor.suppress ('\'');
  hzBarIndex = hySqlServerDescriptor.find ('/');
  if ( hzBarIndex == mpcl::text::TString::npos )
  {
    hySqlServerName = hySqlServerDescriptor;
  }
  else
  {
    hySqlServerName = hySqlServerDescriptor.substr (0, hzBarIndex);
    hyDatabaseName  = hySqlServerDescriptor.substr (hzBarIndex + 1);
  }
  if ( uesqlc::htPostgresql62xManager.isConnected (hyConnectionName) )
  {
    uesqlc::htPostgresql62xManager.unbindConnection (hyConnectionName);
  }
}
if ( uesqlc::htPostgresql62xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "08002";
  goto error;
}
else
{
  if ( hyDatabaseName.empty() )
  {
    //
    //  If there is no PostgreSQL database then connect to 'template1'
    //  (that should always exists).
    //
    hyDatabaseName = "template1";
  }
  uesqlc::htPostgresql62xManager.connect ( hyConnectionName ,
                                     hySqlServerName  ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
  {
    goto error;
  }
  else
  {
    uesqlc::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
  }
}
}
#line 154 "../test-002-schema_creation.uesql.cc"
{
PGresult*             ptPgresult;
mpcl::text::TString   hyConnectionName;
mpcl::text::TString   hyLastConnectionName = uesqlc::htPostgresql62xManager.currentConnectionName();

uesqlc::htPostgresql62xManager.clearQueues();

hyConnectionName = uesqlc::htPostgresql62xManager.instantiate ("TEST_5_SCHEMA");
hyConnectionName.lowercase();
ptPgresult = uesqlc::htPostgresql62xManager.execute (mpcl::text::Format ("CREATE DATABASE %s;", hyConnectionName.c_str()));
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
uesqlc::htPostgresql62xManager.connect (hyConnectionName, hyConnectionName);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
else
{
  uesqlc::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
  
{
PGresult*   ptPgresult;

ptPgresult = uesqlc::htPostgresql62xManager.execute ("CREATE TABLE TABLE_1 (CODE CHAR (3) NOT NULL, WORKER_COUNT INT4);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
{
PGresult*   ptPgresult;

ptPgresult = uesqlc::htPostgresql62xManager.execute ("CREATE TABLE TABLE_2 (WORK_CODE CHAR (3) NOT NULL, NAME CHAR (40) NOT NULL, HOURS DECIMAL (5));");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
  uesqlc::htPostgresql62xManager.unbindCurrentConnection();
  uesqlc::htPostgresql62xManager.setCurrentConnection (hyLastConnectionName);
}
}
#line 161 "../test-002-schema_creation.uesql.cc"
{
mpcl::text::TString   hySqlServerDescriptor;
mpcl::text::TString   hySqlServerName;
mpcl::text::TString   hyDatabaseName;
mpcl::text::TString   hyConnectionName;
mpcl::text::TString   hyUserLogin;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnInputQueue (ComposeSchemaLocation (yDbmsServerName, "test_5_schema"));
uesqlc::htPostgresql62xManager.pushOnInputQueue (yDbmsUserLogin);

{
  mpcl::text::TString::size_type   hzBarIndex;

  hySqlServerDescriptor = uesqlc::htPostgresql62xManager.instantiate ("$z");
  hyConnectionName      = uesqlc::htPostgresql62xManager.instantiate ("_'myconnection'");
  hyUserLogin           = uesqlc::htPostgresql62xManager.instantiate ("$z");
  hySqlServerDescriptor.suppress ('\'');
  hyConnectionName.suppress ('\'');
  hzBarIndex = hySqlServerDescriptor.find ('/');
  if ( hzBarIndex == mpcl::text::TString::npos )
  {
    hySqlServerName = hySqlServerDescriptor;
  }
  else
  {
    hySqlServerName = hySqlServerDescriptor.substr (0, hzBarIndex);
    hyDatabaseName  = hySqlServerDescriptor.substr (hzBarIndex + 1);
  }
}
if ( uesqlc::htPostgresql62xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "08002";
  goto error;
}
else
{
  if ( hyDatabaseName.empty() )
  {
    //
    //  If there is no PostgreSQL database then connect to 'template1'
    //  (that should always exists).
    //
    hyDatabaseName = "template1";
  }
  uesqlc::htPostgresql62xManager.connect ( hyConnectionName ,
                                     hySqlServerName  ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
  {
    goto error;
  }
  else
  {
    uesqlc::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
  }
}
}
#line 166 "../test-002-schema_creation.uesql.cc"
{
PGresult*   ptPgresult;

//
//  'CASCADE' has been omitted.
//
uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("DROP TABLE TABLE_1;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 167 "../test-002-schema_creation.uesql.cc"
{
uesqlc::htPostgresql62xManager.clearQueues();

if ( uesqlc::htPostgresql62xManager.existsDefaultConnection() )
{
  uesqlc::htPostgresql62xManager.resetStatus();
  uesqlc::htPostgresql62xManager.unbindCurrentConnection();
}
else
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection does not exist
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "08003";
  goto error;
}
}
#line 168 "../test-002-schema_creation.uesql.cc"
{
mpcl::text::TString   hyConnectionName;

uesqlc::htPostgresql62xManager.clearQueues();

hyConnectionName = uesqlc::htPostgresql62xManager.instantiate ("DEFAULT");
hyConnectionName.suppress ('\'');
uesqlc::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 169 "../test-002-schema_creation.uesql.cc"


    //
    //  Sleeps for 5  seconds to  let slow  systems (like POSTGRESQL on a i586)
    //  remove connections properly.
    //
    mpcl::system::tSystem.sleep (5);
#line 177 "../test-002-schema_creation.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 181 "../test-002-schema_creation.uesql.cc"
{
PGresult*             ptPgresult;
mpcl::text::TString   hySchemaName;

//
//  'CASCADE' has been omitted.
//
uesqlc::htPostgresql62xManager.clearQueues();

hySchemaName = "TEST_5_SCHEMA";
hySchemaName.lowercase();
ptPgresult = uesqlc::htPostgresql62xManager.execute (mpcl::text::Format ("DROP DATABASE %s;", hySchemaName.c_str()));
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 182 "../test-002-schema_creation.uesql.cc"

  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesqlc::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesqlc::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);

end:
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.erase (cerr);
  }
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
