/*
*  Name:      TString.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   SQL type string.
*  Date:      $Date: 2003/07/29 16:47:50 $
*  Revision:  $Revision: 1.5 $
*
*  Copyright (C) 2002-2003  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package for UESQL.
package org.uesqlc;


///  SQL type for strings.
public class TString implements IType
{

  protected String   yValue = null;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @post ( isNull() )
  */
  public TString() {}

  /**
  *  Builds a new instance.
  *  @param kySTRING Source string.
  *  @post ( isNull() == ( null == kySTRING ) )
  */
  public TString (final String kySTRING)
  {
    if ( kySTRING != null )
    {
      yValue = new String (kySTRING);
    }
  }

  public IType set (final Object ktVALUE)
  {
    if ( ktVALUE == null )
    {
      yValue = null;
    }
    else
    {
      yValue = new String ((String) ktVALUE);
    }
    return this;
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns a new instance built from \a kySOURCE.
  *  @param kySOURCE Source string.
  *  @return A new instance.
  */
  public static TString _valueOf (final String kySOURCE)
  {
    return new TString (kySOURCE);
  }

  /**
  *  Composes a  string as the  concatenation of \a kyHOST_NAME, a solidus, and
  *  then \a kySCHEMA_NAME.
  *  @param kyHOST_NAME   Host name.
  *  @param kySCHEMA_NAME Schema name.
  */
  public static TString _composeSchemaLocation ( final String kyHOST_NAME   ,
                                                 final String kySCHEMA_NAME )
  {
    return new TString (kyHOST_NAME + '/' + kySCHEMA_NAME);
  }

  /**
  *  Composes a  string as the  concatenation of \a kyUSER_NAME, a solidus, and
  *  then \a kyUSER_PASSWORD.
  *  @param kyUSER_NAME     User name.
  *  @param kyUSER_PASSWORD User password.
  */
  public static TString _composeUserLogin ( final String kyUSER_NAME     ,
                                            final String kyUSER_PASSWORD )
  {
    return new TString (kyUSER_NAME + '/' + kyUSER_PASSWORD);
  }

  public String get()
  {

    String   yResult;

    if ( isNull() )
    {
      yResult = "null";
    }
    else
    {
      //
      //  The number of  backslashes is duplicated  due to the way the 'String'
      //  interprets regular expressions.
      //
      yResult = "'" + yValue.replaceAll ("'", "''").replaceAll ("\\\\", "\\\\\\\\") + "'";
    }
    return yResult;

  }  // get()

  public boolean isNull()
  {
    return ( yValue == null );
  }

  public String toString()
  {
    return ( yValue == null ) ? null : yValue.toString();
  }

}  // class TString
