/*
*  Name:        uesql_parser.yy
*  Author:      Rafael Jesus Alcantara Perez
*  Contributor: Francisco Vides Fernndez
*  Summary:     UESQL parser
*  Date:        $Date: 2003/07/11 13:35:54 $
*  Revision:    $Revision: 1.20 $
*
*  Comments:  Section 20. Direct invocation of SQL.
*             Has been removed.
*
*             Subsection 19.1:  <embedded SQL host program>.
*             Has been modified to support only  the C language and derivatives
*             languages. It has also been removed <temporary table declaration>
*             due to it is only reduced in <module> and related.
*
*             Subsection 19.2:  <embedded exception declaration>.
*             Has been modified to support only the C language and derivatives.
*                
*             Subsection 19.4:  <embedded SQL C program>.            
*             Has been partially removed. Terminal <embedded variable name> has
*             been extended to represent simple (without nesting, references or
*             so)  n-adic  function  calls.  The  function  names  can  contain
*             multiple  name-spaces.  It has been also extended  to  support  C
*             array variables.
*
*             Section 17:  <SQL dynamic statement>
*             All dynamic sql support has been disabled, since it caused many
*             potential problems, and it's useless right now. Maybe enabled again
*             in future versions of UESQLI, the universal embedded sql interpreter.
*
*             Subsection 17.20:  <preparable dynamic update statement: positioned>.
*             Has  been removed,  due to that it  is only useful as information
*             about which statements are preparable.
*
*             Subsection 17.19:  <preparable dynamic delete statement: positioned>.
*             Has  been removed,  due to that it  is only useful as information
*             about which statements are preparable.
*
*             Subsection 17.6:  <prepare statement>.
*             Has been partially removed.
*
*             Subsection 15.1:  <connect statement>.
*             In  this  section,  the  non-terminal <SQL-server name> has  been
*             changed   to   <schema location>   (see  README.uesql  for   more
*             information).
*
*             Subsection 13.11: <temporary table declaration>.
*             Has been removed, because it is used in <module>.
*
*             Subsection 12.5:  <SQL procedure statement>.
*             In this section has been  removed  all rules related to character
*             set,  collation and translation  definition, an the corresponding
*             drop statement rules.
*
*             Subsection 12.3:  <procedure>.
*             Has been removed.
*
*             Subsection 12.2:  <module name clause>.
*             Has been removed because <module>s are not embeddable.
*
*             Subsection 12.1:  <module>.
*             Has been removed because <module>s are not embeddable.
*
*             Subsection 11.36:  <grant statement>.
*             All rules related with character sets, have been removed.
*
*             Subsection 11.33:  <drop translation statement>.
*             Has been removed.
*
*             Subsection 11.32:  <translation definition>.
*             Has been removed.
*
*             Subsection 11.31:  <drop collation statement>.
*             Has been removed.
*
*             Subsection 11.30:  <collation definition>.
*             Has been removed.
*
*             Subsection 11.29:  <drop character set statement>.
*             Has been removed.
*
*             Subsection 11.28:  <character set definition>.
*             Has been removed.
*
*             Subsection 11.1:  <schema definition>.
*             In this section have been  removed all rules related to character
*             set,  collation and translation definition, and the corresponding
*             drop statement rules.
*
*             Subsection 10.4:  <character set specification>.
*             Has been removed.
*
*             Subsection 10.2:  <language clause>.
*             Has been removed.
*
*             Subsection 8.5:  <like predicate>.
*             Has been changed.
*
*             Subsection 7.9:  <query specification>.
*             Non-terminal <select sublist>  has been modified to use terminals
*             *_LEVEL_ASTERISK.
*
*             Subsection 6.4:  <column reference>.
*             Non-terminal   <column reference>   has   been  modified  to  use
*             terminals *_LEVEL_IDENTIFIER.  Non-terminal  <qualifier> has been
*             removed due to the inclussion of *_LEVEL_IDENTIFIERs.
*
*             Subsection 6.2:  <value specification> and <target specification>.
*             Due  to syntax rule 2,  <parameter name> shall only be  contained
*             in a <module>,  but  this  is  an  embedded  SQL parser, so rules
*             <simple value specification>   and  <simple target specification>
*             has been modified to remove <parameter name>.
*             By the way, <parameter specification> identifies a parameter or a
*             parameter  and an  indicator  parameter in a <module>,  so  rules
*             <general value specification>,  <target specification>  have been
*             modified to remove <parameter specification>.
*             It also has been removed <indicator variable>  (used in <variable
*             specification>) due to it is no need of truncate checking.
*
*             Subsection 6.1:  <data type>.
*             In this section have been removed  all rules related to character
*             set specification rules.
*
*             Subsection 5.4:  Names and identifiers.
*             Non-terminal  <SQL language identifier>   has  been  modified  to
*             derive from <regular identifier>.
*             All rules related to character  set specification rules have been
*             also  removed.   Non-terminals  <qualified local table name>  and
*             <local table name>  have  been  removed  because it is only  used
*             with modules.
*             Non-terminals  <schema name>  and  <qualified name>   have   been
*             modified to use terminals *_LEVEL_IDENTIFIER.
*             Non-terminal  <user name>  has  been  renamed to <user login>  to
*             enforce  UESQL semantics (see README.uesql for more information).
*
*             Subsection 5.3:  <literal>.
*             Has been partially moved to scanner.
*             All rules related to character set specification rules, have been
*             removed.
*
*             Subsection 5.2:  <token> and <separator>.
*             Has been totally moved to scanner.
*
*             Subsection 17.A:  <declare search condition>.
*             This subsection has been added as an extension to the standard to
*             assign conditions dynamically to an embedded variable.
*
*             Subsection 8.12:  <boolean primary>.
*             This    rule   has   been    extended   with   the   non-terminal
*             <search condition value specification> to support embedded search
*             condition variables.
*
*             Subsection 6.2:  <search condition value specification>.
*             This subsection has been  extended with  that rule to support the
*             embedded variables in the rule <search condition>.
*
*  Copyright (C) 1996-2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/


%{

#include <cstdlib>
#include "application.hh"
#include "defs.hh"
#include "stack_value.hh"


//
//  E X P O R T E D   V A R I A B L E S
//

extern TApplication*   ptApplication;

  
//
//  E X P O R T E D   F U N C T I O N S
//

///  YACC scanner function.
int yylex (void);

///  YACC parser function.
int yyparse (void);

///  YACC error function.
void yyerror (const char*);

%}


/*
*  Y A C C   D E C L A R A T I O N S
*/

%token EMBEDDED_SQL_BEGIN
%token EMBEDDED_SQL_END
%token LEXICAL_ERROR


/*
*  Subsection 19.4:  <embedded SQL C program>.
*/

%token EMBEDDED_N_ADIC_FUNCTION_CALL
%token EMBEDDED_ARRAY
%token EMBEDDED_VARIABLE_NAME


/*
*  Subsection 5.3:  <literal>. 
*/
 
%token CHARACTER_STRING_LITERAL
%token NATIONAL_CHARACTER_STRING_LITERAL
%token BIT_STRING_LITERAL
%token HEX_STRING_LITERAL
%token UNSIGNED_INTEGER
%token REAL_EXACT_NUMERIC_LITERAL
%token APPROXIMATE_NUMERIC_LITERAL
%token DATE_STRING
%token TIME_STRING
%token TIMESTAMP_STRING
%token INTERVAL_STRING


/*
*  Subsection 5.2:  <token> and <separator>. 
*/
 
%token FOURTH_LEVEL_IDENTIFIER
%token THIRD_LEVEL_IDENTIFIER
%token SECOND_LEVEL_IDENTIFIER
%token FOURTH_LEVEL_ASTERISK
%token THIRD_LEVEL_ASTERISK
%token SECOND_LEVEL_ASTERISK

%token IDENTIFIER_BODY
%token DELIMITED_IDENTIFIER
%token MULTI_HOST_IDENTIFIER
%token NOT_EQUALS_OPERATOR
%token GREATER_THAN_OR_EQUALS_OPERATOR
%token LESS_THAN_OR_EQUALS_OPERATOR
%token CONCATENATION_OPERATOR

/*
*  Non-Terminal:  reserved_word
*/
%token RW_ABSOLUTE
%token RW_ACTION
%token RW_ADD
%token RW_ALLOCATE
%token RW_ALL
%token RW_ALTER
%token RW_AND
%token RW_ANY
%token RW_ASC
%token RW_ASSERTION
%token RW_AS
%token RW_AT
%token RW_AUTHORIZATION
%token RW_AVG
%token RW_BETWEEN
%token RW_BIT_LENGTH
%token RW_BIT
%token RW_BOTH
%token RW_BY
%token RW_CASCADED
%token RW_CASCADE
%token RW_CASE
%token RW_CAST
%token RW_CATALOG
%token RW_CHARACTER_LENGTH
%token RW_CHARACTER
%token RW_CHAR_LENGTH
%token RW_CHAR
%token RW_CHECK
%token RW_CLOSE
%token RW_COALESCE
%token RW_COLLATE
%token RW_COLLATION
%token RW_COLUMN
%token RW_COMMIT
%token RW_CONDITION
%token RW_CONNECTION
%token RW_CONNECT
%token RW_CONSTRAINTS
%token RW_CONSTRAINT
%token RW_CONTINUE
%token RW_CONVERT
%token RW_CORRESPONDING
%token RW_COUNT
%token RW_CREATE
%token RW_CROSS
%token RW_CURRENT_DATE
%token RW_CURRENT_TIMESTAMP
%token RW_CURRENT_TIME
%token RW_CURRENT_USER
%token RW_CURRENT
%token RW_CURSOR
%token RW_DATE
%token RW_DAY
%token RW_DEALLOCATE
%token RW_DECIMAL
%token RW_DECLARE
%token RW_DEC
%token RW_DEFAULT
%token RW_DEFERRABLE
%token RW_DEFERRED
%token RW_DELETE
%token RW_DESCRIBE
%token RW_DESCRIPTOR
%token RW_DESC
%token RW_DIAGNOSTICS
%token RW_DISCONNECT
%token RW_DISTINCT
%token RW_DOMAIN
%token RW_DOUBLE
%token RW_DROP
%token RW_ELSE
%token RW_END
%token RW_ESCAPE
%token RW_EXCEPTION
%token RW_EXCEPT
%token RW_EXECUTE
%token RW_EXISTS
%token RW_EXTRACT
%token RW_FALSE
%token RW_FETCH
%token RW_FIRST
%token RW_FLOAT
%token RW_FOREIGN
%token RW_FOR
%token RW_FOUND
%token RW_FROM
%token RW_FULL
%token RW_GET
%token RW_GLOBAL
%token RW_GOTO
%token RW_GO
%token RW_GRANT
%token RW_GROUP
%token RW_HAVING
%token RW_HOUR
%token RW_IMMEDIATE
%token RW_INDICATOR
%token RW_INITIALLY
%token RW_INNER
%token RW_INPUT
%token RW_INSENSITIVE
%token RW_INSERT
%token RW_INTEGER
%token RW_INTERSECT
%token RW_INTERVAL
%token RW_INTO
%token RW_INT
%token RW_IN
%token RW_ISOLATION
%token RW_IS
%token RW_JOIN
%token RW_KEY
%token RW_LAST
%token RW_LEADING
%token RW_LEFT
%token RW_LEVEL
%token RW_LIKE
%token RW_LOCAL
%token RW_LOWER
%token RW_MATCH
%token RW_MAX
%token RW_MIN
%token RW_MINUTE
%token RW_MONTH
%token RW_NAMES
%token RW_NATIONAL
%token RW_NATURAL
%token RW_NCHAR
%token RW_NEXT
%token RW_NOT_LIKE
%token RW_NOT
%token RW_NO
%token RW_NULLIF
%token RW_NULL
%token RW_NUMERIC
%token RW_OCTET_LENGTH
%token RW_OF
%token RW_ONLY
%token RW_ON
%token RW_OPEN
%token RW_OPTION
%token RW_ORDER
%token RW_OR
%token RW_OUTER
%token RW_OUTPUT
%token RW_OVERLAPS
%token RW_PARTIAL
%token RW_POSITION
%token RW_PRECISION
%token RW_PREPARE
%token RW_PRESERVE
%token RW_PRIMARY
%token RW_PRIOR
%token RW_PRIVILEGES
%token RW_PUBLIC
%token RW_READ
%token RW_REAL
%token RW_REFERENCES
%token RW_RELATIVE
%token RW_RESTRICT
%token RW_REVOKE
%token RW_RIGHT
%token RW_ROLLBACK
%token RW_ROWS
%token RW_SCHEMA
%token RW_SCROLL
%token RW_SECOND
%token RW_SELECT
%token RW_SESSION_USER
%token RW_SESSION
%token RW_SET
%token RW_SIZE
%token RW_SMALLINT
%token RW_SOME
%token RW_SQLERROR
%token RW_SQL
%token RW_SUBSTRING
%token RW_SUM
%token RW_SYSTEM_USER
%token RW_TABLE
%token RW_TEMPORARY
%token RW_THEN
%token RW_THROW
%token RW_TIMESTAMP
%token RW_TIMEZONE_HOUR
%token RW_TIMEZONE_MINUTE
%token RW_TIME
%token RW_TO
%token RW_TRAILING
%token RW_TRANSACTION
%token RW_TRANSLATE
%token RW_TRANSLATION
%token RW_TRIM
%token RW_TRUE
%token RW_UNION
%token RW_UNIQUE
%token RW_UNKNOWN
%token RW_UPDATE
%token RW_UPPER
%token RW_USAGE
%token RW_USER
%token RW_USING
%token RW_VALUES
%token RW_VALUE
%token RW_VARCHAR
%token RW_VARYING
%token RW_VIEW
%token RW_WHENEVER
%token RW_WHEN
%token RW_WHERE
%token RW_WITH
%token RW_WORK
%token RW_WRITE
%token RW_YEAR
%token RW_ZONE

/*
*  Non-Terminal:  non_reserved_word
*/
%token NRW_CATALOG_NAME
%token NRW_CHARACTER_SET_CATALOG
%token NRW_CHARACTER_SET_NAME
%token NRW_CHARACTER_SET_SCHEMA
%token NRW_CLASS_ORIGIN
%token NRW_COLLATION_CATALOG
%token NRW_COLLATION_NAME
%token NRW_COLLATION_SCHEMA
%token NRW_COLUMN_NAME
%token NRW_COMMAND_FUNCTION
%token NRW_COMMITTED
%token NRW_CONDITION_NUMBER
%token NRW_CONNECTION_NAME
%token NRW_CONSTRAINT_CATALOG
%token NRW_CONSTRAINT_NAME
%token NRW_CONSTRAINT_SCHEMA
%token NRW_CURSOR_NAME
%token NRW_DATA
%token NRW_DATETIME_INTERVAL_CODE
%token NRW_DATETIME_INTERVAL_PRECISION
%token NRW_DYNAMIC_FUNCTION
%token NRW_LENGTH
%token NRW_MESSAGE_LENGTH
%token NRW_MESSAGE_OCTET_LENGTH
%token NRW_MESSAGE_TEXT
%token NRW_MORE
%token NRW_NAME
%token NRW_NULLABLE
%token NRW_NUMBER
%token NRW_REPEATABLE
%token NRW_RETURNED_LENGTH
%token NRW_RETURNED_OCTET_LENGTH
%token NRW_RETURNED_SQLSTATE
%token NRW_ROW_COUNT
%token NRW_SCALE
%token NRW_SCHEMA_NAME
%token NRW_SERIALIZABLE
%token NRW_SERVER_NAME
%token NRW_SUBCLASS_ORIGIN
%token NRW_TABLE_NAME
%token NRW_TYPE
%token NRW_UNCOMMITTED
%token NRW_UNNAMED


/*
*  Subsection 5.1:  <SQL terminal character>. 
*/

%token LEFT_PAREN
%token RIGHT_PAREN
%token ASTERISK
%token PLUS_SIGN
%token COMMA
%token MINUS_SIGN
%token SOLIDUS
%token SEMICOLON
%token LESS_THAN_OPERATOR 
%token EQUALS_OPERATOR 
%token GREATER_THAN_OPERATOR
%token QUESTION_MARK

/*
*  Precedences.
*/

%left     RW_OR
%left     RW_AND
%right    RW_NOT
%nonassoc EQUALS_OPERATOR NOT_EQUALS_OPERATOR GREATER_THAN_OR_EQUALS_OPERATOR LESS_THAN_OR_EQUALS_OPERATOR LESS_THAN_OPERATOR GREATER_THAN_OPERATOR

/*
*  Initial production rule.
*/
%start C_host_program


%%

/*
*  Y A C C   P R O D U C T I O N   R U L E S   A N D   A C T I O N S
*/

/*
*  Universal Embedded SQL.
*/

C_host_program:
	  C_host_program embedded_SQL_program
	| /* empty */
	;

embedded_SQL_program:
          EMBEDDED_SQL_BEGIN embedded_SQL_host_program EMBEDDED_SQL_END
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("embedded_SQL_host_program", $2.yCode);

            //
            //  Output line directives.
            //
            if ( ptApplication->useLineDirective() )
            {
              ptApplication->outputCode
              (
                ptApplication->getLineDirective (ptApplication->getEmbeddedLineNumber())
              );
            }

            //
            //  Output code for program.
            //
            ptApplication->outputClauseCode ("embedded_SQL_program", "body");

            //
            //  Add prefix and postfix code.
            //
            if ( !ptApplication->isReusingTarget() )
            {
              ptApplication->initCodeParameters();
              ptApplication->preAppend
              (
                ptApplication->getClauseCode ("embedded_SQL_program", "prefix")
              );

              //
              //  Pre-appends line directive.
              //
              if ( ptApplication->useLineDirective() )
              {
                ptApplication->preAppend ("\n");
                ptApplication->preAppend (ptApplication->getLineDirective (1));
              }
              ptApplication->initCodeParameters();
              ptApplication->postAppend
              (
                ptApplication->getClauseCode ("embedded_SQL_program", "postfix")
              );
            }

            //
            //  Output line directive for next line of plain C++ code.
            //
            if ( ptApplication->useLineDirective() )
            {
              ptApplication->outputCode ("\n");
              ptApplication->outputCode (ptApplication->getLineDirective (yylineno));
            }
          }
	;


/*
*  Section 19. Embedded SQL.
*/

/*
*  Subsection 19.4:  <embedded SQL C program>.
*  Description:      Specify an <embedded SQL C program>.
*/

embedded_SQL_C_program:
	  embedded_SQL_C_program embedded_SQL_statement
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("embedded_SQL_C_program", $1.yCode);
            ptApplication->bindCodeParameter ("embedded_SQL_statement", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("embedded_SQL_C_program", "n-adic");
          }
	|                        embedded_SQL_statement
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("embedded_SQL_statement", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("embedded_SQL_C_program", "monadic");
          }
	;


/*
*  Subsection 19.2:  <embedded exception declaration>.
*  Description:      Specify the action to be taken when an SQL-statement causes 
*                    a specific class of condition to be raised.
*/

embedded_exception_declaration:
	  RW_WHENEVER condition condition_action
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("condition",        $2.yCode);
            ptApplication->bindCodeParameter ("condition_action", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("embedded_exception_declaration");
          }
	;

condition:
	  RW_SQLERROR 
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition", "sqlerror");
          }
	| RW_NOT RW_FOUND
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition", "not_found");
          }
	;

condition_action:
	  RW_CONTINUE 
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_action", "continue");
          }
	| go_to
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("go_to", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("condition_action", "go_to");
          }
	| throw
          {
            ptApplication->require (eSQL92_F_19_2_A);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("throw", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("condition_action", "throw");
          }
	;

go_to:
	  RW_GOTO     goto_target
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("goto_target", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("go_to");
          }
	| RW_GO RW_TO goto_target
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("goto_target", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("go_to");
          }
	;
	
goto_target:
	  host_label_identifier
	;

throw:
	  RW_THROW exception_target
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("exception_target", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("throw");
          }
        ;

exception_target:
	  MULTI_HOST_IDENTIFIER
        | identifier_body
          {
            $$.yCode = $1.yOriginalText;
          }
	;

/*
*  Warning:  Let's verify if this rule conforms the
*            C definition of a label.
*/
host_label_identifier:
	  identifier_body
          {
            $$.yCode = $1.yOriginalText;
          }
	;


/*
*  Subsection 19.1:  <embedded SQL host program>.
*  Description:      Specify an <embedded SQL host program>.
*/

embedded_SQL_host_program:
	  embedded_SQL_C_program
	;

/*
*  Handles the possible syntax errors with the token 'error',  that is provided
*  by YACC.
*/
embedded_SQL_statement:
	  statement_or_declaration SQL_terminator
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("statement_or_declaration", $1.yCode);
            if ( !ptApplication->useLineDirective() )
            {
              $$.yCode = ptApplication->getClauseCode ("embedded_SQL_statement");
            }
            else
            {
              $$.yCode  = ptApplication->getLineDirective (yylineno);
              $$.yCode += ptApplication->getClauseCode ("embedded_SQL_statement");
            }
          }
        | error SQL_terminator
        | error
	;
	
/*
*  Warning:  This rule has been modified, by adding <declare search condition>.
*/
statement_or_declaration:
	  declare_cursor
        | declare_search_condition
/* <DISABLED id="dynamic statements disabled"> */
/* 	| dynamic_declare_cursor */
/* </DISABLED> */
	| embedded_exception_declaration
	| SQL_procedure_statement
	;

SQL_terminator:
	  SEMICOLON
	;

embedded_variable_name:
	  EMBEDDED_N_ADIC_FUNCTION_CALL
          {
            ptApplication->require (eSQL92_F_19_1_A);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("embedded_variable_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("embedded_variable_name");
          }
	| EMBEDDED_ARRAY
          {
            ptApplication->require (eSQL92_F_19_1_A);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("embedded_variable_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("embedded_variable_name");
          }
	| EMBEDDED_VARIABLE_NAME
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("embedded_variable_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("embedded_variable_name");
          }
	;


/*
*  Section 18. Diagnostics management.
*/

/*
*  Subsection 18.1:  <get diagnostics statement>.
*  Description:      Get exception or completion condition information from 
*                    the diagnostics area.
*/

get_diagnostics_statement:
	  RW_GET RW_DIAGNOSTICS sql_diagnostics_information
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("sql_diagnostics_information", $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("get_diagnostics_statement");
            $$.iItemCount = $3.iItemCount;
          }
	;

sql_diagnostics_information:
	  statement_information
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("statement_information", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("sql_diagnostics_information", "statement_information");
            $$.iItemCount = $1.iItemCount;
          }
	| condition_information
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("condition_information", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("sql_diagnostics_information", "condition_information");
            $$.iItemCount = $1.iItemCount;
          }
	;

statement_information:
	  statement_information COMMA statement_information_item
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("statement_information",      $1.yCode);
            ptApplication->bindCodeParameter ("statement_information_item", $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("statement_information", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                             statement_information_item
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("statement_information_item", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("statement_information", "monadic");
            $$.iItemCount = 1;
          }
	;

statement_information_item:
	  simple_target_specification EQUALS_OPERATOR statement_information_item_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("simple_target_specification",     $1.yCode);
            ptApplication->bindCodeParameter ("statement_information_item_name", $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("statement_information_item");
            $$.iItemCount = 1;
          }
	;

statement_information_item_name:
	  NRW_NUMBER
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("statement_information_item_name", "number");
          }
	| NRW_MORE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("statement_information_item_name", "more");
          }
	| NRW_COMMAND_FUNCTION
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("statement_information_item_name", "command_function");
          }
	| NRW_DYNAMIC_FUNCTION
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("statement_information_item_name", "dynamic_function");
          }
	| NRW_ROW_COUNT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("statement_information_item_name", "row_count");
          }
	;

/*
*  Auxiliary rule for:  condition_information
*/
condition_information_item_list:
	  condition_information_item_list COMMA condition_information_item
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("condition_information_item_list", $1.yCode);
            ptApplication->bindCodeParameter ("condition_information_item",      $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("condition_information_item_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                                       condition_information_item
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("condition_information_item", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("condition_information_item_list", "monadic");
            $$.iItemCount = 1;
          }
	;

condition_information:
	  RW_EXCEPTION condition_number condition_information_item_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("condition_number",                $2.yCode);
            ptApplication->bindCodeParameter ("condition_information_item_list", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("condition_information");
          }
	;

condition_information_item:
	  simple_target_specification EQUALS_OPERATOR condition_information_item_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("simple_target_specification",     $1.yCode);
            ptApplication->bindCodeParameter ("condition_information_item_name", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("condition_information_item");
          }
	;

condition_information_item_name:
	  NRW_CONDITION_NUMBER
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_information_item_name", "condition_number");
          }
	| NRW_RETURNED_SQLSTATE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_information_item_name", "returned_sqlstate");
          }
	| NRW_CLASS_ORIGIN
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_information_item_name", "class_origin");
          }
	| NRW_SUBCLASS_ORIGIN
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_information_item_name", "subclass_origin");
          }
	| NRW_SERVER_NAME
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_information_item_name", "server_name");
          }
	| NRW_CONNECTION_NAME
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_information_item_name", "connection_name");
          }
	| NRW_CONSTRAINT_CATALOG
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_information_item_name", "constraint_catalog");
          }
	| NRW_CONSTRAINT_SCHEMA
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_information_item_name", "constraint_schema");
          }
	| NRW_CONSTRAINT_NAME
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_information_item_name", "constraint_name");
          }
	| NRW_CATALOG_NAME
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_information_item_name", "catalog_name");
          }
	| NRW_SCHEMA_NAME
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_information_item_name", "schema_name");
          }
	| NRW_TABLE_NAME
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_information_item_name", "table_name");
          }
	| NRW_COLUMN_NAME
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_information_item_name", "column_name");
          }
	| NRW_CURSOR_NAME
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_information_item_name", "cursor_name");
          }
	| NRW_MESSAGE_TEXT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_information_item_name", "message_text");
          }
	| NRW_MESSAGE_LENGTH
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_information_item_name", "message_length");
          }
	| NRW_MESSAGE_OCTET_LENGTH
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("condition_information_item_name", "message_octet_length");
          }
	;
	
condition_number:
	  simple_value_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("simple_value_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("condition_number");
          }
	;


/*
*  Section 17. Dynamic SQL.
*/

/*
*  Subsection 17.A:  <declare search condition>.
*  Description:      Extension to SQL-92 to allow dynamic creation of conditions
*                    using embedded variables.
*/

declare_search_condition:
          RW_DECLARE search_condition_name RW_CONDITION RW_WHERE search_condition
          {
            ptApplication->require (eSQL92_F_17_A_A);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("search_condition_name", $2.yCode);
            ptApplication->bindCodeParameter ("search_condition",      $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("declare_search_condition");
          }
        ;

search_condition_name:
          target_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("target_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("search_condition_name");
          }
	;


/*
*  Subsection 17.18:  <dynamic update statement: positioned>.
*  Description:       Update a row of a table.
*/

/* <DISABLED id="dynamic statements disabled"> */
/* dynamic_update_statement.positioned: */
/* 	  RW_UPDATE table_name */
/* 	  RW_SET set_clause_list */
/* 	  RW_WHERE RW_CURRENT RW_OF dynamic_cursor_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("table_name",          $2.yCode); */
/*             ptApplication->bindCodeParameter ("set_clause_list",     $4.yCode); */
/*             ptApplication->bindCodeParameter ("dynamic_cursor_name", $8.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("dynamic_update_statement.positioned"); */
/*           } */
/* 	; */
/* </DISABLED> */


/*
*  Subsection 17.17:  <dynamic delete statement: positioned>.
*  Description:       Delete a row of a table.
*/

/* <DISABLED id="dynamic statements disabled"> */
/* dynamic_delete_statement.positioned: */
/* 	  RW_DELETE RW_FROM table_name RW_WHERE RW_CURRENT RW_OF dynamic_cursor_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("table_name",          $3.yCode); */
/*             ptApplication->bindCodeParameter ("dynamic_cursor_name", $7.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("dynamic_delete_statement.positioned"); */
/*           } */
/* 	; */
/* </DISABLED> */


/*
*  Subsection 17.16:  <dynamic close statement>.
*  Description:       Close a cursor.
*/

/* <DISABLED id="dynamic statements disabled"> */
/* dynamic_close_statement: */
/* 	  RW_CLOSE dynamic_cursor_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("dynamic_cursor_name", $2.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("dynamic_close_statement"); */
/*           } */
/* 	; */
/* </DISABLED> */


/*
*  Subsection 17.15:  <dynamic fetch statement>.
*  Description:       Fetch a row for a cursor declared with a <dynamic declare cursor>.
*/

/* <DISABLED id="dynamic statements disabled"> */
/* dynamic_fetch_statement: */
/* 	  RW_FETCH fetch_orientation RW_FROM dynamic_cursor_name using_clause */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("fetch_orientation",   $2.yCode); */
/*             ptApplication->bindCodeParameter ("dynamic_cursor_name", $4.yCode); */
/*             ptApplication->bindCodeParameter ("using_clause",        $5.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("dynamic_fetch_statement", "oriented"); */
/*           } */
/* 	| RW_FETCH                   RW_FROM dynamic_cursor_name using_clause */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("dynamic_cursor_name", $3.yCode); */
/*             ptApplication->bindCodeParameter ("using_clause",        $4.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("dynamic_fetch_statement", "not-oriented"); */
/*           } */
/* 	| RW_FETCH                           dynamic_cursor_name using_clause */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("dynamic_cursor_name", $2.yCode); */
/*             ptApplication->bindCodeParameter ("using_clause",        $3.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("dynamic_fetch_statement", "not-oriented"); */
/*           } */
/* 	; */
/* </DISABLED> */


/*
*  Subsection 17.14:  <dynamic open statement>.
*  Description:       Associate input parameters with a <cursor specification> and open
*                     the cursor.
*/

/* <DISABLED id="dynamic statements disabled"> */
/* dynamic_open_statement: */
/* 	  RW_OPEN dynamic_cursor_name using_clause */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("dynamic_cursor_name", $2.yCode); */
/*             ptApplication->bindCodeParameter ("using_clause",        $3.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("dynamic_open_statement", "using"); */
/*           } */
/* 	| RW_OPEN dynamic_cursor_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("dynamic_cursor_name", $2.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("dynamic_open_statement", "without-using"); */
/*           } */
/* 	; */
/* </DISABLED> */


/*
*  Subsection 17.13:  <allocate cursor statement>.
*  Description:       Define a cursor based on a <prepare statement> for 
*                     a <cursor specification>.
*/

/* <DISABLED id="dynamic statements disabled"> */
/* allocate_cursor_statement: */
/* 	  RW_ALLOCATE extended_cursor_name RW_INSENSITIVE RW_SCROLL */
/* 	  RW_CURSOR RW_FOR extended_statement_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("extended_cursor_name",    $2.yCode); */
/*             ptApplication->bindCodeParameter ("extended_statement_name", $7.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("allocate_cursor_statement", "with-both"); */
/*           } */
/* 	| RW_ALLOCATE extended_cursor_name RW_INSENSITIVE  */
/* 	  RW_CURSOR RW_FOR extended_statement_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("extended_cursor_name",    $2.yCode); */
/*             ptApplication->bindCodeParameter ("extended_statement_name", $6.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("allocate_cursor_statement", "without-scroll"); */
/*           } */
/* 	| RW_ALLOCATE extended_cursor_name                RW_SCROLL */
/* 	  RW_CURSOR RW_FOR extended_statement_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("extended_cursor_name",    $2.yCode); */
/*             ptApplication->bindCodeParameter ("extended_statement_name", $6.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("allocate_cursor_statement", "without-insensitive"); */
/*           } */
/* 	| RW_ALLOCATE extended_cursor_name */
/* 	  RW_CURSOR RW_FOR extended_statement_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("extended_cursor_name",    $2.yCode); */
/*             ptApplication->bindCodeParameter ("extended_statement_name", $5.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("allocate_cursor_statement", "without-both"); */
/*           } */
/* 	;   */
/* </DISABLED> */


/*
*  Subsection 17.12:  <dynamic declare cursor>.
*  Description:       Declare a cursor to be associated with a <statement name>, which
*                     may in turn be associated with a <cursor specification>.
*/

/* <DISABLED id="dynamic statements disabled"> */
/* dynamic_declare_cursor: */
/* 	  RW_DECLARE cursor_name RW_INSENSITIVE RW_SCROLL RW_CURSOR RW_FOR statement_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("cursor_name",    $2.yCode); */
/*             ptApplication->bindCodeParameter ("statement_name", $7.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("dynamic_declare_cursor", "with-both"); */
/*           } */
/* 	| RW_DECLARE cursor_name RW_INSENSITIVE           RW_CURSOR RW_FOR statement_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("cursor_name",    $2.yCode); */
/*             ptApplication->bindCodeParameter ("statement_name", $6.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("dynamic_declare_cursor", "without-scroll"); */
/*           } */
/* 	| RW_DECLARE cursor_name                RW_SCROLL RW_CURSOR RW_FOR statement_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("cursor_name",    $2.yCode); */
/*             ptApplication->bindCodeParameter ("statement_name", $6.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("dynamic_declare_cursor", "without-insensitive"); */
/*           } */
/* 	| RW_DECLARE cursor_name                          RW_CURSOR RW_FOR statement_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("cursor_name",    $2.yCode); */
/*             ptApplication->bindCodeParameter ("statement_name", $5.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("dynamic_declare_cursor", "without-both"); */
/*           } */
/* 	; */
/* </DISABLED> */


/*
*  Subsection 17.11:  <execute immediate statement>.
*  Description:       Dynamically prepare and execute a preparable statement.
*/

/* <DISABLED id="dynamic statements disabled"> */
/* execute_immediate_statement: */
/* 	  RW_EXECUTE RW_IMMEDIATE SQL_statement_variable */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("SQL_statement_variable", $3.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("execute_immediate_statement"); */
/*           } */
/* 	; */
/* </DISABLED> */


/*
*  Subsection 17.10:  <execute statement>.
*  Description:       Associate input parameters and output targets with a prepared
*                     statement and execute the statement.
*/

/* <DISABLED id="dynamic statements disabled"> */
/* execute_statement: */
/* 	  RW_EXECUTE SQL_statement_name result_using_clause parameter_using_clause */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("SQL_statement_name",     $2.yCode); */
/*             ptApplication->bindCodeParameter ("result_using_clause",    $3.yCode); */
/*             ptApplication->bindCodeParameter ("parameter_using_clause", $4.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("execute_statement", "with-both"); */
/*           } */
/* 	| RW_EXECUTE SQL_statement_name result_using_clause */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("SQL_statement_name",  $2.yCode); */
/*             ptApplication->bindCodeParameter ("result_using_clause", $3.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("execute_statement", "with-result-using"); */
/*           } */
/* 	| RW_EXECUTE SQL_statement_name                     parameter_using_clause */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("SQL_statement_name",     $2.yCode); */
/*             ptApplication->bindCodeParameter ("parameter_using_clause", $3.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("execute_statement", "with-parameter-using"); */
/*           } */
/* 	| RW_EXECUTE SQL_statement_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("SQL_statement_name", $2.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("execute_statement", "without-both"); */
/*           } */
/* 	; */

/* result_using_clause: */
/* 	  using_clause */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("using_clause", $1.yCode); */
/*             $$.yCode      = ptApplication->getClauseCode ("result_using_clause"); */
/*             $$.iItemCount = $1.iItemCount; */
/*           } */
/* 	; */

/* parameter_using_clause: */
/* 	  using_clause */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("using_clause", $1.yCode); */
/*             $$.yCode      = ptApplication->getClauseCode ("parameter_using_clause"); */
/*             $$.iItemCount = $1.iItemCount; */
/*           } */
/* 	; */
/* </DISABLED> */


/*
*  Subsection 17.9:  <using clause>.
*  Description:      Describe the input/output variables
*                    for an <SQL dynamic statement>.
*/

/* <DISABLED id="dynamic statements disabled"> */
/* using_clause: */
/* 	  using_arguments */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("using_arguments", $1.yCode); */
/*             $$.yCode      = ptApplication->getClauseCode ("using_clause", "arguments"); */
/*             $$.iItemCount = $1.iItemCount; */
/*           } */
/* 	| using_descriptor */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("using_descriptor", $1.yCode); */
/*             $$.yCode      = ptApplication->getClauseCode ("using_clause", "descriptor"); */
/*             $$.iItemCount = $1.iItemCount; */
/*           } */
/* 	; */
/* </DISABLED> */

/*
*  Auxiliary rule for:  using_arguments
*/
/* <DISABLED id="dynamic statements disabled"> */
/* argument_list: */
/* 	  argument_list COMMA argument */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("argument_list", $1.yCode); */
/*             ptApplication->bindCodeParameter ("argument",      $3.yCode); */
/*             $$.yCode      = ptApplication->getClauseCode ("argument_list", "n-adic"); */
/*             $$.iItemCount = $1.iItemCount + 1; */
/*           } */
/* 	|                     argument */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("argument", $1.yCode); */
/*             $$.yCode      = ptApplication->getClauseCode ("argument_list", "monadic"); */
/*             $$.iItemCount = 1; */
/*           } */
/* 	; */
	
/* using_arguments: */
/* 	  RW_USING argument_list */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("argument_list", $2.yCode); */
/*             $$.yCode      = ptApplication->getClauseCode ("using_arguments", "using"); */
/*             $$.iItemCount = $2.iItemCount; */
/*           } */
/* 	| RW_INTO  argument_list */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("argument_list", $2.yCode); */
/*             $$.yCode      = ptApplication->getClauseCode ("using_arguments", "into"); */
/*             $$.iItemCount = $2.iItemCount; */
/*           } */
/* 	; */
	
/* argument: */
/* 	  target_specification */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("target_specification", $1.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("argument"); */
/*           } */
/* 	; */

/* using_descriptor: */
/* 	  RW_USING RW_SQL RW_DESCRIPTOR descriptor_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("descriptor_name", $4.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("using_descriptor", "using"); */
/*           } */
/* 	| RW_INTO  RW_SQL RW_DESCRIPTOR descriptor_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("descriptor_name", $4.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("using_descriptor", "into"); */
/*           } */
/* 	; */
/* </DISABLED> */


/*
*  Subsection 17.8:  <describe statement>.
*  Description:      Obtain information about the <select list> columns or 
*                    <dynamic parameter specification>s contained in a 
*                    prepared statement.
*/

/* <DISABLED id="dynamic statements disabled"> */
/* describe_statement: */
/* 	  describe_input_statement */
/* 	| describe_output_statement */
/* 	; */

/* describe_input_statement: */
/* 	  RW_DESCRIBE RW_INPUT SQL_statement_name using_descriptor */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("SQL_statement_name", $3.yCode); */
/*             ptApplication->bindCodeParameter ("using_descriptor",   $4.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("describe_input_statement"); */
/*           } */
/* 	; */

/* describe_output_statement: */
/* 	  RW_DESCRIBE RW_OUTPUT SQL_statement_name using_descriptor */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("SQL_statement_name", $3.yCode); */
/*             ptApplication->bindCodeParameter ("using_descriptor",   $4.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("describe_output_statement"); */
/*           } */
/* 	| RW_DESCRIBE           SQL_statement_name using_descriptor */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("SQL_statement_name", $2.yCode); */
/*             ptApplication->bindCodeParameter ("using_descriptor",   $3.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("describe_output_statement"); */
/*           } */
/* 	; */
/* </DISABLED> */


/*
*  Subsection 17.7:  <deallocate prepared statement>.
*  Description:      Deallocate SQL-statements that have been prepared with a 
*                    <prepare statement>.
*/

/* <DISABLED id="dynamic statements disabled"> */
/* deallocate_prepared_statement: */
/* 	  RW_DEALLOCATE RW_PREPARE SQL_statement_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("SQL_statement_name", $3.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("deallocate_prepared_statement"); */
/*           } */
/* 	; */
/* </DISABLED> */


/*
*  Subsection 17.6:  <prepare statement>.
*  Description:      Prepare a statement for execution.
*/

/* <DISABLED id="dynamic statements disabled"> */
/* prepare_statement: */
/* 	  RW_PREPARE SQL_statement_name RW_FROM SQL_statement_variable */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("SQL_statement_name",     $2.yCode); */
/*             ptApplication->bindCodeParameter ("SQL_statement_variable", $4.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("prepare_statement"); */
/*           } */
/* 	; */

/* SQL_statement_variable: */
/* 	  simple_value_specification */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("simple_value_specification", $1.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("condition_number"); */
/*           } */
/* 	; */
/* </DISABLED> */


/*
*  Subsection 17.5:  <set descriptor statement>.
*  Description:      Set information in an SQL descriptor area.
*/

/* <DISABLED id="dynamic statements disabled"> */
/* set_descriptor_statement: */
/* 	  RW_SET RW_DESCRIPTOR descriptor_name set_descriptor_information */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("descriptor_name",            $3.yCode); */
/*             ptApplication->bindCodeParameter ("set_descriptor_information", $4.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("set_descriptor_statement"); */
/*           } */
/* 	; */
/* </DISABLED> */

/*
*  Auxiliary rule for:  set_descriptor_information
*/
/* <DISABLED id="dynamic statements disabled"> */
/* set_item_information_list: */
/* 	  set_item_information_list COMMA set_item_information */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("set_item_information_list", $1.yCode); */
/*             ptApplication->bindCodeParameter ("set_item_information",      $3.yCode); */
/*             $$.yCode      = ptApplication->getClauseCode ("set_item_information_list", "n-adic"); */
/*             $$.iItemCount = $1.iItemCount + 1; */
/*           } */
/* 	|                                 set_item_information */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("set_item_information", $1.yCode); */
/*             $$.yCode      = ptApplication->getClauseCode ("set_item_information_list", "monadic"); */
/*             $$.iItemCount = 1; */
/*           } */
/* 	; */

/* set_descriptor_information: */
/* 	  set_count */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("set_count", $1.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("set_descriptor_information", "set_count"); */
/*           } */
/* 	| RW_VALUE item_number set_item_information_list */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("item_number",               $2.yCode); */
/*             ptApplication->bindCodeParameter ("set_item_information_list", $3.yCode); */
/*             $$.yCode      = ptApplication->getClauseCode ("set_descriptor_information", "value"); */
/*             $$.iItemCount = $3.iItemCount; */
/*           } */
/* 	; */

/* set_count: */
/* 	  RW_COUNT EQUALS_OPERATOR simple_value_specification_1 */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("simple_value_specification_1", $3.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("set_count"); */
/*           } */
/* 	; */
	
/* set_item_information: */
/* 	  descriptor_item_name EQUALS_OPERATOR simple_value_specification_2 */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("descriptor_item_name",         $1.yCode); */
/*             ptApplication->bindCodeParameter ("simple_value_specification_2", $3.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("set_item_information"); */
/*           } */
/* 	; */

/* simple_value_specification_1: */
/* 	  simple_value_specification */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("simple_value_specification", $1.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("simple_value_specification_1"); */
/*           } */
/* 	; */

/* simple_value_specification_2: */
/* 	  simple_value_specification */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("simple_value_specification", $1.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("simple_value_specification_2"); */
/*           } */
/* 	; */

/* item_number: */
/* 	  simple_value_specification */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("simple_value_specification", $1.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("item_number"); */
/*           } */
/* 	; */
/* </DISABLED> */


/*
*  Subsection 17.4:  <get descriptor statement>.
*  Description:      Get information from an SQL descriptor area.
*/

/* <DISABLED id="dynamic statements disabled"> */
/* get_descriptor_statement: */
/* 	  RW_GET RW_DESCRIPTOR descriptor_name get_descriptor_information */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("descriptor_name",            $3.yCode); */
/*             ptApplication->bindCodeParameter ("get_descriptor_information", $4.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("get_descriptor_statement"); */
/*           } */
/* 	; */
/* </DISABLED> */

/*
*  Auxiliary rule for:  get_descriptor_information
*/
/* <DISABLED id="dynamic statements disabled"> */
/* get_item_information_list: */
/* 	  get_item_information_list COMMA get_item_information */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("get_item_information_list", $1.yCode); */
/*             ptApplication->bindCodeParameter ("get_item_information",      $3.yCode); */
/*             $$.yCode      = ptApplication->getClauseCode ("get_item_information_list", "n-adic"); */
/*             $$.iItemCount = $1.iItemCount + 1; */
/*           } */
/* 	|                                 get_item_information  */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("get_item_information", $1.yCode); */
/*             $$.yCode      = ptApplication->getClauseCode ("get_item_information_list", "monadic"); */
/*             $$.iItemCount = 1; */
/*           } */
/* 	; */

/* get_descriptor_information: */
/* 	  get_count */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("get_count", $1.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("get_descriptor_information", "get_count"); */
/*           } */
/* 	| RW_VALUE item_number get_item_information_list */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("item_number",               $2.yCode); */
/*             ptApplication->bindCodeParameter ("get_item_information_list", $3.yCode); */
/*             $$.yCode      = ptApplication->getClauseCode ("get_descriptor_information", "value"); */
/*             $$.iItemCount = $3.iItemCount; */
/*           } */
/* 	; */

/* get_count: */
/* 	  simple_target_specification_1 EQUALS_OPERATOR RW_COUNT */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("simple_target_specification_1", $1.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("get_count"); */
/*           } */
/* 	; */
	
/* get_item_information: */
/* 	  simple_target_specification_2 EQUALS_OPERATOR descriptor_item_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("simple_target_specification_2", $1.yCode); */
/*             ptApplication->bindCodeParameter ("descriptor_item_name",          $3.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("get_item_information"); */
/*           } */
/* 	; */

/* simple_target_specification_1: */
/* 	  simple_target_specification */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("simple_target_specification", $1.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("simple_target_specification_1"); */
/*           } */
/* 	; */

/* simple_target_specification_2: */
/* 	  simple_target_specification */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("simple_target_specification", $1.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("simple_target_specification_2"); */
/*           } */
/* 	; */

/* descriptor_item_name: */
/* 	  NRW_TYPE */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "type"); */
/*           } */
/* 	| NRW_LENGTH */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "length"); */
/*           } */
/* 	| RW_OCTET_LENGTH */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "octet_length"); */
/*           } */
/* 	| NRW_RETURNED_LENGTH */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "returned_length"); */
/*           } */
/* 	| NRW_RETURNED_OCTET_LENGTH */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "returned_octet_length"); */
/*           } */
/* 	| RW_PRECISION */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "precision"); */
/*           } */
/* 	| NRW_SCALE */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "scale"); */
/*           } */
/* 	| NRW_DATETIME_INTERVAL_CODE */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "datetime_interval_code"); */
/*           } */
/* 	| NRW_DATETIME_INTERVAL_PRECISION */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "datetime_interval_precision"); */
/*           } */
/* 	| NRW_NULLABLE */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "nullable"); */
/*           } */
/* 	| RW_INDICATOR */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "indicator"); */
/*           } */
/* 	| NRW_DATA */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "data"); */
/*           } */
/* 	| NRW_NAME */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "name"); */
/*           } */
/* 	| NRW_UNNAMED */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "unnamed"); */
/*           } */
/* 	| NRW_COLLATION_CATALOG */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "collation_catalog"); */
/*           } */
/* 	| NRW_COLLATION_SCHEMA */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "collation_schema"); */
/*           } */
/* 	| NRW_COLLATION_NAME */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "collation_name"); */
/*           } */
/* 	| NRW_CHARACTER_SET_CATALOG */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "character_set_catalog"); */
/*           } */
/* 	| NRW_CHARACTER_SET_SCHEMA */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "character_set_schema"); */
/*           } */
/* 	| NRW_CHARACTER_SET_NAME */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             $$.yCode = ptApplication->getClauseCode ("descriptor_item_name", "character_set_name"); */
/*           } */
/* 	; */
/* </DISABLED> */


/*
*  Subsection 17.3:  <deallocate descriptor statement>.
*  Description:      Deallocate an SQL descriptor area.
*/

/* <DISABLED id="dynamic statements disabled"> */
/* deallocate_descriptor_statement: */
/* 	  RW_DEALLOCATE RW_DESCRIPTOR descriptor_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("descriptor_name", $3.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("deallocate_descriptor_statement"); */
/*           } */
/* 	; */
/* </DISABLED> */


/*
*  Subsection 17.2:  <allocate descriptor statement>.
*  Description:      Allocate an SQL descriptor area.
*/

/* <DISABLED id="dynamic statements disabled"> */
/* allocate_descriptor_statement: */
/* 	  RW_ALLOCATE RW_DESCRIPTOR descriptor_name RW_WITH RW_MAX occurrences */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("descriptor_name", $3.yCode); */
/*             ptApplication->bindCodeParameter ("ocurrences",      $6.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("allocate_descriptor_statement", "with-max"); */
/*           } */
/* 	| RW_ALLOCATE RW_DESCRIPTOR descriptor_name */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("descriptor_name", $3.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("allocate_descriptor_statement", "without-max"); */
/*           } */
/* 	; */

/* occurrences: */
/* 	  simple_value_specification */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("simple_value_specification", $1.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("ocurrences"); */
/*           } */
/* 	; */
/* </DISABLED> */


/*
*  Section 16. Session management.
*/

/*
*  Subsection 16.5:  <set local time zone statement>.
*  Description:      Set the default local time zone displacement for the current
*                    SQL-session.
*/

set_local_time_zone_statement:
	  RW_SET RW_TIME RW_ZONE set_time_zone_value
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("set_time_zone_value", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("set_local_time_zone_statement");
          }
	;

set_time_zone_value:
	  interval_value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("set_time_zone_value", "expression");
          }
	| RW_LOCAL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("set_time_zone_value", "local");
          }
	;
	

/*
*  Subsection 16.4:  <set session authorization identifier statement>.
*  Description:      Set the <authorization identifier> of the current SQL-session.
*/

set_session_authorization_identifier_statement:
	  RW_SET RW_SESSION RW_AUTHORIZATION value_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_specification", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("set_session_authorization_identifier_statement");
          }
	;


/*
*  Subsection 16.3:  <set names statement>.
*  Description:      Set the default character set name for <identifier>s and <character
*                    string literal>s in <preparable statement>s that are prepared in
*                    the current SQL-session by an <execute immediate statement> or a
*                    <prepare statement> and in <direct SQL statement>s that are invoked
*                    directly.
*/

set_names_statement:
	  RW_SET RW_NAMES value_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_specification", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("set_names_statement");
          }
	;


/*
*  Subsection 16.2:  <set schema statement>.
*  Description:      Set the default schema name for unqualified <qualified name>s in
*                    <preparable statement>s that are prepared in the current SQL-
*                    session by an <execute immediate statement> or a 
*                    <prepare statement> and in <direct SQL statement>s that are 
*                    invoked directly.
*/

set_schema_statement:
	  RW_SET RW_SCHEMA value_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_specification", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("set_schema_statement");
          }
	;


/*
*  Subsection 16.1:  <set catalog statement>.
*  Description:      Set the default catalog name for unqualified <schema name>s in
*                    <preparable statement>s that are prepared in the current SQL-
*                    session by an <execute immediate statement> or a
*                    <prepare statement> and in <direct SQL statement>s that are 
*                    invoked directly.
*/

set_catalog_statement:
	  RW_SET RW_CATALOG value_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_specification", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("set_catalog_statement");
          }
	;


/*
*  Section 15. Connection management.
*/

/*
*  Subsection 15.3:  <disconnect statement>.
*  Description:      Terminate an SQL-connection.
*/

disconnect_statement:
	  RW_DISCONNECT disconnect_object
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("disconnect_object", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("disconnect_statement");
          }
	;

disconnect_object:
	  connection_object
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("connection_object", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("disconnect_object", "connection_object");
          }
	| RW_ALL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("disconnect_object", "all");
          }
	| RW_CURRENT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("disconnect_object", "current");
          }
	;


/*
*  Subsection 15.2:  <set connection statement>.
*  Description:      Select an SQL-connection from the available SQL-connections.
*/

set_connection_statement:
	  RW_SET RW_CONNECTION connection_object
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("connection_object", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("set_connection_statement");
          }
	;

connection_object:
	  RW_DEFAULT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("connection_object", "default");
          }
	| connection_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("connection_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("connection_object", "connection_name");
          }
	;


/*
*  Subsection 15.1:  <connect statement>.
*  Description:      Establish an SQL-connection.
*/

connect_statement:
	  RW_CONNECT RW_TO connection_target
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("connection_target", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("connect_statement");
          }
	;

connection_target:
	  schema_location RW_AS connection_name RW_USER user_login
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("schema_location", $1.yCode);
            ptApplication->bindCodeParameter ("connection_name", $3.yCode);
            ptApplication->bindCodeParameter ("user_login",      $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("connection_target", "with-both");
          }
	| schema_location RW_AS connection_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("schema_location", $1.yCode);
            ptApplication->bindCodeParameter ("connection_name", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("connection_target", "without-user");
          }
	| schema_location                       RW_USER user_login
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("schema_location", $1.yCode);
            ptApplication->bindCodeParameter ("user_login",      $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("connection_target", "without-as");
          }
	| schema_location
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("schema_location", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("connection_target", "without-both");
          }
	| RW_DEFAULT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("connection_target", "default");
          }
	;


/*
*  Section 14. Transaction management.
*/

/*
*  Subsection 14.4:  <rollback statement>.
*  Description:      Terminate the current SQL-transaction with rollback.
*/

rollback_statement:
	  RW_ROLLBACK RW_WORK
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("rollback_statement");
          }
	| RW_ROLLBACK
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("rollback_statement");
          }
	;


/*
*  Subsection 14.3:  <commit statement>.
*  Description:      Terminate the current SQL-transaction with commit.
*/

commit_statement:
	  RW_COMMIT RW_WORK
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("commit_statement");
          }
	| RW_COMMIT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("commit_statement");
          }
	;


/*
*  Subsection 14.2:  <set constraints mode statement>.
*  Description:      If an SQL-transaction is currently active, then set the constraint
*                    mode for that SQL-transaction in the current SQL-session. If no
*                    SQL-transaction is currently active, then set the constraint mode
*                    for the next SQL-transaction in the current SQL-session for the
*                    SQL-agent.
*/

set_constraints_mode_statement:
	  RW_SET RW_CONSTRAINTS constraint_name_list RW_DEFERRED
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_name_list", $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("set_constraints_mode_statement", "deferred");
            $$.iItemCount = $3.iItemCount;
          }
	| RW_SET RW_CONSTRAINTS constraint_name_list RW_IMMEDIATE
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_name_list", $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("set_constraints_mode_statement", "immediate");
            $$.iItemCount = $3.iItemCount;
          }
	;

/*
*  Auxiliary rule for:  constraint_name_list
*/
constraint_name_list_1:
	  constraint_name_list_1 COMMA constraint_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_name_list_1", $1.yCode);
            ptApplication->bindCodeParameter ("constraint_name",        $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("constraint_name_list_1", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                              constraint_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_name", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("constraint_name_list_1", "monadic");
            $$.iItemCount = 1;
          }
	;

constraint_name_list:
	  RW_ALL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("constraint_name_list_1", "all");
          }
	| constraint_name_list_1
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_name_list_1", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("constraint_name_list_1", "constraint_name_list_1");
            $$.iItemCount = $1.iItemCount;
          }
	;


/*
*  Subsection 14.1:  <set transaction statement>.
*  Description:      Set the attributes of the next SQL-transaction for the SQL-agent.
*/

/*
*  Auxiliary rule for:  set_transaction_statement
*/
transaction_mode_list:
	  transaction_mode_list COMMA transaction_mode
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("transaction_mode_list", $1.yCode);
            ptApplication->bindCodeParameter ("transaction_mode",      $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("transaction_mode_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                             transaction_mode
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("transaction_mode", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("transaction_mode_list", "monadic");
            $$.iItemCount = 1;
          }
	;

set_transaction_statement:
	  RW_SET RW_TRANSACTION transaction_mode_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("transaction_mode_list", $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("set_transaction_statement");
            $$.iItemCount = $3.iItemCount;
          }
	;
	
transaction_mode:
	  isolation_level
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("isolation_level", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("transaction_mode", "isolation_level");
          }
	| transaction_access_mode
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("transaction_access_mode", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("transaction_mode", "transaction_access_mode");
          }
	| diagnostics_size
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("diagnostics_size", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("transaction_mode", "diagnostics_size");
          }
	;

transaction_access_mode:
	  RW_READ RW_ONLY
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("transaction_access_mode", "read-only");
          }
	| RW_READ RW_WRITE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("transaction_access_mode", "read-write");
          }
	;
	
isolation_level:
	  RW_ISOLATION RW_LEVEL level_of_isolation
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("level_of_isolation", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("isolation_level");
          }
	;

level_of_isolation:
	  RW_READ NRW_UNCOMMITTED
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("level_of_isolation", "read-uncommitted");
          }
	| RW_READ NRW_COMMITTED
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("level_of_isolation", "read-committed");
          }
	| NRW_REPEATABLE RW_READ
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("level_of_isolation", "repeatable-read");
          }
	| NRW_SERIALIZABLE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("level_of_isolation", "serializable");
          }
	;

diagnostics_size:
	  RW_DIAGNOSTICS RW_SIZE number_of_conditions
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("number_of_conditions", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("diagnostics_size");
          }
	;
	
number_of_conditions:
	  simple_value_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("simple_value_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("number_of_conditions");
          }
	;


/*
*  Section 13. Data manipulation.
*/

/*
*  Subsection 13.10:  <update statement: searched>.
*  Description:       Update rows of a table.
*/

update_statement.searched:
	  RW_UPDATE table_name RW_SET set_clause_list RW_WHERE search_condition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",       $2.yCode);
            ptApplication->bindCodeParameter ("set_clause_list",  $4.yCode);
            ptApplication->bindCodeParameter ("search_condition", $6.yCode);
            $$.yCode = ptApplication->getClauseCode ("update_statement.searched", "with-where");
          }
	| RW_UPDATE table_name RW_SET set_clause_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",      $2.yCode);
            ptApplication->bindCodeParameter ("set_clause_list", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("update_statement.searched", "without-where");
          }
	;


/*
*  Subsection 13.9:  <update statement: positioned>.
*  Description:      Update a row of a table.
*/

update_statement.positioned:
	  RW_UPDATE table_name
	  RW_SET set_clause_list
	  RW_WHERE RW_CURRENT RW_OF cursor_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",      $2.yCode);
            ptApplication->bindCodeParameter ("set_clause_list", $4.yCode);
            ptApplication->bindCodeParameter ("cursor_name",     $8.yCode);
            $$.yCode = ptApplication->getClauseCode ("update_statement.positioned");
          }
	;

set_clause_list:
	  set_clause_list COMMA set_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("set_clause_list", $1.yCode);
            ptApplication->bindCodeParameter ("set_clause",      $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("set_clause_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                       set_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("set_clause", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("set_clause_list", "monadic");
            $$.iItemCount = 1;
          }
	;
	
set_clause:
	  object_column EQUALS_OPERATOR update_source
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("object_column", $1.yCode);
            ptApplication->bindCodeParameter ("update_source", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("set_clause");
          }
	;

update_source:
	  value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("update_source", "value_expression");
          }
	| null_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("null_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("update_source", "null_specification");
          }
	| RW_DEFAULT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("update_source", "default");
          }
	;

object_column:
	  column_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("object_column");
          }
	;


/*
*  Subsection 13.8:  <insert statement>.
*  Description:      Create new rows in a table.
*/

insert_statement:
	  RW_INSERT RW_INTO table_name insert_columns_and_source
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",                $3.yCode);
            ptApplication->bindCodeParameter ("insert_columns_and_source", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("insert_statement");
          }
	;

insert_columns_and_source:
	  LEFT_PAREN insert_column_list RIGHT_PAREN query_expression
          {
            ptApplication->require (eSQL92_SR_13_8_P5, &$2, &$4);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("insert_column_list", $2.yCode);
            ptApplication->bindCodeParameter ("query_expression",   $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("insert_columns_and_source", "with-column-list");
          }
	|                                           query_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("insert_columns_and_source", "without-column-list");
          }
	| RW_DEFAULT RW_VALUES
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("insert_columns_and_source", "default-values");
          }
	;

insert_column_list:
	  column_name_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name_list", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("insert_column_list");
            $$.iItemCount = $1.iItemCount;
          }
	;


/*
*  Subsection 13.7:  <delete statement: searched>.
*  Description:      Delete rows of a table.
*/

delete_statement.searched:
	  RW_DELETE RW_FROM table_name RW_WHERE search_condition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",       $3.yCode);
            ptApplication->bindCodeParameter ("search_condition", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("delete_statement.searched", "with-where");
          }
	| RW_DELETE RW_FROM table_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("delete_statement.searched", "without-where");
          }
	;


/*
*  Subsection 13.6:  <delete statement: positioned>.
*  Description:       Delete a row of a table.
*/

delete_statement.positioned:
	  RW_DELETE RW_FROM table_name RW_WHERE RW_CURRENT RW_OF cursor_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",  $3.yCode);
            ptApplication->bindCodeParameter ("cursor_name", $7.yCode);
            $$.yCode = ptApplication->getClauseCode ("delete_statement.positioned");
          }
	;


/*
*  Subsection 13.5:  <select statement: single row>.
*  Description:      Retrieve values from a specified row of a table.
*/

select_statement.single_row:
	  RW_SELECT set_quantifier select_list RW_INTO select_target_list table_expression
          {
            ptApplication->require (eSQL92_SR_13_5_P1, &$3, &$5);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("set_quantifier",     $2.yCode);
            ptApplication->bindCodeParameter ("select_list",        $3.yCode);
            ptApplication->bindCodeParameter ("select_target_list", $5.yCode);
            ptApplication->bindCodeParameter ("table_expression",   $6.yCode);
            $$.yCode      = ptApplication->getClauseCode ("select_statement.single_row", "with-set-quantifier");
            $$.iItemCount = $3.iItemCount;
          }
	| RW_SELECT                select_list RW_INTO select_target_list table_expression
          {
            ptApplication->require (eSQL92_SR_13_5_P1, &$2, &$4);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("select_list",        $2.yCode);
            ptApplication->bindCodeParameter ("select_target_list", $4.yCode);
            ptApplication->bindCodeParameter ("table_expression",   $5.yCode);
            $$.yCode      = ptApplication->getClauseCode ("select_statement.single_row", "without-set-quantifier");
            $$.iItemCount = $2.iItemCount;
          }
	;

select_target_list:
	  select_target_list COMMA target_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("select_target_list",   $1.yCode);
            ptApplication->bindCodeParameter ("target_specification", $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("select_target_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                          target_specification 
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("target_specification", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("select_target_list", "monadic");
            $$.iItemCount = 1;
          }
	;

/*
*  Subsection 13.4:  <close statement>.
*  Description:      Close the given cursor        
*
*/

close_statement:
	  RW_CLOSE cursor_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("cursor_name", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("close_statement");
          }
	;


/*
*  Subsection 13.3:  <fetch statement>.
*  Description:      Position a cursor on a specified row of a table and retrieve
*                    values from that row.
*/

fetch_statement:
	  RW_FETCH fetch_orientation RW_FROM cursor_name RW_INTO fetch_target_list
          {
            ptApplication->require (eSQL92_LR_13_3_P2_A);
            ptApplication->require (eSQL92_LR_13_3_P2_B);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("fetch_orientation", $2.yCode);
            ptApplication->bindCodeParameter ("cursor_name",       $4.yCode);
            ptApplication->bindCodeParameter ("fetch_target_list", $6.yCode);
            $$.yCode = ptApplication->getClauseCode ("fetch_statement", "oriented");
          }
	| RW_FETCH                   RW_FROM cursor_name RW_INTO fetch_target_list
          {
            ptApplication->require (eSQL92_LR_13_3_P2_B);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("cursor_name",       $3.yCode);
            ptApplication->bindCodeParameter ("fetch_target_list", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("fetch_statement", "not-oriented");
          }
	| RW_FETCH                           cursor_name RW_INTO fetch_target_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("cursor_name",       $2.yCode);
            ptApplication->bindCodeParameter ("fetch_target_list", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("fetch_statement", "not-oriented");
          }
	;

fetch_orientation:
	  RW_NEXT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("fetch_orientation", "next");
          }
	| RW_PRIOR
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("fetch_orientation", "prior");
          }
	| RW_FIRST
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("fetch_orientation", "first");
          }
	| RW_LAST
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("fetch_orientation", "last");
          }
	| RW_ABSOLUTE simple_value_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("simple_value_specification", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("fetch_orientation", "absolute");
          }
	| RW_RELATIVE simple_value_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("simple_value_specification", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("fetch_orientation", "relative");
          }
	;
	
fetch_target_list:
	  fetch_target_list COMMA target_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("fetch_target_list",    $1.yCode);
            ptApplication->bindCodeParameter ("target_specification", $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("fetch_target_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                         target_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("target_specification", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("fetch_target_list", "monadic");
            $$.iItemCount = 1;
          }
	;

/*
*  Subsection 13.2:  <open statement>.
*  Description:      opens a cursor
*/

open_statement:
	  RW_OPEN cursor_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("cursor_name", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("open_statement");
          }
	;

/*
*  Subsection 13.1:  <declare cursor>.
*  Description:      Define a cursor.
*/

declare_cursor:
	  RW_DECLARE cursor_name RW_INSENSITIVE RW_SCROLL RW_CURSOR RW_FOR cursor_specification
          {
            ptApplication->require (eSQL92_LR_13_1_P1_A);
            ptApplication->require (eSQL92_LR_13_1_P2_A);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("cursor_name",          $2.yCode);
            ptApplication->bindCodeParameter ("cursor_specification", $7.yCode);
            $$.yCode = ptApplication->getClauseCode ("declare_cursor", "with-both");
          }
	| RW_DECLARE cursor_name RW_INSENSITIVE           RW_CURSOR RW_FOR cursor_specification
          {
            ptApplication->require (eSQL92_LR_13_1_P1_A);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("cursor_name",          $2.yCode);
            ptApplication->bindCodeParameter ("cursor_specification", $6.yCode);
            $$.yCode = ptApplication->getClauseCode ("declare_cursor", "without-scroll");
          }
	| RW_DECLARE cursor_name                RW_SCROLL RW_CURSOR RW_FOR cursor_specification
          {
            ptApplication->require (eSQL92_LR_13_1_P2_A);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("cursor_name",          $2.yCode);
            ptApplication->bindCodeParameter ("cursor_specification", $6.yCode);
            $$.yCode = ptApplication->getClauseCode ("declare_cursor", "without-insensitive");
          }
	| RW_DECLARE cursor_name                          RW_CURSOR RW_FOR cursor_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("cursor_name",          $2.yCode);
            ptApplication->bindCodeParameter ("cursor_specification", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("declare_cursor", "without-both");
          }
	;

cursor_specification:
	  query_expression order_by_clause updatability_clause
          {
            ptApplication->require (eSQL92_LR_13_1_P2_B);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_expression",    $1.yCode);
            ptApplication->bindCodeParameter ("order_by_clause",     $2.yCode);
            ptApplication->bindCodeParameter ("updatability_clause", $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("cursor_specification", "with-both");
            $$.iItemCount = $1.iItemCount;
          }
	| query_expression order_by_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_expression", $1.yCode);
            ptApplication->bindCodeParameter ("order_by_clause",  $2.yCode);
            $$.yCode      = ptApplication->getClauseCode ("cursor_specification", "without-updatability");
            $$.iItemCount = $1.iItemCount;
          }
	| query_expression                 updatability_clause
          {
            ptApplication->require (eSQL92_LR_13_1_P2_B);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_expression",    $1.yCode);
            ptApplication->bindCodeParameter ("updatability_clause", $2.yCode);
            $$.yCode      = ptApplication->getClauseCode ("cursor_specification", "without-order-by");
            $$.iItemCount = $1.iItemCount;
          }
	| query_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_expression", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("cursor_specification", "without-both");
            $$.iItemCount = $1.iItemCount;
          }
	;
	
updatability_clause:
	  RW_FOR RW_READ RW_ONLY
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("updatability_clause", "for-read-only");
          }
	| RW_FOR RW_UPDATE RW_OF column_name_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name_list", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("updatability_clause", "for-update-of");
          }
	| RW_FOR RW_UPDATE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("updatability_clause", "for-update");
          }
	;

order_by_clause:
	  RW_ORDER RW_BY sort_specification_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("sort_specification_list", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("order_by_clause");
          }
	;

sort_specification_list:
	  sort_specification_list COMMA sort_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("sort_specification_list", $1.yCode);
            ptApplication->bindCodeParameter ("sort_specification",      $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("sort_specification_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                               sort_specification 
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("sort_specification", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("sort_specification_list", "monadic");
            $$.iItemCount = 1;
          }
	;

sort_specification:
	  sort_key collate_clause ordering_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("sort_key",               $1.yCode);
            ptApplication->bindCodeParameter ("collate_clause",         $2.yCode);
            ptApplication->bindCodeParameter ("ordering_specification", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("sort_specification", "with-both");
          }
	| sort_key collate_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("sort_key",       $1.yCode);
            ptApplication->bindCodeParameter ("collate_clause", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("sort_specification", "without-ordering_specification");
          }
	| sort_key                ordering_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("sort_key",               $1.yCode);
            ptApplication->bindCodeParameter ("ordering_specification", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("sort_specification", "without-collate_clause");
          }
	| sort_key
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("sort_key", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("sort_specification", "without-both");
          }
	;

sort_key:
	  column_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("sort_key", "column_name");
          }
	| UNSIGNED_INTEGER
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unsigned_integer", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("sort_key", "unsigned_integer");
          }
	;

ordering_specification:
	  RW_ASC 
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("ordering_specification", "asc");
          }
	| RW_DESC
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("ordering_specification", "desc");
          }
	;


/*
*  Subsection 12.5:  <SQL procedure statement>.
*  Description:      Define all of the SQL-statements that are 
*                    <SQL procedure statement>s.
*/

SQL_procedure_statement:
	  SQL_schema_statement
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("SQL_schema_statement", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("SQL_procedure_statement", "SQL_schema_statement");
          }
	| SQL_data_statement
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("SQL_data_statement", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("SQL_procedure_statement", "SQL_data_statement");
          }
	| SQL_transaction_statement
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("SQL_transaction_statement", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("SQL_procedure_statement", "SQL_transaction_statement");
          }
	| SQL_connection_statement
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("SQL_connection_statement", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("SQL_procedure_statement", "SQL_connection_statement");
          }
	| SQL_session_statement
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("SQL_session_statement", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("SQL_procedure_statement", "SQL_session_statement");
          }
/* <DISABLED id="dynamic statements disabled"> */
/* 	| SQL_dynamic_statement */
/*           { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("SQL_dynamic_statement", $1.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("SQL_procedure_statement", "SQL_dynamic_statement"); */
/*           } */
/* </DISABLED> */
	| SQL_diagnostics_statement
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("SQL_diagnostics_statement", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("SQL_procedure_statement", "SQL_diagnostics_statement");
          }
	;

SQL_schema_statement:
	  SQL_schema_definition_statement
	| SQL_schema_manipulation_statement
	;

/*
*  Warning:  Character set, collation and translation statements
*            has been removed.
*/
SQL_schema_definition_statement:
	  schema_definition
	| table_definition
	| view_definition
	| grant_statement
	| domain_definition
	| assertion_definition
	;

/*
*  Warning:  Character set, collation and translation drop statements
*            has been removed.
*/
SQL_schema_manipulation_statement:
	  drop_schema_statement
	| alter_table_statement
	| drop_table_statement
	| drop_view_statement
	| revoke_statement
	| alter_domain_statement
	| drop_domain_statement
	| drop_assertion_statement
	;

SQL_data_statement:
        | open_statement
	| fetch_statement
        | close_statement
	| select_statement.single_row
	| SQL_data_change_statement
	;

SQL_data_change_statement:
	  delete_statement.searched
        | delete_statement.positioned
	| insert_statement
        | update_statement.positioned
	| update_statement.searched
	;

SQL_transaction_statement:
	  set_transaction_statement
	| set_constraints_mode_statement
	| commit_statement
	| rollback_statement
	;

SQL_connection_statement:
	  connect_statement
	| set_connection_statement
	| disconnect_statement
	;

SQL_session_statement:
	  set_catalog_statement
	| set_schema_statement
	| set_names_statement
	| set_session_authorization_identifier_statement
	| set_local_time_zone_statement
	;

/* <DISABLED id="dynamic statements disabled"> */
/* SQL_dynamic_statement: */
/* 	  system_descriptor_statement */
/* 	| prepare_statement */
/* 	| deallocate_prepared_statement */
/* 	| describe_statement */
/* 	| execute_statement */
/* 	| execute_immediate_statement */
/* 	| SQL_dynamic_data_statement */
/* 	; */

/* SQL_dynamic_data_statement: */
/* 	  allocate_cursor_statement */
/* 	| dynamic_open_statement */
/* 	| dynamic_fetch_statement */
/* 	| dynamic_close_statement */
/* 	| dynamic_delete_statement.positioned */
/* 	| dynamic_update_statement.positioned */
/* 	; */
/* </DISABLED> */

/* <DISABLED id="dynamic statements disabled"> */
/* system_descriptor_statement: */
/* 	  allocate_descriptor_statement */
/* 	| deallocate_descriptor_statement */
/* 	| set_descriptor_statement */
/* 	| get_descriptor_statement */
/* 	; */
/* </DISABLED> */

SQL_diagnostics_statement:
	  get_diagnostics_statement
	;


/*
*  Section 11. Schema definition and manipulation.
*/

/*
*  Subsection 11.37:  <revoke statement>.
*  Description:       Destroy privileges.
*/

revoke_statement:
	  RW_REVOKE RW_GRANT RW_OPTION RW_FOR privileges
	  RW_ON object_name RW_FROM grantee_list drop_behavior
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("privileges",    $5.yCode);
            ptApplication->bindCodeParameter ("object_name",   $7.yCode);
            ptApplication->bindCodeParameter ("grantee_list",  $9.yCode);
            ptApplication->bindCodeParameter ("drop_behavior", $10.yCode);
            $$.yCode = ptApplication->getClauseCode ("revoke_statement", "with-grant");
          }
	| RW_REVOKE                           privileges
	  RW_ON object_name RW_FROM grantee_list drop_behavior
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("privileges",    $2.yCode);
            ptApplication->bindCodeParameter ("object_name",   $4.yCode);
            ptApplication->bindCodeParameter ("grantee_list",  $6.yCode);
            ptApplication->bindCodeParameter ("drop_behavior", $7.yCode);
            $$.yCode = ptApplication->getClauseCode ("revoke_statement", "without-grant");
          }
	;


/*
*  Subsection 11.36:  <grant statement>.
*  Description:       Define privileges.
*/

/*
*  Auxiliary rule for:  grant_statement
*                       revoke_statement
*/
grantee_list:
	  grantee_list COMMA grantee
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("grantee_list", $1.yCode);
            ptApplication->bindCodeParameter ("grantee",      $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("grantee_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                    grantee
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("grantee", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("grantee_list", "monadic");
            $$.iItemCount = 1;
          }
	;
	
grant_statement:
	  RW_GRANT privileges RW_ON object_name
	  RW_TO grantee_list
          RW_WITH RW_GRANT RW_OPTION
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("privileges",   $2.yCode);
            ptApplication->bindCodeParameter ("object_name",  $4.yCode);
            ptApplication->bindCodeParameter ("grantee_list", $6.yCode);
            $$.yCode = ptApplication->getClauseCode ("grant_statement", "with-grant");
          }
	| RW_GRANT privileges RW_ON object_name
	  RW_TO grantee_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("privileges",   $2.yCode);
            ptApplication->bindCodeParameter ("object_name",  $4.yCode);
            ptApplication->bindCodeParameter ("grantee_list", $6.yCode);
            $$.yCode = ptApplication->getClauseCode ("grant_statement", "without-grant");
          }
	;
	
/*
*  Warning:  Character set has been removed.
*/
object_name:
	  RW_TABLE table_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("object_name", "table");
          }
	|          table_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("object_name", "table");
          }
	| RW_DOMAIN domain_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("object_name", "domain");
          }
	| RW_COLLATION collation_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("collation_name", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("object_name", "collation");
          }
	| RW_TRANSLATION translation_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("collation_name", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("object_name", "translation");
          }
	;


/*
*  Subsection 11.35:  <drop assertion statement>.
*  Description:       Destroy an assertion.
*/

drop_assertion_statement:
	  RW_DROP RW_ASSERTION constraint_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_name", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("drop_assertion_statement");
          }
	;


/*
*  Subsection 11.34:  <assertion definition>.
*  Description:       Specify an integrity constraint by means of an assertion and 
*                     specify the initial default time for checking the assertion.
*/

assertion_definition:
	  RW_CREATE RW_ASSERTION constraint_name assertion_check constraint_attributes
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_name",       $3.yCode);
            ptApplication->bindCodeParameter ("assertion_check",       $4.yCode);
            ptApplication->bindCodeParameter ("constraint_attributes", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("assertion_definition", "with-constraint_attributes");
          }
	| RW_CREATE RW_ASSERTION constraint_name assertion_check
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_name", $3.yCode);
            ptApplication->bindCodeParameter ("assertion_check", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("assertion_definition", "without-constraint_attributes");
          }
	;

assertion_check:
	  RW_CHECK LEFT_PAREN search_condition RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("search_condition", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("assertion_check");
          }
	;


/*
*  Subsection 11.27:  <drop domain statement>.
*  Description:       Destroy a domain.
*/

drop_domain_statement:
	  RW_DROP RW_DOMAIN domain_name drop_behavior
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name",   $3.yCode);
            ptApplication->bindCodeParameter ("drop_behavior", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("drop_domain_statement");
          }
	;
	

/*
*  Subsection 11.26:  <drop domain constraint definition>.
*  Description:       Destroy a constraint on a domain.
*/

drop_domain_constraint_definition:
	  RW_DROP RW_CONSTRAINT constraint_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_name", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("drop_domain_constraint_definition");
          }
	;


/*
*  Subsection 11.25:  <add domain constraint definition>.
*  Description:       Add a constraint to a domain.
*/

add_domain_constraint_definition:
	  RW_ADD domain_constraint
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_constraint", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("add_domain_constraint_definition");
          }
	;


/*
*  Subsection 11.24:  <drop domain default clause>.
*  Description:       Remove the default clause of a domain.
*/

drop_domain_default_clause:
	  RW_DROP RW_DEFAULT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("drop_domain_default_clause");
          }
	;


/*
*  Subsection 11.23:  <set domain default clause>.
*  Description:       Set the default value in a domain.
*/

set_domain_default_clause:
	  RW_SET default_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("default_clause", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("set_domain_default_clause");
          }
	;


/*
*  Subsection 11.22:  <alter domain statement>.
*  Description:       Change a domain and its definition.
*/

alter_domain_statement:
	  RW_ALTER RW_DOMAIN domain_name alter_domain_action
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name",         $3.yCode);
            ptApplication->bindCodeParameter ("alter_domain_action", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("alter_domain_statement");
          }
	;

alter_domain_action:
	  set_domain_default_clause
	| drop_domain_default_clause
	| add_domain_constraint_definition
	| drop_domain_constraint_definition
	;


/*
*  Subsection 11.21:  <domain definition>.
*  Description:       Define a domain.
*/

/*
*  Auxiliary rule for:  domain_definition
*/
domain_constraint_list:
	  domain_constraint_list domain_constraint
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_constraint_list", $1.yCode);
            ptApplication->bindCodeParameter ("domain_constraint",      $2.yCode);
            $$.yCode      = ptApplication->getClauseCode ("domain_constraint_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                        domain_constraint
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_constraint", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("domain_constraint_list", "monadic");
            $$.iItemCount = 1;
          }
	;
	
domain_definition:
	  RW_CREATE RW_DOMAIN domain_name RW_AS data_type
	  default_clause domain_constraint_list collate_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name",            $3.yCode);
            ptApplication->bindCodeParameter ("data_type",              $5.yCode);
            ptApplication->bindCodeParameter ("default_clause",         $6.yCode);
            ptApplication->bindCodeParameter ("domain_constraint_list", $7.yCode);
            ptApplication->bindCodeParameter ("collate_clause",         $8.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_definition", "with-all");
          }
	| RW_CREATE RW_DOMAIN domain_name       data_type
	  default_clause domain_constraint_list collate_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name",            $3.yCode);
            ptApplication->bindCodeParameter ("data_type",              $4.yCode);
            ptApplication->bindCodeParameter ("default_clause",         $5.yCode);
            ptApplication->bindCodeParameter ("domain_constraint_list", $6.yCode);
            ptApplication->bindCodeParameter ("collate_clause",         $7.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_definition", "with-all");
          }
	| RW_CREATE RW_DOMAIN domain_name RW_AS data_type
	  default_clause domain_constraint_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name",            $3.yCode);
            ptApplication->bindCodeParameter ("data_type",              $5.yCode);
            ptApplication->bindCodeParameter ("default_clause",         $6.yCode);
            ptApplication->bindCodeParameter ("domain_constraint_list", $7.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_definition", "without-collate_clause");
          }
	| RW_CREATE RW_DOMAIN domain_name       data_type
	  default_clause domain_constraint_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name",            $3.yCode);
            ptApplication->bindCodeParameter ("data_type",              $4.yCode);
            ptApplication->bindCodeParameter ("default_clause",         $5.yCode);
            ptApplication->bindCodeParameter ("domain_constraint_list", $6.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_definition", "without-collate_clause");
          }
	| RW_CREATE RW_DOMAIN domain_name RW_AS data_type
	                 domain_constraint_list collate_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name",            $3.yCode);
            ptApplication->bindCodeParameter ("data_type",              $5.yCode);
            ptApplication->bindCodeParameter ("domain_constraint_list", $6.yCode);
            ptApplication->bindCodeParameter ("collate_clause",         $7.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_definition", "without-default_clause");
          }
	| RW_CREATE RW_DOMAIN domain_name       data_type
	                 domain_constraint_list collate_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name",            $3.yCode);
            ptApplication->bindCodeParameter ("data_type",              $4.yCode);
            ptApplication->bindCodeParameter ("domain_constraint_list", $5.yCode);
            ptApplication->bindCodeParameter ("collate_clause",         $6.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_definition", "without-default_clause");
          }
	| RW_CREATE RW_DOMAIN domain_name RW_AS data_type
	                 domain_constraint_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name",            $3.yCode);
            ptApplication->bindCodeParameter ("data_type",              $5.yCode);
            ptApplication->bindCodeParameter ("domain_constraint_list", $6.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_definition", "without-default_clause-without-collate_clause");
          }
	| RW_CREATE RW_DOMAIN domain_name       data_type
	                 domain_constraint_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name",            $3.yCode);
            ptApplication->bindCodeParameter ("data_type",              $4.yCode);
            ptApplication->bindCodeParameter ("domain_constraint_list", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_definition", "without-default_clause-without-collate_clause");
          }
	| RW_CREATE RW_DOMAIN domain_name RW_AS data_type
	  default_clause                        collate_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name",    $3.yCode);
            ptApplication->bindCodeParameter ("data_type",      $5.yCode);
            ptApplication->bindCodeParameter ("default_clause", $6.yCode);
            ptApplication->bindCodeParameter ("collate_clause", $7.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_definition", "without-domain_constraint_list");
          }
	| RW_CREATE RW_DOMAIN domain_name       data_type
	  default_clause                        collate_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name",    $3.yCode);
            ptApplication->bindCodeParameter ("data_type",      $4.yCode);
            ptApplication->bindCodeParameter ("default_clause", $5.yCode);
            ptApplication->bindCodeParameter ("collate_clause", $6.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_definition", "without-domain_constraint_list");
          }
	| RW_CREATE RW_DOMAIN domain_name RW_AS data_type
	  default_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name",    $3.yCode);
            ptApplication->bindCodeParameter ("data_type",      $5.yCode);
            ptApplication->bindCodeParameter ("default_clause", $6.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_definition", "without-domain_constraint_list-without-collate_clause");
          }
	| RW_CREATE RW_DOMAIN domain_name       data_type
	  default_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name",    $3.yCode);
            ptApplication->bindCodeParameter ("data_type",      $4.yCode);
            ptApplication->bindCodeParameter ("default_clause", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_definition", "without-domain_constraint_list-without-collate_clause");
          }
	| RW_CREATE RW_DOMAIN domain_name RW_AS data_type
	                                        collate_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name",    $3.yCode);
            ptApplication->bindCodeParameter ("data_type",      $5.yCode);
            ptApplication->bindCodeParameter ("collate_clause", $6.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_definition", "without-default_clause-without-domain_constraint_list");
          }
	| RW_CREATE RW_DOMAIN domain_name       data_type
	                                        collate_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name",    $3.yCode);
            ptApplication->bindCodeParameter ("data_type",      $4.yCode);
            ptApplication->bindCodeParameter ("collate_clause", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_definition", "without-default_clause-without-domain_constraint_list");
          }
	| RW_CREATE RW_DOMAIN domain_name RW_AS data_type
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name", $3.yCode);
            ptApplication->bindCodeParameter ("data_type",   $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_definition", "without-all");
          }
	| RW_CREATE RW_DOMAIN domain_name       data_type
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name", $3.yCode);
            ptApplication->bindCodeParameter ("data_type",   $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_definition", "without-all");
          }
	;

domain_constraint:
	  constraint_name_definition check_constraint_definition constraint_attributes
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_name_definition",  $1.yCode);
            ptApplication->bindCodeParameter ("check_constraint_definition", $2.yCode);
            ptApplication->bindCodeParameter ("constraint_attributes",       $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_constraint", "with-both");
          }
	| constraint_name_definition check_constraint_definition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_name_definition",  $1.yCode);
            ptApplication->bindCodeParameter ("check_constraint_definition", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_constraint", "without-constraint_attributes");
          }
	|                            check_constraint_definition constraint_attributes
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("check_constraint_definition", $1.yCode);
            ptApplication->bindCodeParameter ("constraint_attributes",       $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_constraint", "without-constraint_name_definition");
          }
	|                            check_constraint_definition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("check_constraint_definition", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_constraint", "without-both");
          }
	;


/*
*  Subsection 11.20:  <drop view statement>.
*  Description:       Destroy a view.
*/

drop_view_statement:
	  RW_DROP RW_VIEW table_name drop_behavior
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",    $3.yCode);
            ptApplication->bindCodeParameter ("drop_behavior", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("drop_view_statement");
          }
	;


/*
*  Subsection 11.19:  <view definition>.
*  Description:       Define a viewed table.
*/

view_definition:
	  RW_CREATE RW_VIEW table_name LEFT_PAREN view_column_list RIGHT_PAREN
	  RW_AS query_expression
	  RW_WITH levels_clause RW_CHECK RW_OPTION
          {
            ptApplication->require (eSQL92_SR_11_19_P10, &$5, &$8);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",       $3.yCode);
            ptApplication->bindCodeParameter ("view_column_list", $5.yCode);
            ptApplication->bindCodeParameter ("query_expression", $8.yCode);
            ptApplication->bindCodeParameter ("levels_clause",    $10.yCode);
            $$.yCode = ptApplication->getClauseCode ("view_definition", "with-all");
          }
	| RW_CREATE RW_VIEW table_name LEFT_PAREN view_column_list RIGHT_PAREN
	  RW_AS query_expression
	  RW_WITH RW_CHECK RW_OPTION
          {
            ptApplication->require (eSQL92_SR_11_19_P10, &$5, &$8);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",       $3.yCode);
            ptApplication->bindCodeParameter ("view_column_list", $5.yCode);
            ptApplication->bindCodeParameter ("query_expression", $8.yCode);
            $$.yCode = ptApplication->getClauseCode ("view_definition", "without-levels_clause");
          }
	| RW_CREATE RW_VIEW table_name LEFT_PAREN view_column_list RIGHT_PAREN
	  RW_AS query_expression
          {
            ptApplication->require (eSQL92_SR_11_19_P10, &$5, &$8);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",       $3.yCode);
            ptApplication->bindCodeParameter ("view_column_list", $5.yCode);
            ptApplication->bindCodeParameter ("query_expression", $8.yCode);
            $$.yCode = ptApplication->getClauseCode ("view_definition", "without-with");
          }
	| RW_CREATE RW_VIEW table_name
	  RW_AS query_expression
	  RW_WITH levels_clause RW_CHECK RW_OPTION
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",       $3.yCode);
            ptApplication->bindCodeParameter ("query_expression", $5.yCode);
            ptApplication->bindCodeParameter ("levels_clause",    $7.yCode);
            $$.yCode = ptApplication->getClauseCode ("view_definition", "without-view_column_list");
          }
	| RW_CREATE RW_VIEW table_name
	  RW_AS query_expression
	  RW_WITH RW_CHECK RW_OPTION
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",       $3.yCode);
            ptApplication->bindCodeParameter ("query_expression", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("view_definition", "without-view_column_list-without-levels_clause");
          }
	| RW_CREATE RW_VIEW table_name
	  RW_AS query_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",       $3.yCode);
            ptApplication->bindCodeParameter ("query_expression", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("view_definition", "without-view_column_list-without-with");
          }
	;
	
levels_clause:
	  RW_CASCADED 
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("levels_clause", "cascaded");
          }
	| RW_LOCAL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("levels_clause", "local");
          }
	;

view_column_list:
	  column_name_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name_list", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("view_column_list");
            $$.iItemCount = $1.iItemCount;
          }
	;


/*
*  Subsection 11.18:  <drop table statement>.
*  Description:       Destroy a table.
*/

drop_table_statement:
	  RW_DROP RW_TABLE table_name drop_behavior
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",    $3.yCode);
            ptApplication->bindCodeParameter ("drop_behavior", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("drop_table_statement");
          }
	;


/*
*  Subsection 11.17:  <drop table constraint definition>.
*  Description:       Destroy a constraint on a table.
*/

drop_table_constraint_definition:
	  RW_DROP RW_CONSTRAINT constraint_name drop_behavior
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_name", $3.yCode);
            ptApplication->bindCodeParameter ("drop_behavior",   $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("drop_table_constraint_definition");
          }
	;


/*
*  Subsection 11.16:  <add table constraint definition>.
*  Description:       Add a constraint to a table.
*/

add_table_constraint_definition:
	  RW_ADD table_constraint_definition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_constraint_definition", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("add_table_constraint_definition");
          }
	;


/*
*  Subsection 11.15:  <drop column definition>.
*  Description:       Destroy a column.
*/

drop_column_definition:
	  RW_DROP RW_COLUMN column_name drop_behavior
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",   $3.yCode);
            ptApplication->bindCodeParameter ("drop_behavior", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("drop_column_definition");
          }
	| RW_DROP           column_name drop_behavior
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",   $2.yCode);
            ptApplication->bindCodeParameter ("drop_behavior", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("drop_column_definition");
          }
	;
	

/*
*  Subsection 11.14:  <drop column default clause>.
*  Description:       Drop the default clause from a column.
*/

drop_column_default_clause:
	  RW_DROP RW_DEFAULT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("drop_column_default_clause");
          }
	;


/*
*  Subsection 11.13:  <set column default clause>.
*  Description:       Set the default clause for a column.
*/

set_column_default_clause:
	  RW_SET default_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("default_clause", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("set_column_default_clause");
          }
	;
	

/*
*  Subsection 11.12:  <alter column definition>.
*  Description:       Change a column and its definition.
*/

alter_column_definition:
	  RW_ALTER RW_COLUMN column_name alter_column_action
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",         $3.yCode);
            ptApplication->bindCodeParameter ("alter_column_action", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("alter_column_definition");
          }
	| RW_ALTER           column_name alter_column_action
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",         $2.yCode);
            ptApplication->bindCodeParameter ("alter_column_action", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("alter_column_definition");
          }
	;

alter_column_action:
	  set_column_default_clause
	| drop_column_default_clause
	;


/*
*  Subsection 11.11:  <add column definition>.
*  Description:       Add a column to a table.
*/

add_column_definition:
	  RW_ADD RW_COLUMN column_definition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_definition", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("add_column_definition");
          }
	| RW_ADD           column_definition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_definition", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("add_column_definition");
          }
	;


/*
*  Subsection 11.10:  <alter table statement>.
*  Description:       Change the definition of a table.
*/

alter_table_statement:
	  RW_ALTER RW_TABLE table_name alter_table_action
          {
            ptApplication->require (eSQL92_LR_11_10_P2);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",         $3.yCode);
            ptApplication->bindCodeParameter ("alter_table_action", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("alter_table_statement");
          }
	;
	
alter_table_action:
	  add_column_definition
	| alter_column_definition
	| drop_column_definition
	| add_table_constraint_definition
	| drop_table_constraint_definition
	;


/*
*  Subsection 11.9:  <check constraint definition>.
*  Description:      Specify a condition for the SQL-data.
*/

check_constraint_definition:
	  RW_CHECK LEFT_PAREN search_condition RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("search_condition", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("check_constraint_definition");
          }
	;


/*
*  Subsection 11.8:  <referential constraint definition>.
*  Description:      Specify a referential constraint.
*/

referential_constraint_definition:
	  RW_FOREIGN RW_KEY LEFT_PAREN referencing_columns RIGHT_PAREN references_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("referencing_columns",      $4.yCode);
            ptApplication->bindCodeParameter ("references_specification", $6.yCode);
            $$.yCode = ptApplication->getClauseCode ("referential_constraint_definition");
          }
	;
	
references_specification:
	  RW_REFERENCES referenced_table_and_columns RW_MATCH match_type referential_triggered_action
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("referenced_table_and_columns", $2.yCode);
            ptApplication->bindCodeParameter ("match_type",                   $4.yCode);
            ptApplication->bindCodeParameter ("referential_triggered_action", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("references_specification", "with-both");
          }
	| RW_REFERENCES referenced_table_and_columns RW_MATCH match_type
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("referenced_table_and_columns", $2.yCode);
            ptApplication->bindCodeParameter ("match_type",                   $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("references_specification", "without-referential_triggered_action");
          }
	| RW_REFERENCES referenced_table_and_columns                     referential_triggered_action
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("referenced_table_and_columns", $2.yCode);
            ptApplication->bindCodeParameter ("referential_triggered_action", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("references_specification", "without-match");
          }
	| RW_REFERENCES referenced_table_and_columns
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("referenced_table_and_columns", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("references_specification", "without-both");
          }
	;

match_type:
	  RW_FULL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("match_type", "full");
          }
	| RW_PARTIAL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("match_type", "partial");
          }
	;

referencing_columns:
	  reference_column_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("reference_column_list", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("referencing_columns");
            $$.iItemCount = $1.iItemCount;
          }
	;

referenced_table_and_columns:
	  table_name LEFT_PAREN reference_column_list RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",            $1.yCode);
            ptApplication->bindCodeParameter ("reference_column_list", $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("referenced_table_and_columns", "with-reference_column_list");
            $$.iItemCount = $3.iItemCount;
          }
	| table_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("referenced_table_and_columns", "without-reference_column_list");
          }
	;

reference_column_list:
	  column_name_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name_list", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("reference_column_list");
            $$.iItemCount = $1.iItemCount;
          }
	;

referential_triggered_action:
	  update_rule delete_rule
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("update_rule", $1.yCode);
            ptApplication->bindCodeParameter ("delete_rule", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("referential_triggered_action", "with-both");
          }
	| delete_rule update_rule
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("delete_rule", $1.yCode);
            ptApplication->bindCodeParameter ("update_rule", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("referential_triggered_action", "with-both");
          }
	| update_rule
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("update_rule", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("referential_triggered_action", "with-update_rule");
          }
	| delete_rule
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("delete_rule", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("referential_triggered_action", "with-delete_rule");
          }
	;
	
update_rule:
	  RW_ON RW_UPDATE referential_action
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("referential_action", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("update_rule");
          }
	;

delete_rule:
	  RW_ON RW_DELETE referential_action
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("referential_action", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("delete_rule");
          }
	;

referential_action:
	  RW_CASCADE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("referential_action", "cascade");
          }
	| RW_SET RW_NULL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("referential_action", "set-null");
          }
	| RW_SET RW_DEFAULT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("referential_action", "set-default");
          }
	| RW_NO RW_ACTION
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("referential_action", "no-action");
          }
	;


/*
*  Subsection 11.7:  <unique constraint definition>.
*  Description:      Specify a uniqueness constraint for a table.
*/

unique_constraint_definition:
	  unique_specification LEFT_PAREN unique_column_list RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unique_specification", $1.yCode);
            ptApplication->bindCodeParameter ("unique_column_list",   $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("unique_constraint_definition");
            $$.iItemCount = $3.iItemCount;
          }
	;

unique_specification:
	  RW_UNIQUE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("unique_specification", "unique");
          }
	| RW_PRIMARY RW_KEY
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("unique_specification", "primary-key");
          }
	;

unique_column_list:
	  column_name_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name_list", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("unique_column_list");
            $$.iItemCount = $1.iItemCount;
          }
	;


/*
*  Subsection 11.6:  <table constraint definition>.
*  Description:      Specify an integrity constraint.
*/

table_constraint_definition:
	  constraint_name_definition table_constraint constraint_attributes
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_name_definition", $1.yCode);
            ptApplication->bindCodeParameter ("table_constraint",           $2.yCode);
            ptApplication->bindCodeParameter ("constraint_attributes",      $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_constraint_definition", "with-both");
          }
	| constraint_name_definition table_constraint
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_name_definition", $1.yCode);
            ptApplication->bindCodeParameter ("table_constraint",           $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_constraint_definition", "without-constraint_attributes");
          }
	|                            table_constraint constraint_attributes
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_constraint",      $1.yCode);
            ptApplication->bindCodeParameter ("constraint_attributes", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_constraint_definition", "without-constraint_name_definition");
          }
	|                            table_constraint
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_constraint", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_constraint_definition", "without-both");
          }
	;

table_constraint:
	  unique_constraint_definition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unique_constraint_definition", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_constraint", "unique_constraint_definition");
          }
	| referential_constraint_definition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("referential_constraint_definition", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_constraint", "referential_constraint_definition");
          }
	| check_constraint_definition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("check_constraint_definition", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_constraint", "check_constraint_definition");
          }
	;


/*
*  Subsection 11.5:  <default clause>.
*  Description:      Specify the default for a column or domain.
*/

default_clause:
	  RW_DEFAULT default_option
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("default_option", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("default_clause");
          }
	;

default_option:
	  literal
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("default_option", "literal");
          }
	| datetime_value_function
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("datetime_value_function", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("default_option", "datetime_value_function");
          }
	| RW_USER
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("default_option", "user");
          }
	| RW_CURRENT_USER
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("default_option", "current_user");
          }
	| RW_SESSION_USER
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("default_option", "session_user");
          }
	| RW_SYSTEM_USER
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("default_option", "system_user");
          }
	| RW_NULL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("default_option", "null");
          }
	;


/*
*  Subsection 11.4:  <column definition>.
*  Description:      Define a column of a table.
*/

/*
*  Auxiliary rule for:  column_definition
*/
column_constraint_definition_list:
	  column_constraint_definition_list column_constraint_definition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_constraint_definition_list", $1.yCode);
            ptApplication->bindCodeParameter ("column_constraint_definition",      $2.yCode);
            $$.yCode      = ptApplication->getClauseCode ("column_constraint_definition_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                                   column_constraint_definition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_constraint_definition", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("column_constraint_definition_list", "monadic");
            $$.iItemCount = 1;
          }
	;
	
column_definition:
	  column_name data_type default_clause column_constraint_definition_list collate_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",                       $1.yCode);
            ptApplication->bindCodeParameter ("data_type",                         $2.yCode);
            ptApplication->bindCodeParameter ("default_clause",                    $3.yCode);
            ptApplication->bindCodeParameter ("column_constraint_definition_list", $4.yCode);
            ptApplication->bindCodeParameter ("collate_clause",                    $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_definition", "with-data_type-with-all");
          }
	| column_name data_type default_clause column_constraint_definition_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",                       $1.yCode);
            ptApplication->bindCodeParameter ("data_type",                         $2.yCode);
            ptApplication->bindCodeParameter ("default_clause",                    $3.yCode);
            ptApplication->bindCodeParameter ("column_constraint_definition_list", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_definition", "with-data_type-without-collate_clause");
          }
	| column_name data_type                column_constraint_definition_list collate_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",                       $1.yCode);
            ptApplication->bindCodeParameter ("data_type",                         $2.yCode);
            ptApplication->bindCodeParameter ("column_constraint_definition_list", $3.yCode);
            ptApplication->bindCodeParameter ("collate_clause",                    $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_definition", "with-data_type-without-default_clause");
          }
	| column_name data_type                column_constraint_definition_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",                       $1.yCode);
            ptApplication->bindCodeParameter ("data_type",                         $2.yCode);
            ptApplication->bindCodeParameter ("column_constraint_definition_list", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_definition", "with-data_type-without-default_clause-without-collate_clause");
          }
	| column_name domain_name default_clause column_constraint_definition_list collate_clause
          {
            ptApplication->require (eSQL92_LR_11_4_P2_A);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",                       $1.yCode);
            ptApplication->bindCodeParameter ("domain_name",                       $2.yCode);
            ptApplication->bindCodeParameter ("default_clause",                    $3.yCode);
            ptApplication->bindCodeParameter ("column_constraint_definition_list", $4.yCode);
            ptApplication->bindCodeParameter ("collate_clause",                    $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_definition", "with-domain_name-with-all");
          }
	| column_name domain_name default_clause column_constraint_definition_list
          {
            ptApplication->require (eSQL92_LR_11_4_P2_A);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",                       $1.yCode);
            ptApplication->bindCodeParameter ("domain_name",                       $2.yCode);
            ptApplication->bindCodeParameter ("default_clause",                    $3.yCode);
            ptApplication->bindCodeParameter ("column_constraint_definition_list", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_definition", "with-domain_name-without-collate_clause");
          }
        | column_name domain_name                column_constraint_definition_list collate_clause
          {
            ptApplication->require (eSQL92_LR_11_4_P2_A);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",                       $1.yCode);
            ptApplication->bindCodeParameter ("domain_name",                       $2.yCode);
            ptApplication->bindCodeParameter ("column_constraint_definition_list", $3.yCode);
            ptApplication->bindCodeParameter ("collate_clause",                    $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_definition", "with-domain_name-without-default_clause");
          }
	| column_name domain_name                column_constraint_definition_list
          {
            ptApplication->require (eSQL92_LR_11_4_P2_A);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",                       $1.yCode);
            ptApplication->bindCodeParameter ("domain_name",                       $2.yCode);
            ptApplication->bindCodeParameter ("column_constraint_definition_list", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_definition", "with-domain_name-without-default_clause-without-collate_clause");
          }
	| column_name data_type default_clause                                     collate_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",    $1.yCode);
            ptApplication->bindCodeParameter ("data_type",      $2.yCode);
            ptApplication->bindCodeParameter ("default_clause", $3.yCode);
            ptApplication->bindCodeParameter ("collate_clause", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_definition", "with-data_type-without-column_constraint_definition_list");
          }
	| column_name data_type default_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",    $1.yCode);
            ptApplication->bindCodeParameter ("data_type",      $2.yCode);
            ptApplication->bindCodeParameter ("default_clause", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_definition", "with-data_type-without-column_constraint_definition_list-without-collate_clause");
          }
	| column_name data_type                                                  collate_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",    $1.yCode);
            ptApplication->bindCodeParameter ("data_type",      $2.yCode);
            ptApplication->bindCodeParameter ("collate_clause", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_definition", "with-data_type-without-default_clause-without-column_constraint_definition_list");
          }
	| column_name data_type
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name", $1.yCode);
            ptApplication->bindCodeParameter ("data_type",   $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_definition", "with-data_type-without-all");
          }
	| column_name domain_name default_clause                                 collate_clause
          {
            ptApplication->require (eSQL92_LR_11_4_P2_A);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",    $1.yCode);
            ptApplication->bindCodeParameter ("domain_name",    $2.yCode);
            ptApplication->bindCodeParameter ("default_clause", $3.yCode);
            ptApplication->bindCodeParameter ("collate_clause", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_definition", "with-domain_name-without-column_constraint_definition_list");
          }
	| column_name domain_name default_clause
          {
            ptApplication->require (eSQL92_LR_11_4_P2_A);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",    $1.yCode);
            ptApplication->bindCodeParameter ("domain_name",    $2.yCode);
            ptApplication->bindCodeParameter ("default_clause", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_definition", "with-domain_name-without-column_constraint_definition_list-without-collate_clause");
          }
        | column_name domain_name                                                  collate_clause
          {
            ptApplication->require (eSQL92_LR_11_4_P2_A);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name",    $1.yCode);
            ptApplication->bindCodeParameter ("domain_name",    $2.yCode);
            ptApplication->bindCodeParameter ("collate_clause", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_definition", "with-domain_name-without-default_clause-without-column_constraint_definition_list");
          }
	| column_name domain_name
          {
            ptApplication->require (eSQL92_LR_11_4_P2_A);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name", $1.yCode);
            ptApplication->bindCodeParameter ("domain_name", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_definition", "with-domain_name-without-all");
          }
	;

column_constraint_definition:
	  constraint_name_definition column_constraint constraint_attributes
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_name_definition", $1.yCode);
            ptApplication->bindCodeParameter ("column_constraint",          $2.yCode);
            ptApplication->bindCodeParameter ("constraint_attributes",      $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_constraint_definition", "with-both");
          }
	| constraint_name_definition column_constraint
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_name_definition", $1.yCode);
            ptApplication->bindCodeParameter ("column_constraint",          $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_constraint_definition", "without-constraint_attributes");
          }
	|                            column_constraint constraint_attributes
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_constraint",     $1.yCode);
            ptApplication->bindCodeParameter ("constraint_attributes", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_constraint_definition", "without-constraint_name_definition");
          }
	|                            column_constraint
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_constraint", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_constraint_definition", "without-both");
          }
	;

column_constraint:
	  RW_NOT RW_NULL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("column_constraint", "not-null");
          }
	| unique_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unique_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_constraint", "unique_specification");
          }
	| references_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("references_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_constraint", "references_specification");
          }
	| check_constraint_definition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("check_constraint_definition", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_constraint", "check_constraint_definition");
          }
	;


/*
*  Subsection 11.3:  <table definition>.
*  Description:      Define a persistent base table, a created local temporary table,
*                    or a global temporary table.
*/

/*
*  Auxiliary rule for:  table_definition
*/
first_part_table_definition:
	  RW_GLOBAL RW_TEMPORARY
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("first_part_table_definition", "global");
          }
	| RW_LOCAL  RW_TEMPORARY
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("first_part_table_definition", "local");
          }
	;
	  
/*
*  Auxiliary rule for:  table_definition
*/
last_part_table_definition:
	  RW_ON RW_COMMIT RW_DELETE   RW_ROWS
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("last_part_table_definition", "delete");
          }
	| RW_ON RW_COMMIT RW_PRESERVE RW_ROWS
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("last_part_table_definition", "preserve");
          }
	;
	  
table_definition:
	  RW_CREATE first_part_table_definition
	  RW_TABLE table_name
	  table_element_list
	  last_part_table_definition
          {
            ptApplication->require (eSQL92_SR_11_3_P5, &$5);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("first_part_table_definition", $2.yCode);
            ptApplication->bindCodeParameter ("table_name",                  $4.yCode);
            ptApplication->bindCodeParameter ("table_element_list",          $5.yCode);
            ptApplication->bindCodeParameter ("last_part_table_definition",  $6.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_definition", "with-first_part_table_definition-with-last_part_table_definition");
          }
        | RW_CREATE first_part_table_definition
	  RW_TABLE table_name
	  table_element_list
          {
            ptApplication->require (eSQL92_SR_11_3_P5, &$5);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("first_part_table_definition", $2.yCode);
            ptApplication->bindCodeParameter ("table_name",                  $4.yCode);
            ptApplication->bindCodeParameter ("table_element_list",          $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_definition", "with-first_part_table_definition-without-last_part_table_definition");
          }
        | RW_CREATE RW_TABLE table_name table_element_list 
          {
            ptApplication->require (eSQL92_SR_11_3_P5, &$4);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",         $3.yCode);
            ptApplication->bindCodeParameter ("table_element_list", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_definition", "without-first_part_table_definition-without-last_part_table_definition");
          }
	;

/*
*  Auxiliary rule for:  table_element_list
*/
middle_part_table_element_list:
	  middle_part_table_element_list COMMA table_element
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("middle_part_table_element_list", $1.yCode);
            ptApplication->bindCodeParameter ("table_element",                  $3.yCode);
            $$.yCode                = ptApplication->getClauseCode ("middle_part_table_element_list", "n-adic");
            $$.iItemCount           = $1.iItemCount + 1;
            $$.gHasColumnDefinition = ( $1.gHasColumnDefinition || $3.gHasColumnDefinition );
          }
	|                                      table_element
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_element", $1.yCode);
            $$.yCode                = ptApplication->getClauseCode ("middle_part_table_element_list", "monadic");
            $$.iItemCount           = 1;
            $$.gHasColumnDefinition = $1.gHasColumnDefinition;
          }
	;
	 
table_element_list:
	  LEFT_PAREN middle_part_table_element_list RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("middle_part_table_element_list", $2.yCode);
            $$.yCode                = ptApplication->getClauseCode ("table_element_list");
            $$.iItemCount           = $2.iItemCount;
            $$.gHasColumnDefinition = $2.gHasColumnDefinition;
          }
	;

table_element:
	  column_definition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_definition", $1.yCode);
            $$.yCode                = ptApplication->getClauseCode ("table_element", "column_definition");
            $$.gHasColumnDefinition = true;            
          }
	| table_constraint_definition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_constraint_definition", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_element", "table_constraint_definition");
          }
	;


/*
*  Subsection 11.2:  <drop schema statement>.
*  Description:      Destroy a schema.
*/

drop_schema_statement:
	  RW_DROP RW_SCHEMA schema_name drop_behavior
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("schema_name",   $3.yCode);
            ptApplication->bindCodeParameter ("drop_behavior", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("drop_schema_statement");
          }
	;

drop_behavior:
	  RW_CASCADE 
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("drop_behavior", "cascade");
          }
	| RW_RESTRICT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("drop_behavior", "restrict");
          }
	;


/*
*  Subsection 11.1:  <schema definition>.
*  Description:      Define a schema.
*/

/*
*  Auxiliary rule for:  schema_definition
*/
schema_element_list:
	  schema_element_list schema_element
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("schema_element_list", $1.yCode);
            ptApplication->bindCodeParameter ("schema_element",      $2.yCode);
            $$.yCode      = ptApplication->getClauseCode ("schema_element_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                     schema_element
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("schema_element", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("schema_element_list", "monadic");
            $$.iItemCount = 1;
          }
	;

/*
*  Warning:  Character set specification has been removed.
*/
schema_definition:
	  RW_CREATE RW_SCHEMA schema_name_clause schema_element_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("schema_name_clause",  $3.yCode);
            ptApplication->bindCodeParameter ("schema_element_list", $4.yCode);
            $$.yCode      = ptApplication->getClauseCode ("schema_definition", "with-schema_element_list");
            $$.iItemCount = $4.iItemCount;
          }
	| RW_CREATE RW_SCHEMA schema_name_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("schema_name_clause", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("schema_definition", "without-schema_element_list");
          }
	;

schema_name_clause:
	  schema_name RW_AUTHORIZATION schema_authorization_identifier
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("schema_name",                     $1.yCode);
            ptApplication->bindCodeParameter ("schema_authorization_identifier", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("schema_name_clause", "with-both");
          }
	| schema_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("schema_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("schema_name_clause", "without-authorization");
          }
	|             RW_AUTHORIZATION schema_authorization_identifier
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("schema_authorization_identifier", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("schema_name_clause", "without-schema_name");
          }
	;

schema_authorization_identifier:
	  authorization_identifier
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("authorization_identifier", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("schema_authorization_identifier");
          }
	;

schema_element:
	  domain_definition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_definition", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("schema_element", "domain_definition");
          }
	| table_definition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_definition", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("schema_element", "table_definition");
          }
	| view_definition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("view_definition", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("schema_element", "view_definition");
          }
	| grant_statement
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("grant_statement", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("schema_element", "grant_statement");
          }
	| assertion_definition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("assertion_definition", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("schema_element", "assertion_definition");
          }
	;


/*
*  Section 10. Additional common elements.
*/

/*
*  Subsection 10.6:  <constraint name definition> and <constraint attributes>.
*  Description:      Specify the name of a constraint and its attributes.
*/

constraint_name_definition:
	  RW_CONSTRAINT constraint_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_name", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("constraint_name_definition");
          }
	;

constraint_attributes:
	  constraint_check_time RW_NOT RW_DEFERRABLE
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_check_time", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("constraint_attributes", "with-constraint_check_time-not-deferrable");
          }
	| constraint_check_time        RW_DEFERRABLE
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_check_time", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("constraint_attributes", "with-constraint_check_time-deferrable");
          }
	| constraint_check_time
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_check_time", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("constraint_attributes", "with-constraint_check_time");
          }
	| RW_NOT RW_DEFERRABLE constraint_check_time
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_check_time", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("constraint_attributes", "with-constraint_check_time-not-deferrable");
          }
	| RW_NOT RW_DEFERRABLE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("constraint_attributes", "without-constraint_check_time-not-deferrable");
          }
	|        RW_DEFERRABLE constraint_check_time
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("constraint_check_time", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("constraint_attributes", "with-constraint_check_time-deferrable");
          }
	|        RW_DEFERRABLE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("constraint_attributes", "without-constraint_check_time-deferrable");
          }
	;
	
constraint_check_time:
	  RW_INITIALLY RW_DEFERRED
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("constraint_check_time", "initially-deferred");
          }
	| RW_INITIALLY RW_IMMEDIATE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("constraint_check_time", "initially-immediate");
          }
	;


/*
*  Subsection 10.5:  <collate clause>.
*  Description:      Specify a collating sequence.
*/

collate_clause:
	  RW_COLLATE collation_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("collation_name", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("collate_clause");
          }
	;


/*
*  Subsection 10.3:  <privileges>.
*  Description:      Specify privileges.
*/

privileges:
	  RW_ALL RW_PRIVILEGES
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("privileges", "all-privileges");
          }
	| action_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("action_list", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("privileges", "action_list");
            $$.iItemCount = $1.iItemCount;
          }
	;

action_list:
	  action_list COMMA action
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("action_list", $1.yCode);
            ptApplication->bindCodeParameter ("action",      $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("action_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                   action
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("action", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("action_list", "monadic");
            $$.iItemCount = 1;
          }
	;

action:
	  RW_SELECT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("action", "select");
          }
	| RW_DELETE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("action", "delete");
          }
	| RW_INSERT     LEFT_PAREN privilege_column_list RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("privilege_column_list", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("action", "insert-with-privilege_column_list");
          }
	| RW_INSERT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("action", "insert");
          }
	| RW_UPDATE     LEFT_PAREN privilege_column_list RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("privilege_column_list", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("action", "update-with-privilege_column_list");
          }
	| RW_UPDATE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("action", "update");
          }
	| RW_REFERENCES LEFT_PAREN privilege_column_list RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("privilege_column_list", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("action", "references-with-privilege_column_list");
          }
	| RW_REFERENCES
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("action", "references");
          }
	| RW_USAGE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("action", "usage");
          }
	;

privilege_column_list:
	  column_name_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name_list", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("privilege_column_list");
            $$.iItemCount = $1.iItemCount;
          }
	;

grantee:
	  RW_PUBLIC
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("grantee", "public");
          }
	| authorization_identifier
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("authorization_identifier", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("grantee", "authorization_identifier");
          }
	;


/*
*  Subsection 10.1:  <interval qualifier>.
*  Description:      Specify the precision of an interval data type.
*/

interval_qualifier:
	  start_field RW_TO end_field
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("start_field", $1.yCode);
            ptApplication->bindCodeParameter ("end_field",   $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_qualifier", "triadic");
          }
	| single_datetime_field
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("single_datetime_field", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_qualifier", "monadic");
          }
	;

start_field:
	  non_second_datetime_field LEFT_PAREN interval_leading_field_precision RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("non_second_datetime_field",        $1.yCode);
            ptApplication->bindCodeParameter ("interval_leading_field_precision", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("start_field", "with-interval_leading_field_precision");
          }
	| non_second_datetime_field
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("non_second_datetime_field", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("start_field", "without-interval_leading_field_precision");
          }
	;

end_field:
	  non_second_datetime_field
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("non_second_datetime_field", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("end_field", "non_second_datetime_field");
          }
	| RW_SECOND LEFT_PAREN interval_fractional_seconds_precision RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_fractional_seconds_precision", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("end_field", "with-interval_fractional_seconds_precision");
          }
	| RW_SECOND
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("end_field", "without-interval_fractional_seconds_precision");
          }
	;

single_datetime_field:
	  non_second_datetime_field
          LEFT_PAREN interval_leading_field_precision RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("non_second_datetime_field",        $1.yCode);
            ptApplication->bindCodeParameter ("interval_leading_field_precision", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("single_datetime_field", "non_second_datetime_field-monadic");
          }
        | non_second_datetime_field
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("non_second_datetime_field", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("single_datetime_field", "non_second_datetime_field");
          }
	| RW_SECOND 
	  LEFT_PAREN interval_leading_field_precision COMMA interval_fractional_seconds_precision RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_leading_field_precision",      $3.yCode);
            ptApplication->bindCodeParameter ("interval_fractional_seconds_precision", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("single_datetime_field", "second-triadic");
          }
	| RW_SECOND LEFT_PAREN interval_leading_field_precision RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_leading_field_precision", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("single_datetime_field", "second-monadic");
          }
	| RW_SECOND 
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("single_datetime_field", "second");
          }
	;
	
datetime_field:
	  non_second_datetime_field
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("non_second_datetime_field", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("datetime_field", "non_second_datetime_field");
          }
	| RW_SECOND
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("datetime_field", "second");
          }
	;

non_second_datetime_field:
	  RW_YEAR 
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("non_second_datetime_field", "year");
          }
	| RW_MONTH 
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("non_second_datetime_field", "month");
          }
	| RW_DAY 
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("non_second_datetime_field", "day");
          }
	| RW_HOUR 
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("non_second_datetime_field", "hour");
          }
	| RW_MINUTE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("non_second_datetime_field", "minute");
          }
	;

interval_fractional_seconds_precision:
	  UNSIGNED_INTEGER
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unsigned_integer", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_fractional_seconds_precision");
          }
	;

interval_leading_field_precision:
	  UNSIGNED_INTEGER
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unsigned_integer", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_leading_field_precision");
          }
	;


/*
*  Section 8. Predicates.
*/

/*
*  Subsection 8.12:  <search condition>.
*  Description:      Specify a condition that has the truth value true, false, or
*                    unknown, depending on the result of applying boolean operators
*                    to specified conditions.
*/

search_condition:
	  boolean_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("boolean_term", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("search_condition", "monadic");
          }
	| search_condition RW_OR boolean_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("search_condition", $1.yCode);
            ptApplication->bindCodeParameter ("boolean_term",     $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("search_condition", "triadic");
          }
	;

boolean_term:
	  boolean_factor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("boolean_factor", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("boolean_term", "monadic");
          }
	| boolean_term RW_AND boolean_factor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("boolean_term",   $1.yCode);
            ptApplication->bindCodeParameter ("boolean_factor", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("boolean_term", "triadic");
          }
	;

boolean_factor:
	  RW_NOT boolean_test
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("boolean_test", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("boolean_factor", "dyadic");
          }
	|        boolean_test
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("boolean_test", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("boolean_factor", "monadic");
          }
	;

boolean_test:
	  boolean_primary RW_IS RW_NOT truth_value
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("boolean_primary", $1.yCode);
            ptApplication->bindCodeParameter ("truth_value",     $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("boolean_test", "n-adic");
          }
	| boolean_primary RW_IS        truth_value
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("boolean_primary", $1.yCode);
            ptApplication->bindCodeParameter ("truth_value",     $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("boolean_test", "triadic");
          }
	| boolean_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("boolean_primary", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("boolean_test", "monadic");
          }
	;

truth_value:
	  RW_TRUE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("truth_value", "true");
          }
	| RW_FALSE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("truth_value", "false");
          }
	| RW_UNKNOWN
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("truth_value", "unknown");
          }
	;

/*
*  Warning:  This rule has been modified, by adding <search condition value specification>.
*/
boolean_primary:
	  predicate
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("predicate", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("boolean_primary", "predicate");
          }
	| LEFT_PAREN search_condition RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("search_condition", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("boolean_primary", "search_condition");
          }
        | search_condition_value_specification
          {
            ptApplication->require (eSQL92_F_17_A_B);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("search_condition_value_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("boolean_primary", "search_condition_value_specification");
          }
	;


/*
*  Subsection 8.11:  <overlaps predicate>.
*  Description:      Specify a test for an overlap between two events.
*/

overlaps_predicate:
	  row_value_constructor_1 RW_OVERLAPS row_value_constructor_2
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor_1", $1.yCode);
            ptApplication->bindCodeParameter ("row_value_constructor_2", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("overlaps_predicate");
          }
	;

row_value_constructor_1:
	  row_value_constructor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("row_value_constructor_1");
	    $$.iItemCount = $1.iItemCount;
          }
	;

row_value_constructor_2:
	  row_value_constructor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("row_value_constructor_2");
	    $$.iItemCount = $1.iItemCount;
          }
	;


/*
*  Subsection 8.10:  <match predicate>.
*  Description:      Specify a test for matching rows.
*/

match_predicate:
	  row_value_constructor RW_MATCH RW_UNIQUE RW_PARTIAL table_subquery
          {
            ptApplication->require (eSQL92_LR_8_10_P1);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor", $1.yCode);
            ptApplication->bindCodeParameter ("table_subquery",        $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("match_predicate", "match-unique-partial");
          }
	| row_value_constructor RW_MATCH RW_UNIQUE RW_FULL    table_subquery
          {
            ptApplication->require (eSQL92_LR_8_10_P1);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor", $1.yCode);
            ptApplication->bindCodeParameter ("table_subquery",        $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("match_predicate", "match-unique-full");
          }
	| row_value_constructor RW_MATCH RW_UNIQUE            table_subquery
          {
            ptApplication->require (eSQL92_LR_8_10_P1);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor", $1.yCode);
            ptApplication->bindCodeParameter ("table_subquery",        $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("match_predicate", "match-unique");
          }
	| row_value_constructor RW_MATCH           RW_PARTIAL table_subquery
          {
            ptApplication->require (eSQL92_LR_8_10_P1);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor", $1.yCode);
            ptApplication->bindCodeParameter ("table_subquery",        $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("match_predicate", "match-partial");
          }
	| row_value_constructor RW_MATCH           RW_FULL    table_subquery
          {
            ptApplication->require (eSQL92_LR_8_10_P1);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor", $1.yCode);
            ptApplication->bindCodeParameter ("table_subquery",        $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("match_predicate", "match-full");
          }
	| row_value_constructor RW_MATCH                      table_subquery
          {
            ptApplication->require (eSQL92_LR_8_10_P1);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor", $1.yCode);
            ptApplication->bindCodeParameter ("table_subquery",        $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("match_predicate", "match");
          }
	;


/*
*  Subsection 8.9:  <unique predicate>.
*  Description:     Specify a test for the absence of duplicate rows.
*/

unique_predicate:
	  RW_UNIQUE table_subquery
          {
            ptApplication->require (eSQL92_LR_8_9_P2);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_subquery", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("unique_predicate");
          }
	;


/*
*  Subsection 8.8:  <exists predicate>.
*  Description:     Specify a test for a non-empty set.
*/

exists_predicate:
	  RW_EXISTS table_subquery
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_subquery", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("exists_predicate");
          }
	;


/*
*  Subsection 8.7:  <quantified comparison predicate>.
*  Description:     Specify a quantified comparison.
*/

quantified_comparison_predicate:
	  row_value_constructor comp_op quantifier table_subquery
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor", $1.yCode);
            ptApplication->bindCodeParameter ("comp_op",               $2.yCode);
            ptApplication->bindCodeParameter ("quantifier",            $3.yCode);
            ptApplication->bindCodeParameter ("table_subquery",        $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("quantified_comparison_predicate");
          }
	;

quantifier:
	  all 
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("all", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("quantifier", "all");
          }
	| some
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("some", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("quantifier", "some");
          }
	;

all:
	  RW_ALL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("all");
          }
	;

some:
	  RW_SOME 
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("some", "some");
          }
	| RW_ANY
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("some", "any");
          }
	;


/*
*  Subsection 8.6:  <null predicate>.
*  Description:     Specify a test for a null value.
*/

null_predicate:
	  row_value_constructor RW_IS RW_NOT RW_NULL
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("null_predicate", "is-not-null");
          }
	| row_value_constructor RW_IS        RW_NULL
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("null_predicate", "is-null");
          }
	;
	

/*
*  Subsection 8.5:  <like predicate>.
*  Description:     Specify a pattern-match comparison.
*/

/*
*  Warning:  This rule has been modified, changing "RW_NOT RW_LIKE" to
*            a new token called RW_NOT_LIKE.
*/
like_predicate:
	  match_value RW_NOT_LIKE pattern RW_ESCAPE escape_character
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("match_value",      $1.yCode);
            ptApplication->bindCodeParameter ("pattern",          $3.yCode);
            ptApplication->bindCodeParameter ("escape_character", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("like_predicate", "not_like-with-escape");
          }
	| match_value RW_NOT_LIKE pattern
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("match_value",      $1.yCode);
            ptApplication->bindCodeParameter ("pattern",          $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("like_predicate", "not_like");
          }
	| match_value RW_LIKE pattern RW_ESCAPE escape_character
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("match_value",      $1.yCode);
            ptApplication->bindCodeParameter ("pattern",          $3.yCode);
            ptApplication->bindCodeParameter ("escape_character", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("like_predicate", "like-with-escape");
          }
	| match_value RW_LIKE pattern
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("match_value",      $1.yCode);
            ptApplication->bindCodeParameter ("pattern",          $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("like_predicate", "like");
          }
	;

match_value:
	  character_value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("match_value");
          }
	;

pattern:
	  character_value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("pattern");
          }
	;

escape_character:
	  character_value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("escape_character");
          }
	;


/*
*  Subsection 8.4:  <in predicate>.
*  Description:     Specify a quantified comparison.
*/

in_predicate:
	  row_value_constructor RW_NOT RW_IN in_predicate_value
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor", $1.yCode);
            ptApplication->bindCodeParameter ("in_predicate_value",    $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("in_predicate", "not-in");
          }
	| row_value_constructor        RW_IN in_predicate_value
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor", $1.yCode);
            ptApplication->bindCodeParameter ("in_predicate_value",    $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("in_predicate", "in");
          }
	;
	
in_predicate_value:
	  table_subquery
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_subquery", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("in_predicate_value", "table_subquery");
            $$.iItemCount = $1.iItemCount;
          }
	| LEFT_PAREN in_value_list RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("in_value_list", $2.yCode);
            $$.yCode      = ptApplication->getClauseCode ("in_predicate_value", "in_value_list");
            $$.iItemCount = $2.iItemCount;
          }
	;

in_value_list:
	  in_value_list COMMA value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("in_value_list",    $1.yCode);
            ptApplication->bindCodeParameter ("value_expression", $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("in_value_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                     value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("in_value_list", "monadic");
            $$.iItemCount = 1;
          }
	;


/*
*  Subsection 8.3:  <between predicate>.
*  Description:     Specify a range comparison.
*/

/*
*  Warning:  Terminal row_value_constructor parameter has been
*            substituted by its name plus an ordinal, to let it
*            appear three times.
*/

between_predicate:
	  row_value_constructor RW_NOT RW_BETWEEN row_value_constructor RW_AND row_value_constructor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor_1", $1.yCode);
            ptApplication->bindCodeParameter ("row_value_constructor_2", $4.yCode);
            ptApplication->bindCodeParameter ("row_value_constructor_3", $6.yCode);
            $$.yCode = ptApplication->getClauseCode ("between_predicate", "not-between");
          }
	| row_value_constructor        RW_BETWEEN row_value_constructor RW_AND row_value_constructor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor_1", $1.yCode);
            ptApplication->bindCodeParameter ("row_value_constructor_2", $3.yCode);
            ptApplication->bindCodeParameter ("row_value_constructor_3", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("between_predicate", "between");
          }
	;


/*
*  Subsection 8.2:  <comparison predicate>.
*  Description:     Specify a comparison of two row values.
*/

comparison_predicate:
	  row_value_constructor comp_op row_value_constructor
          {
            ptApplication->require (eSQL92_SR_8_2_P1, &$1, &$3);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor_1", $1.yCode);
            ptApplication->bindCodeParameter ("comp_op",                 $2.yCode);
            ptApplication->bindCodeParameter ("row_value_constructor_2", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("comparison_predicate");
          }
	;

comp_op:
	  EQUALS_OPERATOR
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("comp_op", "equals_operator");
          }
	| NOT_EQUALS_OPERATOR
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("comp_op", "not_equals_operator");
          }
	| LESS_THAN_OPERATOR
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("comp_op", "less_than_operator");
          }
	| GREATER_THAN_OPERATOR
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("comp_op", "greater_than_operator");
          }
	| LESS_THAN_OR_EQUALS_OPERATOR
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("comp_op", "less_than_or_equals_operator");
          }
	| GREATER_THAN_OR_EQUALS_OPERATOR
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("comp_op", "greater_than_or_equals_operator");
          }
	;


/*
*  Subsection 8.1:  <predicate>.
*  Description:     Specify a condition that can be evaluated to give a truth value of
*                   true, false, or unknown.
*/

predicate:
	  comparison_predicate
	| between_predicate
	| in_predicate
	| like_predicate
	| null_predicate
	| quantified_comparison_predicate
	| exists_predicate
	| unique_predicate
	| match_predicate
	| overlaps_predicate
	;


/*
*  Section 7. Query expressions.
*/

/*
*  Subsection 7.11:  <scalar subquery>, <row subquery>, and <table subquery>.
*  Description:      Specify a scalar value, a row, or a table derived from a <query
*                    expression>.
*/

scalar_subquery:
	  subquery
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("subquery", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("scalar_subquery");
            $$.iItemCount = $1.iItemCount;
          }
	;

row_subquery:
	  subquery
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("subquery", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("row_subquery");
            $$.iItemCount = $1.iItemCount;
          }
	;

table_subquery:
	  subquery
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("subquery", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("table_subquery");
            $$.iItemCount = $1.iItemCount;
          }
	;

subquery:
	  LEFT_PAREN query_expression RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_expression", $2.yCode);
            $$.yCode      = ptApplication->getClauseCode ("subquery");
            $$.iItemCount = $2.iItemCount;
          }
	;


/*
*  Subsection 7.10:  <query expression>.
*  Description:      Specify a table.
*/

query_expression:
	  non_join_query_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("non_join_query_expression", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("query_expression", "non_join_query_expression");
            $$.iItemCount = $1.iItemCount;
          }
	| joined_table
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("joined_table", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("query_expression", "joined_table");
            $$.iItemCount = $1.iItemCount;
          }
	;

non_join_query_expression:
	  non_join_query_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("non_join_query_term", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("non_join_query_expression", "non_join_query_term");
            $$.iItemCount = $1.iItemCount;
          }
	| query_expression RW_UNION  RW_ALL corresponding_spec query_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_expression",   $1.yCode);
            ptApplication->bindCodeParameter ("corresponding_spec", $4.yCode);
            ptApplication->bindCodeParameter ("query_term",         $5.yCode);
            $$.yCode      = ptApplication->getClauseCode ("non_join_query_expression", "union-all-with-corresponding_spec");
            $$.iItemCount = $1.iItemCount;
          }
	| query_expression RW_UNION  RW_ALL                    query_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_expression", $1.yCode);
            ptApplication->bindCodeParameter ("query_term",       $4.yCode);
            $$.yCode      = ptApplication->getClauseCode ("non_join_query_expression", "union-all");
            $$.iItemCount = $1.iItemCount;
          }
	| query_expression RW_UNION         corresponding_spec query_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_expression",   $1.yCode);
            ptApplication->bindCodeParameter ("corresponding_spec", $3.yCode);
            ptApplication->bindCodeParameter ("query_term",         $4.yCode);
            $$.yCode      = ptApplication->getClauseCode ("non_join_query_expression", "union-with-corresponding_spec");
            $$.iItemCount = $1.iItemCount;
          }
	| query_expression RW_UNION                            query_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_expression", $1.yCode);
            ptApplication->bindCodeParameter ("query_term",       $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("non_join_query_expression", "union");
            $$.iItemCount = $1.iItemCount;
          }
	| query_expression RW_EXCEPT RW_ALL corresponding_spec query_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_expression",   $1.yCode);
            ptApplication->bindCodeParameter ("corresponding_spec", $4.yCode);
            ptApplication->bindCodeParameter ("query_term",         $5.yCode);
            $$.yCode      = ptApplication->getClauseCode ("non_join_query_expression", "except-all-with-corresponding_spec");
            $$.iItemCount = $1.iItemCount;
          }
	| query_expression RW_EXCEPT RW_ALL                    query_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_expression", $1.yCode);
            ptApplication->bindCodeParameter ("query_term",       $4.yCode);
            $$.yCode      = ptApplication->getClauseCode ("non_join_query_expression", "except-all");
            $$.iItemCount = $1.iItemCount;
          }
	| query_expression RW_EXCEPT        corresponding_spec query_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_expression",   $1.yCode);
            ptApplication->bindCodeParameter ("corresponding_spec", $3.yCode);
            ptApplication->bindCodeParameter ("query_term",         $4.yCode);
            $$.yCode      = ptApplication->getClauseCode ("non_join_query_expression", "except-with-corresponding_spec");
            $$.iItemCount = $1.iItemCount;
          }
	| query_expression RW_EXCEPT                           query_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_expression", $1.yCode);
            ptApplication->bindCodeParameter ("query_term",       $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("non_join_query_expression", "except");
            $$.iItemCount = $1.iItemCount;
          }
	;
	
query_term:
	  non_join_query_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("non_join_query_term", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("query_term", "non_join_query_term");
            $$.iItemCount = $1.iItemCount;
          }
	| joined_table
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("joined_table", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("query_term", "joined_table");
            $$.iItemCount = $1.iItemCount;
          }
	;

non_join_query_term:
	  non_join_query_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("non_join_query_primary", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("non_join_query_term", "non_join_query_primary");
            $$.iItemCount = $1.iItemCount;
          }
	| query_term RW_INTERSECT RW_ALL corresponding_spec query_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_term",         $1.yCode);
            ptApplication->bindCodeParameter ("corresponding_spec", $4.yCode);
            ptApplication->bindCodeParameter ("query_primary",      $5.yCode);
            $$.yCode      = ptApplication->getClauseCode ("non_join_query_term", "intersect-all-with-corresponding_spec");
            $$.iItemCount = $1.iItemCount;
          }
	| query_term RW_INTERSECT RW_ALL                    query_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_term",    $1.yCode);
            ptApplication->bindCodeParameter ("query_primary", $4.yCode);
            $$.yCode      = ptApplication->getClauseCode ("non_join_query_term", "intersect-all");
            $$.iItemCount = $1.iItemCount;
          }
	| query_term RW_INTERSECT        corresponding_spec query_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_term",         $1.yCode);
            ptApplication->bindCodeParameter ("corresponding_spec", $3.yCode);
            ptApplication->bindCodeParameter ("query_primary",      $4.yCode);
            $$.yCode      = ptApplication->getClauseCode ("non_join_query_term", "intersect-with-corresponding_spec");
            $$.iItemCount = $1.iItemCount;
          }
	| query_term RW_INTERSECT                           query_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("query_term",    $1.yCode);
            ptApplication->bindCodeParameter ("query_primary", $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("non_join_query_term", "intersect");
            $$.iItemCount = $1.iItemCount;
          }
	;

query_primary:
	  non_join_query_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("non_join_query_primary", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("query_primary", "non_join_query_primary");
            $$.iItemCount = $1.iItemCount;
          }
	| joined_table
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("joined_table", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("query_primary", "joined_table");
            $$.iItemCount = $1.iItemCount;
          }
	;

non_join_query_primary:
	  simple_table
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("simple_table", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("non_join_query_primary", "simple_table");
            $$.iItemCount = $1.iItemCount;
          }
	| LEFT_PAREN non_join_query_expression RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("non_join_query_expression", $2.yCode);
            $$.yCode      = ptApplication->getClauseCode ("non_join_query_primary", "non_join_query_expression");
            $$.iItemCount = $2.iItemCount;
          }
	;
	
simple_table:
	  query_specification
	| table_value_constructor
	| explicit_table
	;

explicit_table:
	  RW_TABLE table_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("explicit_table");
          }
	;

corresponding_spec:
	  RW_CORRESPONDING RW_BY LEFT_PAREN corresponding_column_list RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("corresponding_column_list", $4.yCode);
            $$.yCode      = ptApplication->getClauseCode ("corresponding_spec", "corresponding-by");
            $$.iItemCount = $4.iItemCount;
          }
	| RW_CORRESPONDING
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("corresponding_spec", "corresponding");
          }
	;

corresponding_column_list:
	  column_name_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name_list", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("corresponding_column_list");
            $$.iItemCount = $1.iItemCount;
          }
	;


/*
*  Subsection 7.9:  <query specification>.
*  Description:     Specify a table derived from the result of a <table expression>.
*/

/*
*  Warning:  This rules have been modified cause to
*            some problems with inner periods.
*            Originally were:
*
*  select_sublist:
*         derived_column
*	| qualifier PERIOD asterisk
*       ;
*/

query_specification:
	  RW_SELECT set_quantifier select_list table_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("set_quantifier",   $2.yCode);
            ptApplication->bindCodeParameter ("select_list",      $3.yCode);
            ptApplication->bindCodeParameter ("table_expression", $4.yCode);
            $$.yCode      = ptApplication->getClauseCode ("query_specification", "select-with-set_quantifier");
            $$.iItemCount = $3.iItemCount;
          }
	| RW_SELECT                select_list table_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("select_list",      $2.yCode);
            ptApplication->bindCodeParameter ("table_expression", $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("query_specification", "select");
            $$.iItemCount = $2.iItemCount;
          }
	;

/*
*  Auxiliary rule for:  select_list
*/
select_sublist_list:
	  select_sublist_list COMMA select_sublist
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("select_sublist_list", $1.yCode);
            ptApplication->bindCodeParameter ("select_sublist",      $3.yCode);
            $$.yCode        = ptApplication->getClauseCode ("select_sublist_list", "n-adic");
            $$.iItemCount   = $1.iItemCount + 1;
            $$.gHasAsterisk = ( $1.gHasAsterisk || $3.gHasAsterisk );
          }
	|                           select_sublist
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("select_sublist", $1.yCode);
            $$.yCode        = ptApplication->getClauseCode ("select_sublist_list", "monadic");
            $$.iItemCount   = 1;
            $$.gHasAsterisk = $1.gHasAsterisk;
          }
	;

select_list:
	  ASTERISK
          {
            ptApplication->initCodeParameters();
            $$.yCode        = ptApplication->getClauseCode ("select_list", "asterisk");
            $$.gHasAsterisk = true;
          }
	| select_sublist_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("select_sublist_list", $1.yCode);
            $$.yCode        = ptApplication->getClauseCode ("select_list", "select_sublist_list");
            $$.iItemCount   = $1.iItemCount;
            $$.gHasAsterisk = $1.gHasAsterisk;
          }
	;

select_sublist:
	  derived_column
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("derived_column", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("select_sublist", "derived_column");
          }
        | FOURTH_LEVEL_ASTERISK
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("catalog_name",            $1.yFourthIdentifier);
            ptApplication->bindCodeParameter ("unqualified_schema_name", $1.yThirdIdentifier);
            ptApplication->bindCodeParameter ("qualified_identifier",    $1.ySecondIdentifier);
            $$.yCode        = ptApplication->getClauseCode ("select_sublist", "with-all");
            $$.gHasAsterisk = true;
          }
        | THIRD_LEVEL_ASTERISK
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unqualified_schema_name", $1.yThirdIdentifier);
            ptApplication->bindCodeParameter ("qualified_identifier",    $1.ySecondIdentifier);
            $$.yCode        = ptApplication->getClauseCode ("select_sublist", "without-catalog_name");
            $$.gHasAsterisk = true;
          }
        | SECOND_LEVEL_ASTERISK
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("qualified_identifier", $1.ySecondIdentifier);
            $$.yCode        = ptApplication->getClauseCode ("select_sublist", "with-qualified_identifier");
            $$.gHasAsterisk = true;
          }
	;

derived_column:
	  value_expression as_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression", $1.yCode);
            ptApplication->bindCodeParameter ("as_clause",        $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("derived_column", "with-as_clause");
          }
	| value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("derived_column", "without-as_clause");
          }
	;
	
as_clause:
	  RW_AS column_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("as_clause");
          }
	|       column_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("as_clause");
          }
	;


/*
*  Subsection 7.8:  <having clause>.
*  Description:     Specify a grouped table derived by the elimination of groups from
*                   the result of the previously specified clause that do not meet the
*                   <search condition>.
*/

having_clause:
	  RW_HAVING search_condition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("search_condition", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("having_clause");
          }
	;


/*
*  Subsection 7.7:  <group by clause>.
*  Description:     Specify a grouped table derived by the application of the <group by
*                   clause> to the result of the previously specified clause.
*/

group_by_clause:
	  RW_GROUP RW_BY grouping_column_reference_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("grouping_column_reference_list", $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("group_by_clause");
            $$.iItemCount = $3.iItemCount;
          }
	;

grouping_column_reference_list:
	  grouping_column_reference_list COMMA grouping_column_reference
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("grouping_column_reference_list", $1.yCode);
            ptApplication->bindCodeParameter ("grouping_column_reference",      $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("grouping_column_reference_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                                      grouping_column_reference
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("grouping_column_reference", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("grouping_column_reference_list", "monadic");
            $$.iItemCount = 1;
          }
	;

grouping_column_reference:
	  column_reference collate_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_reference", $1.yCode);
            ptApplication->bindCodeParameter ("collate_clause",   $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("grouping_column_reference", "with-collate_clause");
          }
	| column_reference
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_reference", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("grouping_column_reference", "without-collate_clause");
          }
	;
	

/*
*  Subsection 7.6:  <where clause>.
*  Description:     Specify a table derived by the application of a <search condition>
*                   to the result of the preceding <from clause>.
*/

where_clause:
	  RW_WHERE search_condition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("search_condition", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("where_clause");
          }
	;


/*
*  Subsection 7.5:  <joined table>.
*  Description:     Specify a table derived from a Cartesian product, inner or outer
*                   join, or union join.
*/

joined_table:
	  cross_join
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("cross_join", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("joined_table", "cross_join");
          }
	| qualified_join
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("qualified_join", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("joined_table", "qualified_join");
            $$.iItemCount = $1.iItemCount;
          }
	| LEFT_PAREN joined_table RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("joined_table", $2.yCode);
            $$.yCode      = ptApplication->getClauseCode ("joined_table", "joined_table");
            $$.iItemCount = $2.iItemCount;
          }
	;

/*
*  Warning:  Terminal table_reference parameter has been
*            substituted by its name plus an ordinal, to
*            let it appear two times.
*/

cross_join:
	  table_reference RW_CROSS RW_JOIN table_reference
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_reference_1", $1.yCode);
            ptApplication->bindCodeParameter ("table_reference_2", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("cross_join");
          }
	;

/*
*  Warning:  Terminal table_reference parameter has been
*            substituted by its name plus an ordinal, to
*            let it appear two times.
*/

qualified_join:
	  table_reference RW_NATURAL join_type RW_JOIN table_reference join_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_reference_1",  $1.yCode);
            ptApplication->bindCodeParameter ("join_type",          $3.yCode);
            ptApplication->bindCodeParameter ("table_reference_2",  $5.yCode);
            ptApplication->bindCodeParameter ("join_specification", $6.yCode);
            $$.yCode = ptApplication->getClauseCode ("qualified_join", "natural-with-both");
          }
	| table_reference RW_NATURAL join_type RW_JOIN table_reference
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_reference_1",  $1.yCode);
            ptApplication->bindCodeParameter ("join_type",          $3.yCode);
            ptApplication->bindCodeParameter ("table_reference_2",  $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("qualified_join", "natural-without-join_specification");
          }
	| table_reference RW_NATURAL           RW_JOIN table_reference join_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_reference_1",  $1.yCode);
            ptApplication->bindCodeParameter ("table_reference_2",  $4.yCode);
            ptApplication->bindCodeParameter ("join_specification", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("qualified_join", "natural-without-join_type");
          }
	| table_reference            join_type RW_JOIN table_reference join_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_reference_1",  $1.yCode);
            ptApplication->bindCodeParameter ("join_type",          $2.yCode);
            ptApplication->bindCodeParameter ("table_reference_2",  $4.yCode);
            ptApplication->bindCodeParameter ("join_specification", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("qualified_join", "with-both");
          }
	| table_reference RW_NATURAL           RW_JOIN table_reference
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_reference_1", $1.yCode);
            ptApplication->bindCodeParameter ("table_reference_2", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("qualified_join", "natural-without-both");
          }
        | table_reference            join_type RW_JOIN table_reference
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_reference_1",  $1.yCode);
            ptApplication->bindCodeParameter ("join_type",          $2.yCode);
            ptApplication->bindCodeParameter ("table_reference_2",  $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("qualified_join", "without-join_specification");
          }
	| table_reference                      RW_JOIN table_reference join_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_reference_1",  $1.yCode);
            ptApplication->bindCodeParameter ("table_reference_2",  $3.yCode);
            ptApplication->bindCodeParameter ("join_specification", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("qualified_join", "without-join_type");
          }
	| table_reference                      RW_JOIN table_reference
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_reference_1", $1.yCode);
            ptApplication->bindCodeParameter ("table_reference_2", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("qualified_join", "without-both");
          }
	;

join_specification:
	  join_condition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("join_condition", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("join_specification", "join_condition");
          }
	| named_columns_join
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("named_columns_join", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("join_specification", "named_columns_join");
          }
	;

join_condition:
	  RW_ON search_condition
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("search_condition", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("join_condition");
          }
	;
	
named_columns_join:
	  RW_USING LEFT_PAREN join_column_list RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("join_column_list", $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("named_columns_join");
            $$.iItemCount = $3.iItemCount;
          }
	;
	
join_type:
	  RW_INNER
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("join_type", "inner");
          }
	| outer_join_type RW_OUTER
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("outer_join_type", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("join_type", "outer_join_type-with-outer");
          }
	| outer_join_type
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("outer_join_type", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("join_type", "outer_join_type");
          }
	| RW_UNION
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("join_type", "union");
          }
	;

outer_join_type:
	  RW_LEFT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("outer_join_type", "left");
          }
	| RW_RIGHT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("outer_join_type", "right");
          }
	| RW_FULL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("outer_join_type", "full");
          }
	;
	
join_column_list: 
	  column_name_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name_list", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("join_column_list");
            $$.iItemCount = $1.iItemCount;
          }
	;


/*
*  Subsection 7.4:  <from clause>.
*  Description:     Specify a table derived from one or more named tables.
*/

/*
*  Auxiliary rule for:  from_clause
*/
table_reference_list:
	  table_reference_list COMMA table_reference
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_reference_list", $1.yCode);
            ptApplication->bindCodeParameter ("table_reference",      $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("table_reference_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                            table_reference
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_reference", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("table_reference_list", "monadic");
            $$.iItemCount = 1;
          }
	;

from_clause:
	  RW_FROM table_reference_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_reference_list", $2.yCode);
            $$.yCode      = ptApplication->getClauseCode ("from_clause");
            $$.iItemCount = $2.iItemCount;
          }
	;


/*
*  Subsection 7.3:  <table expression>.
*  Description:     Specify a table or a grouped table.
*/

table_expression:
	  from_clause where_clause group_by_clause having_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("from_clause",     $1.yCode);
            ptApplication->bindCodeParameter ("where_clause",    $2.yCode);
            ptApplication->bindCodeParameter ("group_by_clause", $3.yCode);
            ptApplication->bindCodeParameter ("having_clause",   $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_expression", "with-all");
          }
	| from_clause where_clause group_by_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("from_clause",     $1.yCode);
            ptApplication->bindCodeParameter ("where_clause",    $2.yCode);
            ptApplication->bindCodeParameter ("group_by_clause", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_expression", "without-having_clause");
          }
	| from_clause where_clause                 having_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("from_clause",   $1.yCode);
            ptApplication->bindCodeParameter ("where_clause",  $2.yCode);
            ptApplication->bindCodeParameter ("having_clause", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_expression", "without-group_by_clause");
          }
	| from_clause              group_by_clause having_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("from_clause",     $1.yCode);
            ptApplication->bindCodeParameter ("group_by_clause", $2.yCode);
            ptApplication->bindCodeParameter ("having_clause",   $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_expression", "without-where_clause");
          }
	| from_clause where_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("from_clause",  $1.yCode);
            ptApplication->bindCodeParameter ("where_clause", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_expression", "with-where_clause");
          }
	| from_clause              group_by_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("from_clause",     $1.yCode);
            ptApplication->bindCodeParameter ("group_by_clause", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_expression", "with-group_by_clause");
          }
	| from_clause                              having_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("from_clause",   $1.yCode);
            ptApplication->bindCodeParameter ("having_clause", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_expression", "with-having_clause");
          }
	| from_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("from_clause", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_expression", "without-all");
          }
	;


/*
*  Subsection 7.2:  <table value constructor>.
*  Description:     Specify a set of <row value constructor>s to be constructed into a
*                   table.
*/

table_value_constructor:
	  RW_VALUES table_value_constructor_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_value_constructor_list", $2.yCode);
            $$.yCode      = ptApplication->getClauseCode ("table_value_constructor");
            $$.iItemCount = $2.iItemCount;
          }
	;

table_value_constructor_list:
	  table_value_constructor_list COMMA row_value_constructor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_value_constructor_list", $1.yCode);
            ptApplication->bindCodeParameter ("row_value_constructor",        $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("table_value_constructor_list", "n-adic");
            $$.iItemCount = $1.iItemCount + $3.iItemCount;
          }
	|                                    row_value_constructor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("table_value_constructor_list", "monadic");
            $$.iItemCount = $1.iItemCount;
          }
	;


/*
*  Subsection 7.1:  <row value constructor>.
*  Description:     Specify an ordered set of values to be constructed into a row or
*                   partial row.
*/

row_value_constructor:
	  row_value_constructor_element
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor_element", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("row_value_constructor", "row_value_constructor_element");
            $$.iItemCount = 1;
          }
	| LEFT_PAREN row_value_constructor_list RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor_list", $2.yCode);
            $$.yCode      = ptApplication->getClauseCode ("row_value_constructor", "row_value_constructor_list");
            $$.iItemCount = $2.iItemCount;
          }
	| row_subquery
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_subquery", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("row_value_constructor", "row_subquery");
            $$.iItemCount = $1.iItemCount;
          }
	;

row_value_constructor_list:
	  row_value_constructor_list COMMA row_value_constructor_element
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor_list",    $1.yCode);
            ptApplication->bindCodeParameter ("row_value_constructor_element", $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("row_value_constructor_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                                  row_value_constructor_element
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("row_value_constructor_element", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("row_value_constructor_list", "monadic");
            $$.iItemCount = 1;
          }
	;
	
row_value_constructor_element:
	  value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("row_value_constructor_element", "value_expression");
          }
	| null_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("null_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("row_value_constructor_element", "null_specification");
          }
	| default_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("default_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("row_value_constructor_element", "default_specification");
          }
	;

null_specification:
	  RW_NULL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("null_specification");
          }
	;

default_specification:
	  RW_DEFAULT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("default_specification");
          }
	;


/*
*  Section 6. Scalar expressions.
*/

/*
*  Subsection 6.15:  <interval value expression>.
*  Description:      Specify an interval value.
*/

interval_value_expression:
	  interval_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_term", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_value_expression", "interval_term");
          }
	| interval_value_expression_1 PLUS_SIGN  interval_term_1
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_value_expression_1", $1.yCode);
            ptApplication->bindCodeParameter ("interval_term_1",             $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_value_expression", "plus_sign");
          }
	| interval_value_expression_1 MINUS_SIGN interval_term_1
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_value_expression_1", $1.yCode);
            ptApplication->bindCodeParameter ("interval_term_1",             $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_value_expression", "minus_sign");
          }
	| LEFT_PAREN datetime_value_expression MINUS_SIGN datetime_term RIGHT_PAREN interval_qualifier
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("datetime_value_expression", $2.yCode);
            ptApplication->bindCodeParameter ("datetime_term",             $4.yCode);
            ptApplication->bindCodeParameter ("interval_qualifier",        $6.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_value_expression", "datetime_value_expression");
          }
	;

interval_term:
	  interval_factor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_factor", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_term", "interval_factor");
          }
	| interval_term_2 ASTERISK factor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_term_2", $1.yCode);
            ptApplication->bindCodeParameter ("factor",          $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_term", "asterisk");
          }
	| interval_term_2 SOLIDUS  factor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_term_2", $1.yCode);
            ptApplication->bindCodeParameter ("factor",          $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_term", "solidus");
          }
	| term ASTERISK interval_factor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_term_2", $1.yCode);
            ptApplication->bindCodeParameter ("factor",          $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_term", "term");
          }
	;

interval_factor:
	  sign interval_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("sign",             $1.yCode);
            ptApplication->bindCodeParameter ("interval_primary", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_factor", "with-sign");
          }
	|      interval_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_primary", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_factor", "without-sign");
          }
	;

interval_primary:
	  value_expression_primary interval_qualifier
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression_primary", $1.yCode);
            ptApplication->bindCodeParameter ("interval_qualifier",       $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_primary", "with-interval_qualifier");
          }
	| value_expression_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression_primary", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_primary", "without-interval_qualifier");
          }
	;

interval_value_expression_1:
	  interval_value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_value_expression_1");
          }
	;

interval_term_1:
	  interval_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_term", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_term_1");
          }
	;  

interval_term_2:
	  interval_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_term", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_term_2");
          }
	;


/*
*  Subsection 6.14:  <datetime value expression>.
*  Description:      Specify a datetime value.
*/

datetime_value_expression:
	  datetime_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("datetime_term", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("datetime_value_expression", "datetime_term");
          }
	| interval_value_expression PLUS_SIGN  datetime_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_value_expression", $1.yCode);
            ptApplication->bindCodeParameter ("datetime_term",             $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("datetime_value_expression", "interval_value_expression");
          }
	| datetime_value_expression PLUS_SIGN  interval_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("datetime_value_expression", $1.yCode);
            ptApplication->bindCodeParameter ("interval_term",             $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("datetime_value_expression", "datetime_value_expression-with-plus_sign");
          }
	| datetime_value_expression MINUS_SIGN interval_term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("datetime_value_expression", $1.yCode);
            ptApplication->bindCodeParameter ("interval_term",             $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("datetime_value_expression", "datetime_value_expression-with-minus_sign");
          }
	;
	
datetime_term:
	  datetime_factor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("datetime_factor", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("datetime_term");
          }
	;

datetime_factor:
	  datetime_primary time_zone
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("datetime_primary", $1.yCode);
            ptApplication->bindCodeParameter ("time_zone",        $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("datetime_factor", "with-time_zone");
          }
	| datetime_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("datetime_primary", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("datetime_factor", "without-time_zone");
          }
	;

datetime_primary:
	  value_expression_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression_primary", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("datetime_primary", "value_expression_primary");
          }
	| datetime_value_function
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("datetime_value_function", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("datetime_primary", "datetime_value_function");
          }
	;

time_zone:
	  RW_AT time_zone_specifier
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("time_zone_specifier", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("time_zone");
          }
	;

time_zone_specifier:
	  RW_LOCAL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("time_zone_specifier", "local");
          }
	| RW_TIME RW_ZONE interval_value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_value_expression", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("time_zone_specifier", "time-zone");
          }
	;


/*
*  Subsection 6.13:  <string value expression>.
*  Description:      Specify a character string value or a bit string value.
*/

string_value_expression:
	  character_value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("string_value_expression", "character_value_expression");
          }
	| bit_value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("bit_value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("string_value_expression", "bit_value_expression");
          }
	;

character_value_expression:
	  concatenation
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("concatenation", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("character_value_expression", "concatenation");
          }
	| character_factor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_factor", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("character_value_expression", "character_factor");
          }
	;

concatenation:
	  character_value_expression CONCATENATION_OPERATOR character_factor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_value_expression", $1.yCode);
            ptApplication->bindCodeParameter ("character_factor",           $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("concatenation");
          }
	;

character_factor:
	  character_primary collate_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_primary", $1.yCode);
            ptApplication->bindCodeParameter ("collate_clause",    $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("character_factor", "with-collate_clause");
          }
	| character_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_primary", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("character_factor", "without-collate_clause");
          }
	;

character_primary:
	  value_expression_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression_primary", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("character_primary", "value_expression_primary");
          }
	| string_value_function
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("string_value_function", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("character_primary", "string_value_function");
          }
	;

bit_value_expression:
	  bit_concatenation
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("bit_concatenation", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("bit_value_expression", "bit_concatenation");
          }
	| bit_factor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("bit_factor", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("bit_value_expression", "bit_factor");
          }
	;

bit_concatenation:
	  bit_value_expression CONCATENATION_OPERATOR bit_factor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("bit_value_expression", $1.yCode);
            ptApplication->bindCodeParameter ("bit_factor",           $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("bit_concatenation");
          }
	;
	
bit_factor:
	  bit_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("bit_primary", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("bit_factor");
          }
	;
	
bit_primary:
	  value_expression_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression_primary", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("bit_primary", "value_expression_primary");
          }
	| string_value_function
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("string_value_function", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("bit_primary", "string_value_function");
          }
	;


/*
*  Subsection 6.12:  <numeric value expression>.
*  Description:      Specify a numeric value.
*/

numeric_value_expression:
	  term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("term", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("numeric_value_expression", "term");
          }
	| numeric_value_expression PLUS_SIGN  term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("numeric_value_expression", $1.yCode);
            ptApplication->bindCodeParameter ("term",                     $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("numeric_value_expression", "plus_sign");
          }
	| numeric_value_expression MINUS_SIGN term
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("numeric_value_expression", $1.yCode);
            ptApplication->bindCodeParameter ("term",                     $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("numeric_value_expression", "minus_sign");
          }
	;
	
term:
	  factor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("factor", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("term", "factor");
          }
	| term ASTERISK factor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("term",   $1.yCode);
            ptApplication->bindCodeParameter ("factor", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("term", "asterisk");
          }
	| term SOLIDUS  factor
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("term",   $1.yCode);
            ptApplication->bindCodeParameter ("factor", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("term", "solidus");
          }
	;
	
factor:
	  sign numeric_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("sign",            $1.yCode);
            ptApplication->bindCodeParameter ("numeric_primary", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("factor", "with-sign");
          }
	|      numeric_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("numeric_primary", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("factor", "without-sign");
          }
	;

numeric_primary:
	  value_expression_primary
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression_primary", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("numeric_primary", "value_expression_primary");
          }
	| numeric_value_function
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("numeric_value_function", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("numeric_primary", "numeric_value_function");
          }
	;


/*
*  Subsection 6.11:  <value expression>.
*  Description:      Specify a value.
*/

value_expression:
	  numeric_value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("numeric_value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("value_expression", "numeric_value_expression");
          }
	| string_value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("string_value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("value_expression", "string_value_expression");
          }
	| datetime_value_expression
          {
	    ptApplication->require (eSQL92_LR_6_11_P2_A);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("datetime_value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("value_expression", "datetime_value_expression");
          }
	| interval_value_expression
          {
	    ptApplication->require (eSQL92_LR_6_11_P2_B);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("value_expression", "interval_value_expression");
          }
	;
	
value_expression_primary:
	  unsigned_value_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unsigned_value_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("value_expression_primary", "unsigned_value_specification");
          }
	| column_reference
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_reference", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("value_expression_primary", "column_reference");
          }
	| set_function_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("set_function_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("value_expression_primary", "set_function_specification");
          }
	| scalar_subquery
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("scalar_subquery", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("value_expression_primary", "scalar_subquery");
            $$.iItemCount = $1.iItemCount;
          }
	| case_expression
          {
	    ptApplication->require (eSQL92_LR_6_11_P2_C);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("case_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("value_expression_primary", "case_expression");
          }
	| LEFT_PAREN value_expression RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("value_expression_primary", "value_expression");
          }
	| cast_specification
          {
	    ptApplication->require (eSQL92_LR_6_11_P2_D);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("cast_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("value_expression_primary", "cast_specification");
          }
	;


/*
*  Subsection 6.10:  <cast specification>.
*  Description:      Specify a conditional value.
*/

cast_specification:
	  RW_CAST LEFT_PAREN cast_operand RW_AS cast_target RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("cast_operand", $3.yCode);
            ptApplication->bindCodeParameter ("cast_target",  $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("cast_specification");
          }
	;

cast_operand:
	  value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("cast_operand", "value_expression");
          }
	| RW_NULL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("cast_operand", "null");
          }
	;

cast_target:
	  domain_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("domain_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("cast_target", "domain_name");
          }
	| data_type
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("data_type", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("cast_target", "data_type");
          }
	;


/*
*  Subsection 6.9:  <case expression>.
*  Description:     Specify a conditional value.
*/

case_expression:
	  case_abbreviation
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("case_abbreviation", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("case_expression", "case_abbreviation");
          }
	| case_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("case_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("case_expression", "case_specification");
          }
	;

/*
*  Auxiliary rule for:  case_abbreviation
*/
value_expression_list:
	  value_expression_list COMMA value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression_list", $1.yCode);
            ptApplication->bindCodeParameter ("value_expression",      $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("value_expression_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                             value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("value_expression_list", "monadic");
            $$.iItemCount = 1;
          }
	;

/*
*  Warning:  Terminal value_expression parameter has been
*            substituted by its name plus an ordinal, to
*            let it appear two times.
*/

case_abbreviation:
	  RW_NULLIF   LEFT_PAREN value_expression COMMA value_expression RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression_1", $3.yCode);
            ptApplication->bindCodeParameter ("value_expression_2", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("case_abbreviation", "nullif");
          }
	| RW_COALESCE LEFT_PAREN value_expression_list                   RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression_list", $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("case_abbreviation", "coalesce");
            $$.iItemCount = $3.iItemCount;
          }
	;

case_specification:
	  simple_case
	| searched_case
	;

/*
*  Auxiliary rule for:  simple_case
*/
simple_when_clause_list:
	  simple_when_clause_list simple_when_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("simple_when_clause_list", $1.yCode);
            ptApplication->bindCodeParameter ("simple_when_clause",      $2.yCode);
            $$.yCode      = ptApplication->getClauseCode ("simple_when_clause_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                         simple_when_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("simple_when_clause", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("simple_when_clause_list", "monadic");
            $$.iItemCount = 1;
          }
	;

simple_case:
	  RW_CASE case_operand simple_when_clause_list else_clause RW_END
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("case_operand",            $2.yCode);
            ptApplication->bindCodeParameter ("simple_when_clause_list", $3.yCode);
            ptApplication->bindCodeParameter ("else_clause",             $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("simple_case", "with-else_clause");
          }
	| RW_CASE case_operand simple_when_clause_list             RW_END
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("case_operand",            $2.yCode);
            ptApplication->bindCodeParameter ("simple_when_clause_list", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("simple_case", "without-else_clause");
          }
	;

/*
*  Auxiliary rule for:  searched_case
*/
searched_when_clause_list:
	  searched_when_clause_list searched_when_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("searched_when_clause_list", $1.yCode);
            ptApplication->bindCodeParameter ("searched_when_clause",      $2.yCode);
            $$.yCode      = ptApplication->getClauseCode ("searched_when_clause_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                           searched_when_clause
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("searched_when_clause", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("searched_when_clause_list", "monadic");
            $$.iItemCount = 1;
          }
	;

searched_case:
	  RW_CASE searched_when_clause_list else_clause RW_END
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("searched_when_clause_list", $2.yCode);
            ptApplication->bindCodeParameter ("else_clause",               $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("searched_case", "with-else_clause");
          }
	| RW_CASE searched_when_clause_list             RW_END
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("searched_when_clause_list", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("searched_case", "without-else_clause");
          }
	;
	  
simple_when_clause:
	  RW_WHEN when_operand RW_THEN result
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("when_operand", $2.yCode);
            ptApplication->bindCodeParameter ("result",       $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("simple_when_clause");
          }
	;

searched_when_clause:
	  RW_WHEN search_condition RW_THEN result
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("search_condition", $2.yCode);
            ptApplication->bindCodeParameter ("result",           $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("searched_when_clause");
          }
	;

else_clause:
	  RW_ELSE result
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("result", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("else_clause");
          }
	;

case_operand:
	  value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("case_operand");
          }
	;

when_operand:
	  value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("when_operand");
          }
	;

result:
	  result_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("result_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("result", "result_expression");
          }
	| RW_NULL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("result", "null");
          }
	;

result_expression:
	  value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("result_expression");
          }
	;


/*
*  Subsection 6.8:  <datetime value function>.
*  Description:     Specify a function yielding a value of type datetime.
*/

datetime_value_function:
	  current_date_value_function
          {
	    ptApplication->require (eSQL92_LR_6_8_P2);
            $$ = $1;
          }
	| current_time_value_function
          {
	    ptApplication->require (eSQL92_LR_6_8_P2);
            $$ = $1;
          }
	| current_timestamp_value_function
          {
	    ptApplication->require (eSQL92_LR_6_8_P2);
            $$ = $1;
          }
	;

current_date_value_function:
	  RW_CURRENT_DATE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("current_date_value_function");
          }
	;

current_time_value_function:
	  RW_CURRENT_TIME LEFT_PAREN time_precision RIGHT_PAREN
          {
	    ptApplication->require (eSQL92_LR_6_8_P1);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("time_precision", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("current_time_value_function", "with-time_precision");
          }
	| RW_CURRENT_TIME
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("current_time_value_function", "without-time_precision");
          }
	;
	
current_timestamp_value_function:
	  RW_CURRENT_TIMESTAMP LEFT_PAREN timestamp_precision RIGHT_PAREN
          {
	    ptApplication->require (eSQL92_LR_6_8_P1);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("timestamp_precision", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("current_timestamp_value_function", "with-timestamp_precision");
          }
	| RW_CURRENT_TIMESTAMP
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("current_timestamp_value_function", "without-timestamp_precision");
          }
	;


/*
*  Subsection 6.7:  <string value function>.
*  Description:     Specify a function yielding a value of type character
*                   string or bit string.
*/

string_value_function:
	  character_value_function
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_value_function", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("string_value_function", "character_value_function");
          }
	| bit_value_function
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("bit_value_function", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("string_value_function", "bit_value_function");
          }
	;

character_value_function:
	  character_substring_function
	| fold
	| form_of_use_conversion
	| character_translation
	| trim_function
	;

character_substring_function:
	  RW_SUBSTRING LEFT_PAREN character_value_expression
	               RW_FROM start_position
                       RW_FOR string_length
                       RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_value_expression", $3.yCode);
            ptApplication->bindCodeParameter ("start_position",             $5.yCode);
            ptApplication->bindCodeParameter ("string_length",              $7.yCode);
            $$.yCode = ptApplication->getClauseCode ("character_substring_function", "with-string_length");
          }
	| RW_SUBSTRING LEFT_PAREN character_value_expression 
	               RW_FROM start_position
	               RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_value_expression", $3.yCode);
            ptApplication->bindCodeParameter ("start_position",             $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("character_substring_function", "without-string_length");
          }
	;

fold:
	  RW_UPPER LEFT_PAREN character_value_expression RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_value_expression", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("fold", "upper");
          }
	| RW_LOWER LEFT_PAREN character_value_expression RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_value_expression", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("fold", "lower");
          }
	;

form_of_use_conversion:
	  RW_CONVERT LEFT_PAREN character_value_expression
	             RW_USING form_of_use_conversion_name
	             RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_value_expression",  $3.yCode);
            ptApplication->bindCodeParameter ("form_of_use_conversion_name", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("form_of_use_conversion");
          }
	;

character_translation:
	  RW_TRANSLATE LEFT_PAREN character_value_expression
	               RW_USING translation_name
	               RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_value_expression", $3.yCode);
            ptApplication->bindCodeParameter ("translation_name",           $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("character_translation");
          }
	;

trim_function:
	  RW_TRIM LEFT_PAREN trim_operands RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("trim_operands", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("trim_function");
          }
	;

trim_operands:
	  trim_specification trim_character RW_FROM trim_source
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("trim_specification", $1.yCode);
            ptApplication->bindCodeParameter ("trim_character",     $2.yCode);
            ptApplication->bindCodeParameter ("trim_source",        $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("trim_operands", "with-both");
          }
	| trim_specification                RW_FROM trim_source
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("trim_specification", $1.yCode);
            ptApplication->bindCodeParameter ("trim_source",        $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("trim_operands", "without-trim_character");
          }
	|                    trim_character RW_FROM trim_source
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("trim_character", $1.yCode);
            ptApplication->bindCodeParameter ("trim_source",    $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("trim_operands", "without-trim_specification");
          }
	|                                   RW_FROM trim_source
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("trim_source", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("trim_operands", "without-both");
          }
	|                                           trim_source
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("trim_source", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("trim_operands", "without-both");
          }
	;

trim_source:
	  character_value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("trim_source");
          }
	;

trim_specification:
	  RW_LEADING
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("trim_source", "leading");
          }
	| RW_TRAILING
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("trim_source", "trailing");
          }
	| RW_BOTH
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("trim_source", "both");
          }
	;
	
trim_character:
	  character_value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("trim_character");
          }
	;

bit_value_function:
	  bit_substring_function
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("bit_substring_function", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("bit_value_function");
          }
	;

bit_substring_function:
	  RW_SUBSTRING LEFT_PAREN bit_value_expression
	               RW_FROM start_position
	               RW_FOR string_length
	               RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("bit_value_expression", $3.yCode);
            ptApplication->bindCodeParameter ("start_position",       $5.yCode);
            ptApplication->bindCodeParameter ("string_length",        $7.yCode);
            $$.yCode = ptApplication->getClauseCode ("bit_substring_function", "with-string_length");
          }
	| RW_SUBSTRING LEFT_PAREN bit_value_expression 
	               RW_FROM start_position
	               RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("bit_value_expression", $3.yCode);
            ptApplication->bindCodeParameter ("start_position",       $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("bit_substring_function", "without-string_length");
          }
	;

start_position:
	  numeric_value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("numeric_value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("start_position");
          }
	;

string_length:
	  numeric_value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("numeric_value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("string_length");
          }
	;


/*
*  Subsection 6.6:  <numeric value function>.
*  Description:     Specify a function yielding a value of type numeric.
*/

numeric_value_function:
	  position_expression
	| extract_expression
	| length_expression
	;

/*
*  Warning:  Terminal character_value_expression parameter has been
*            substituted by its name plus an ordinal, to let it
*            appear two times.
*/

position_expression:
	  RW_POSITION LEFT_PAREN character_value_expression
	              RW_IN character_value_expression
	              RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_value_expression_1", $3.yCode);
            ptApplication->bindCodeParameter ("character_value_expression_2", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("position_expression");
          }
	;
	
length_expression:
	  char_length_expression
	| octet_length_expression
	| bit_length_expression
	;

char_length_expression:
	  RW_CHAR_LENGTH      LEFT_PAREN string_value_expression RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("string_value_expression", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("char_length_expression");
          }
	| RW_CHARACTER_LENGTH LEFT_PAREN string_value_expression RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("string_value_expression", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("char_length_expression");
          }
	;
	
octet_length_expression:
	  RW_OCTET_LENGTH LEFT_PAREN string_value_expression RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("string_value_expression", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("octet_length_expression");
          }
	;

bit_length_expression:
	  RW_BIT_LENGTH LEFT_PAREN string_value_expression RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("string_value_expression", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("bit_length_expression");
          }
	;

extract_expression:
	  RW_EXTRACT LEFT_PAREN extract_field RW_FROM extract_source RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("extract_field",  $3.yCode);
            ptApplication->bindCodeParameter ("extract_source", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("extract_expression");
          }
	;

extract_field:
	  datetime_field
	| time_zone_field
	;

time_zone_field:
	  RW_TIMEZONE_HOUR
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("time_zone_field", "timezone_hour");
          }
	| RW_TIMEZONE_MINUTE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("time_zone_field", "timezone_minute");
          }
	;

extract_source:
	  datetime_value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("datetime_value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("extract_source", "datetime_value_expression");
          }
	| interval_value_expression
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_value_expression", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("extract_source", "interval_value_expression");
          }
	;


/*
*  Subsection 6.5:  <set function specification>.
*  Description:     Specify a value derived by the application of a function
*                   to an argument.
*/

set_function_specification:
	  RW_COUNT LEFT_PAREN ASTERISK RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("set_function_specification", "count");
          }
	| general_set_function
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("general_set_function", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("set_function_specification", "general_set_function");
          }
	;

general_set_function:
	  set_function_type LEFT_PAREN set_quantifier value_expression RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("set_function_type", $1.yCode);
            ptApplication->bindCodeParameter ("set_quantifier",    $3.yCode);
            ptApplication->bindCodeParameter ("value_expression",  $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("general_set_function", "with-set_quantifier");
          }
	| set_function_type LEFT_PAREN                value_expression RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("set_function_type", $1.yCode);
            ptApplication->bindCodeParameter ("value_expression",  $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("general_set_function", "without-set_quantifier");
          }
	;

set_function_type:
	  RW_AVG
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("set_function_type", "avg");
          }
	| RW_MAX
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("set_function_type", "max");
          }
	| RW_MIN
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("set_function_type", "min");
          }
	| RW_SUM
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("set_function_type", "sum");
          }
	| RW_COUNT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("set_function_type", "count");
          }
	;

set_quantifier:
	  RW_DISTINCT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("set_quantifier", "distinct");
          }
	| RW_ALL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("set_quantifier", "all");
          }
	;


/*
*  Subsection 6.4:  <column reference>.
*  Description:     Reference a column.
*/

/*
*  Warning:  This rules have been modified cause to
*            some problems with inner periods.
*            Originally were:
*
*  column_reference:
*	  qualifier PERIOD column_name
*	|                  column_name
*       ;
*/

column_reference:
          FOURTH_LEVEL_IDENTIFIER
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("catalog_name",            $1.yFourthIdentifier);
            ptApplication->bindCodeParameter ("unqualified_schema_name", $1.yThirdIdentifier);
            ptApplication->bindCodeParameter ("qualified_identifier",    $1.ySecondIdentifier);
            ptApplication->bindCodeParameter ("column_name",             $1.yFirstIdentifier);
            $$.yCode = ptApplication->getClauseCode ("column_reference", "with-all");
          }
        | THIRD_LEVEL_IDENTIFIER
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unqualified_schema_name", $1.yThirdIdentifier);
            ptApplication->bindCodeParameter ("qualified_identifier",    $1.ySecondIdentifier);
            ptApplication->bindCodeParameter ("column_name",             $1.yFirstIdentifier);
            $$.yCode = ptApplication->getClauseCode ("column_reference", "without-catalog_name");
          }
        | SECOND_LEVEL_IDENTIFIER
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("qualified_identifier",    $1.ySecondIdentifier);
            ptApplication->bindCodeParameter ("column_name",             $1.yFirstIdentifier);
            $$.yCode = ptApplication->getClauseCode ("column_reference", "without-catalog_name-without-unqualified_schema_name");
          }
	| column_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_reference", "with-column_name");
          }
	;


/*
*  Subsection 6.3:  <table reference>.
*  Description:     Reference a table.
*/

table_reference:
	  table_name RW_AS correlation_name LEFT_PAREN derived_column_list RIGHT_PAREN
          {
	    ptApplication->require (eSQL92_LR_6_3_P2_B);
	    ptApplication->require (eSQL92_LR_6_3_P2_C);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",          $1.yCode);
            ptApplication->bindCodeParameter ("correlation_name",    $3.yCode);
            ptApplication->bindCodeParameter ("derived_column_list", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_reference", "table_name-with-both");
          }
	| table_name       correlation_name LEFT_PAREN derived_column_list RIGHT_PAREN
          {
	    ptApplication->require (eSQL92_LR_6_3_P2_C);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",          $1.yCode);
            ptApplication->bindCodeParameter ("correlation_name",    $2.yCode);
            ptApplication->bindCodeParameter ("derived_column_list", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_reference", "table_name-with-both");
          }
	| table_name RW_AS correlation_name
          {
	    ptApplication->require (eSQL92_LR_6_3_P2_B);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",       $1.yCode);
            ptApplication->bindCodeParameter ("correlation_name", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_reference", "table_name-with-correlation_name");
          }
	| table_name       correlation_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name",       $1.yCode);
            ptApplication->bindCodeParameter ("correlation_name", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_reference", "table_name-with-correlation_name");
          }
	| table_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_reference", "table_name");
          }
	| derived_table RW_AS correlation_name LEFT_PAREN derived_column_list RIGHT_PAREN
          {
	    ptApplication->require (eSQL92_LR_6_3_P2_B);
	    ptApplication->require (eSQL92_LR_6_3_P2_C);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("derived_table",       $1.yCode);
            ptApplication->bindCodeParameter ("correlation_name",    $3.yCode);
            ptApplication->bindCodeParameter ("derived_column_list", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_reference", "derived_table-with-derived_column_list");
          }
	| derived_table       correlation_name LEFT_PAREN derived_column_list RIGHT_PAREN
          {
	    ptApplication->require (eSQL92_LR_6_3_P2_C);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("derived_table",       $1.yCode);
            ptApplication->bindCodeParameter ("correlation_name",    $2.yCode);
            ptApplication->bindCodeParameter ("derived_column_list", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_reference", "derived_table-with-derived_column_list");
          }
	| derived_table RW_AS correlation_name
          {
	    ptApplication->require (eSQL92_LR_6_3_P2_B);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("derived_table",    $1.yCode);
            ptApplication->bindCodeParameter ("correlation_name", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_reference", "derived_table");
          }
	| derived_table       correlation_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("derived_table",    $1.yCode);
            ptApplication->bindCodeParameter ("correlation_name", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_reference", "derived_table");
          }
	| joined_table
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("joined_table", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_reference", "joined_table");
          }
	;

derived_table:
	  table_subquery
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("table_subquery", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("derived_table");
            $$.iItemCount = $1.iItemCount;
          }
	;

derived_column_list:
	  column_name_list
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name_list", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("derived_column_list");
            $$.iItemCount = $1.iItemCount;
          }
	;

column_name_list:
	  column_name_list COMMA column_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name_list", $1.yCode);
            ptApplication->bindCodeParameter ("column_name",      $3.yCode);
            $$.yCode      = ptApplication->getClauseCode ("column_name_list", "n-adic");
            $$.iItemCount = $1.iItemCount + 1;
          }
	|                        column_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("column_name", $1.yCode);
            $$.yCode      = ptApplication->getClauseCode ("column_name_list", "monadic");
            $$.iItemCount = 1;
          }
	;


/*
*  Subsection 6.2:  <value specification> and <target specification>.
*  Description:     Specify one or more values, parameters, or variables.
*/

value_specification:
	  literal
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("value_specification", "literal");
          }
	| general_value_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("general_value_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("value_specification", "general_value_specification");
          }
	;

unsigned_value_specification:
	  unsigned_literal
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unsigned_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("unsigned_value_specification", "unsigned_literal");
          }
	| general_value_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("general_value_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("unsigned_value_specification", "general_value_specification");
          }
	;

/*
*  Warning:  This rule has been modified to
*            remove <parameter specification>.
*/
general_value_specification:
	  dynamic_parameter_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("dynamic_parameter_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("general_value_specification", "dynamic_parameter_specification");
          }
	| variable_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("variable_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("general_value_specification", "variable_specification");
          }
	| RW_USER
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("general_value_specification", "user");
          }
	| RW_CURRENT_USER
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("general_value_specification", "current_user");
          }
	| RW_SESSION_USER
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("general_value_specification", "session_user");
          }
	| RW_SYSTEM_USER
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("general_value_specification", "system_user");
          }
	| RW_VALUE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("general_value_specification", "value");
          }
	;

/*
*  Warning:  This rule has been modified to
*            remove <parameter name>.
*/
simple_value_specification:
	  embedded_variable_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("embedded_variable_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("simple_value_specification", "embedded_variable_name");
          }
	| literal
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("simple_value_specification", "literal");
          }
	;

/*
*  Warning:  This rule has been modified to
*            remove <parameter specification>.
*/
target_specification:
	  variable_specification
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("variable_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("target_specification");
          }
	;

/*
*  Warning:  This rule has been modified to
*            remove <parameter name>.
*/
simple_target_specification:
	  embedded_variable_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("embedded_variable_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("simple_target_specification");
          }
	;

dynamic_parameter_specification:
	  QUESTION_MARK
	;

/*
*  Warning:  This rule has been modified to
*            remove <indicator variable>.
*/
variable_specification:
          embedded_variable_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("embedded_variable_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("variable_specification");
          }
	;

/*
*  Warning:  This rule has been added to support
*            the rule <declare search condition>.
*/
search_condition_value_specification:
          embedded_variable_name
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("embedded_variable_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("search_condition_value_specification");
          }
	;


/*
*  Subsection 6.1:  <data type>.
*  Description:     Specify a data type.
*/

/*
*  Warning:  Character set specification has been removed.
*/
data_type:
	  character_string_type
	| national_character_string_type
	| bit_string_type
	| numeric_type
	| datetime_type
	| interval_type
	;

character_string_type:
	  RW_CHARACTER            LEFT_PAREN length RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("length", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("character_string_type", "character-with-length");
          }
	| RW_CHARACTER
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("character_string_type", "character");
          }
	| RW_CHAR                 LEFT_PAREN length RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("length", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("character_string_type", "character-with-length");
          }
	| RW_CHAR
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("character_string_type", "character");
          }
	| RW_CHARACTER RW_VARYING LEFT_PAREN length RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("length", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("character_string_type", "character-varying-with-length");
          }
	| RW_CHAR      RW_VARYING LEFT_PAREN length RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("length", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("character_string_type", "character-varying-with-length");
          }
	| RW_VARCHAR              LEFT_PAREN length RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("length", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("character_string_type", "character-varying-with-length");
          }
	;

national_character_string_type:
	  RW_NATIONAL RW_CHARACTER            LEFT_PAREN length RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("length", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("national_character_string_type", "character-with-length");
          }
	| RW_NATIONAL RW_CHARACTER
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("national_character_string_type", "character");
          }
	| RW_NATIONAL RW_CHAR                 LEFT_PAREN length RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("length", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("national_character_string_type", "character-with-length");
          }
	| RW_NATIONAL RW_CHAR
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("national_character_string_type", "character");
          }
	| RW_NCHAR                            LEFT_PAREN length RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("length", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("national_character_string_type", "character-with-length");
          }
	| RW_NCHAR
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("national_character_string_type", "character");
          }
	| RW_NATIONAL RW_CHARACTER RW_VARYING LEFT_PAREN length RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("length", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("national_character_string_type", "character-varying-with-length");
          }
	| RW_NATIONAL RW_CHAR      RW_VARYING LEFT_PAREN length RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("length", $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("national_character_string_type", "character-varying-with-length");
          }
	| RW_NCHAR                 RW_VARYING LEFT_PAREN length RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("length", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("national_character_string_type", "character-varying-with-length");
          }
	;
	
bit_string_type:
	  RW_BIT RW_VARYING LEFT_PAREN length RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("length", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("bit_string_type", "bit-varying-with-length");
          }
	| RW_BIT            LEFT_PAREN length RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("length", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("bit_string_type", "bit-with-length");
          }
	| RW_BIT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("bit_string_type", "bit");
          }
	;
	
numeric_type:
	  exact_numeric_type
	| approximate_numeric_type
	;

exact_numeric_type:
	  RW_NUMERIC LEFT_PAREN precision COMMA scale RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("precision", $3.yCode);
            ptApplication->bindCodeParameter ("scale",     $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("exact_numeric_type", "numeric-with-precision-with-scale");
          }
	| RW_NUMERIC LEFT_PAREN precision RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("precision", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("exact_numeric_type", "numeric-with-precision");
          }
	| RW_NUMERIC
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("exact_numeric_type", "numeric");
          }
	| RW_DECIMAL LEFT_PAREN precision COMMA scale RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("precision", $3.yCode);
            ptApplication->bindCodeParameter ("scale",     $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("exact_numeric_type", "decimal-with-precision-with-scale");
          }
	| RW_DECIMAL LEFT_PAREN precision RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("precision", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("exact_numeric_type", "decimal-with-precision");
          }
	| RW_DECIMAL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("exact_numeric_type", "decimal");
          }
	| RW_DEC LEFT_PAREN precision COMMA scale RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("precision", $3.yCode);
            ptApplication->bindCodeParameter ("scale",     $5.yCode);
            $$.yCode = ptApplication->getClauseCode ("exact_numeric_type", "decimal-with-precision-with-scale");
          }
	| RW_DEC LEFT_PAREN precision RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("precision", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("exact_numeric_type", "decimal-with-precision");
          }
	| RW_DEC
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("exact_numeric_type", "decimal");
          }
	| RW_INTEGER
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("exact_numeric_type", "integer");
          }
	| RW_INT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("exact_numeric_type", "integer");
          }
	| RW_SMALLINT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("exact_numeric_type", "smallint");
          }
	;

approximate_numeric_type:
	  RW_FLOAT LEFT_PAREN precision RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("precision", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("approximate_numeric_type", "float-with-precision");
          }
	| RW_FLOAT
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("approximate_numeric_type", "float");
          }
	| RW_REAL
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("approximate_numeric_type", "real");
          }
	| RW_DOUBLE RW_PRECISION
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("approximate_numeric_type", "double-precision");
          }
	;

length:
	  UNSIGNED_INTEGER
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unsigned_integer", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("length");
          }
	;

precision:
	  UNSIGNED_INTEGER
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unsigned_integer", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("precision");
          }
	;

scale:
	  UNSIGNED_INTEGER
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unsigned_integer", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("scale");
          }
	;

datetime_type:
	  RW_DATE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("datetime_type", "date");
          }
	| RW_TIME LEFT_PAREN time_precision RIGHT_PAREN RW_WITH RW_TIME RW_ZONE
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("time_precision", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("datetime_type", "time-with-time_precision-with-time-zone");
          }
	| RW_TIME LEFT_PAREN time_precision RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("time_precision", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("datetime_type", "time-with-time_precision");
          }
	| RW_TIME RW_WITH RW_TIME RW_ZONE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("datetime_type", "time-with-time-zone");
          }
	| RW_TIME
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("datetime_type", "time");
          }
	| RW_TIMESTAMP LEFT_PAREN timestamp_precision RIGHT_PAREN RW_WITH RW_TIME RW_ZONE
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("timestamp_precision", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("datetime_type", "timestamp-with-timestamp_precision-with-time-zone");
          }
	| RW_TIMESTAMP LEFT_PAREN timestamp_precision RIGHT_PAREN
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("timestamp_precision", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("datetime_type", "timestamp-with-timestamp_precision");
          }
	| RW_TIMESTAMP RW_WITH RW_TIME RW_ZONE
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("datetime_type", "timestamp-with-time-zone");
          }
	| RW_TIMESTAMP
          {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("datetime_type", "timestamp");
          }
	;

time_precision:
	  time_fractional_seconds_precision
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("time_fractional_seconds_precision", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("time_precision");
          }
	;

timestamp_precision:
	  time_fractional_seconds_precision
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("time_fractional_seconds_precision", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("timestamp_precision");
          }
	;

time_fractional_seconds_precision:
	  UNSIGNED_INTEGER
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unsigned_integer", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("time_fractional_seconds_precision");
          }
	;

interval_type:
	  RW_INTERVAL interval_qualifier
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_qualifier", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_type");
          }
	;


/*
*  Section 5. Lexical Elements.
*/

/*
*  Subsection 5.4:  Names and identifiers.
*  Description:     Specify names.
*/

identifier:
	  actual_identifier
	;

actual_identifier:
	  regular_identifier
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("regular_identifier", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("actual_identifier", "regular_identifier");
          }
	| DELIMITED_IDENTIFIER
	  {
            //
            //  Replace <double quote symbol> by its value.
            //  Replace <quote symbol> by its value.
            //
            $1.yCode.replaceAll ("\"\"", "\"");
            $1.yCode.replaceAll ("''",   "'");
	    ptApplication->require (eSQL92_LR_5_2_P2_A, &$1);
	    ptApplication->require (eSQL92_SR_5_2_P9,   &$1);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("delimited_identifier", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("actual_identifier", "delimited_identifier");
          }
	;

authorization_identifier:
	  identifier
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("identifier", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("authorization_identifier", "identifier");
          }
        | embedded_variable_name
          {
            ptApplication->require (eSQL92_F_5_4_B);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("embedded_variable_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("authorization_identifier", "embedded_variable_name");
          }
	;

table_name:
	  qualified_name
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("qualified_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("table_name");
          }
	;

domain_name:
	  qualified_name
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("qualified_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("domain_name");
          }
	;

/*
*  Warning:  This rules have been modified cause to
*            some problems with inner periods.
*            Original rules were:
*
*            schema_name:
*	               catalog_name PERIOD unqualified_schema_name
*	             |                     unqualified_schema_name
*                    ;
*/
schema_name:
          SECOND_LEVEL_IDENTIFIER
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("catalog_name",            $1.ySecondIdentifier);
            ptApplication->bindCodeParameter ("unqualified_schema_name", $1.yFirstIdentifier);
            $$.yCode = ptApplication->getClauseCode ("schema_name", "with-all");
          }
	| unqualified_schema_name
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unqualified_schema_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("schema_name", "without-catalog_name");
          }
	;

unqualified_schema_name:
	  identifier
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("identifier", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("unqualified_schema_name", "identifier");
          }
        | embedded_variable_name
          {
            ptApplication->require (eSQL92_F_5_4_A);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("embedded_variable_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("unqualified_schema_name", "embedded_variable_name");
          }
	;

/*
*  Warning:  This rules have been modified cause to
*            some problems with inner periods.
*            Originally was:
*
*            qualified_name:
*	               schema_name PERIOD qualified_identifier
*	             |                    qualified_identifier
*                    ;
*/
qualified_name:
          THIRD_LEVEL_IDENTIFIER
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("catalog_name",            $1.yThirdIdentifier);
            ptApplication->bindCodeParameter ("unqualified_schema_name", $1.ySecondIdentifier);
            ptApplication->bindCodeParameter ("qualified_identifier",    $1.yFirstIdentifier);
            $$.yCode = ptApplication->getClauseCode ("qualified_name", "with-all");
          }
        | SECOND_LEVEL_IDENTIFIER
          {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unqualified_schema_name", $1.ySecondIdentifier);
            ptApplication->bindCodeParameter ("qualified_identifier",    $1.yFirstIdentifier);
            $$.yCode = ptApplication->getClauseCode ("qualified_name", "without-catalog_name");
          }
	| qualified_identifier
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("qualified_identifier", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("qualified_name", "with-qualified_identifier");
          }
	;

qualified_identifier:
	  identifier
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("identifier", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("qualified_identifier");
          }
	;

column_name:
	  identifier
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("identifier", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("column_name");
          }
	;

correlation_name:
	  identifier
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("identifier", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("correlation_name");
          }
	;

cursor_name:
	  identifier
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("identifier", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("cursor_name", "identifier");
          }
        | embedded_variable_name
          {
            ptApplication->require (eSQL92_F_5_4_C);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("embedded_variable_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("cursor_name", "embedded_variable_name");
          }
	;

SQL_statement_name:
	  statement_name
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("statement_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("SQL_statement_name", "statement_name");
          }
	| extended_statement_name
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("extended_statement_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("SQL_statement_name", "extended_statement_name");
          }
	;

statement_name:
	  identifier
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("identifier", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("statement_name");
          }
	;

extended_statement_name:
	  scope_option simple_value_specification
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("scope_option",               $1.yCode);
            ptApplication->bindCodeParameter ("simple_value_specification", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("extended_statement_name", "with-scope_option");
          }
	|              simple_value_specification
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("simple_value_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("extended_statement_name", "without-scope_option");
          }
	;

/* <DISABLED id="dynamic statements disabled"> */
/* dynamic_cursor_name: */
/* 	  cursor_name */
/* 	  { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("cursor_name", $1.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("dynamic_cursor_name", "cursor_name"); */
/*           } */
/* 	| extended_cursor_name */
/* 	  { */
/*             ptApplication->initCodeParameters(); */
/*             ptApplication->bindCodeParameter ("extended_cursor_name", $1.yCode); */
/*             $$.yCode = ptApplication->getClauseCode ("dynamic_cursor_name", "extended_cursor_name"); */
/*           } */
/* 	; */
/* </DISABLED> */

extended_cursor_name:
	  scope_option simple_value_specification
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("scope_option",               $1.yCode);
            ptApplication->bindCodeParameter ("simple_value_specification", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("extended_cursor_name", "with-scope_option");
          }
	|              simple_value_specification
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("simple_value_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("extended_cursor_name", "without-scope_option");
          }
	;

descriptor_name:
	  scope_option simple_value_specification
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("scope_option",               $1.yCode);
            ptApplication->bindCodeParameter ("simple_value_specification", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("descriptor_name", "with-scope_option");
          }
	|              simple_value_specification
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("descriptor_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("extended_cursor_name", "without-scope_option");
          }
	;

scope_option:
	  RW_GLOBAL
	  {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("scope_option", "global");
          }
	| RW_LOCAL
	  {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("scope_option", "local");
          }
	;
	
constraint_name:
	  qualified_name
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("qualified_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("constraint_name");
          }
	;

collation_name:
	  qualified_name
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("qualified_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("collation_name");
          }
	;

translation_name:
	  qualified_name
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("qualified_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("translation_name");
          }
	;
	
form_of_use_conversion_name:
	  qualified_name
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("qualified_name", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("form_of_use_conversion_name");
          }
	;

connection_name:
	  simple_value_specification
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("simple_value_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("connection_name");
          }
	;

schema_location:
	  simple_value_specification
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("simple_value_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("schema_location");
          }
	;

user_login:
	  simple_value_specification
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("simple_value_specification", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("user_login");
          }
	;

/*
*  Warning:  This rules are needed for using with
*            non-terminal <column reference>. These
*            are the equivalent rules:
*
*            So the new rules must fulfil the same
*            invariants than the original rules.
*/

/*
*  Subsection 5.3:  <literal>.
*  Description:     Specify a non-null value.
*/

literal:
          signed_numeric_literal
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("signed_numeric_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("literal", "signed_numeric_literal");
          }
        | general_literal
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("general_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("literal", "general_literal");
          }
        ;

unsigned_literal:
          unsigned_numeric_literal
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unsigned_numeric_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("unsigned_literal", "unsigned_numeric_literal");
          }
        | general_literal
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("general_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("unsigned_literal", "general_literal");
          }
        ;

general_literal:
          CHARACTER_STRING_LITERAL
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("character_string_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("general_literal", "character_string_literal");
          }
        | NATIONAL_CHARACTER_STRING_LITERAL
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("national_character_string_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("general_literal", "national_character_string_literal");
          }
        | BIT_STRING_LITERAL
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("bit_string_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("general_literal", "bit_string_literal");
          }
        | HEX_STRING_LITERAL
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("hex_string_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("general_literal", "hex_string_literal");
          }
        | datetime_literal
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("datetime_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("general_literal", "datetime_literal");
          }
        | interval_literal
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("general_literal", "interval_literal");
          }
        ;

signed_numeric_literal:
	  sign unsigned_numeric_literal
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("sign",                     $1.yCode);
            ptApplication->bindCodeParameter ("unsigned_numeric_literal", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("signed_numeric_literal", "with-sign");
          }
	|      unsigned_numeric_literal
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unsigned_numeric_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("signed_numeric_literal", "without-sign");
          }
	;

unsigned_numeric_literal:
	  exact_numeric_literal
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("exact_numeric_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("unsigned_numeric_literal", "exact_numeric_literal");
          }
	| approximate_numeric_literal
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("approximate_numeric_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("unsigned_numeric_literal", "approximate_numeric_literal");
          }
	;

exact_numeric_literal:
	  REAL_EXACT_NUMERIC_LITERAL
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("real_exact_numeric_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("exact_numeric_literal", "real_exact_numeric_literal");
          }
	| UNSIGNED_INTEGER
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("unsigned_integer", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("exact_numeric_literal", "unsigned_integer");
          }
	;

sign:
	  PLUS_SIGN
	  {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("sign", "plus_sign");
          }
	| MINUS_SIGN
	  {
            ptApplication->initCodeParameters();
            $$.yCode = ptApplication->getClauseCode ("sign", "minus_sign");
          }
	;

approximate_numeric_literal:
	  APPROXIMATE_NUMERIC_LITERAL
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("approximate_numeric_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("approximate_numeric_literal");
          }
	;

datetime_literal:
	  date_literal
	  {
	    ptApplication->require (eSQL92_SR_5_3_P21, &$1);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("date_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("datetime_literal", "date_literal");
          }
	| time_literal
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("time_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("datetime_literal", "time_literal");
          }
	| timestamp_literal
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("timestamp_literal", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("datetime_literal", "timestamp_literal");
          }
	;

date_literal:
	  RW_DATE DATE_STRING
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("date_string", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("date_literal");
          }
	;

time_literal:
	  RW_TIME TIME_STRING
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("time_string", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("time_literal");
          }
	;

timestamp_literal:
	  RW_TIMESTAMP TIMESTAMP_STRING
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("timestamp_string", $2.yCode);
            $$.yCode = ptApplication->getClauseCode ("timestamp_literal");
          }
	;

interval_literal:
	  RW_INTERVAL sign INTERVAL_STRING interval_qualifier
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("sign",               $2.yCode);
            ptApplication->bindCodeParameter ("interval_string",    $3.yCode);
            ptApplication->bindCodeParameter ("interval_qualifier", $4.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_literal", "with-sign");
          }
	| RW_INTERVAL      INTERVAL_STRING interval_qualifier
	  {
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("interval_string",    $2.yCode);
            ptApplication->bindCodeParameter ("interval_qualifier", $3.yCode);
            $$.yCode = ptApplication->getClauseCode ("interval_literal", "without-sign");
          }
	;


/*
*  Subsection 5.2:  <token> and <separator>.
*  Description:     Specify lexical units (tokens and separators)
*                   that participate in SQL language.
*/

regular_identifier:
	  identifier_body
          {
	    ptApplication->require (eSQL92_LR_5_2_P1,   &$1);
	    ptApplication->require (eSQL92_LR_5_2_P2_A, &$1);
            ptApplication->initCodeParameters();
            ptApplication->bindCodeParameter ("identifier_body", $1.yCode);
            $$.yCode = ptApplication->getClauseCode ("regular_identifier");
          }
	;

identifier_body:
	  NRW_CATALOG_NAME
	| NRW_CHARACTER_SET_CATALOG
	| NRW_CHARACTER_SET_NAME
	| NRW_CHARACTER_SET_SCHEMA
	| NRW_CLASS_ORIGIN
	| NRW_COLLATION_CATALOG
	| NRW_COLLATION_NAME
	| NRW_COLLATION_SCHEMA
	| NRW_COLUMN_NAME
	| NRW_COMMAND_FUNCTION
	| NRW_COMMITTED
	| NRW_CONDITION_NUMBER
	| NRW_CONNECTION_NAME
	| NRW_CONSTRAINT_CATALOG
	| NRW_CONSTRAINT_NAME
	| NRW_CONSTRAINT_SCHEMA
	| NRW_CURSOR_NAME
	| NRW_DATA
	| NRW_DATETIME_INTERVAL_CODE
	| NRW_DATETIME_INTERVAL_PRECISION
	| NRW_DYNAMIC_FUNCTION
	| NRW_LENGTH
	| NRW_MESSAGE_LENGTH
	| NRW_MESSAGE_OCTET_LENGTH
	| NRW_MESSAGE_TEXT
	| NRW_MORE
	| NRW_NAME
	| NRW_NULLABLE
	| NRW_NUMBER
	| NRW_REPEATABLE
	| NRW_RETURNED_LENGTH
	| NRW_RETURNED_OCTET_LENGTH
	| NRW_RETURNED_SQLSTATE
	| NRW_ROW_COUNT
	| NRW_SCALE
	| NRW_SCHEMA_NAME
	| NRW_SERIALIZABLE
	| NRW_SERVER_NAME
	| NRW_SUBCLASS_ORIGIN
	| NRW_TABLE_NAME
	| NRW_TYPE
	| NRW_UNCOMMITTED
	| NRW_UNNAMED
	| IDENTIFIER_BODY
          {
	    ptApplication->require (eSQL92_SR_5_2_P8,   &$1);
	    ptApplication->require (eSQL92_LR_5_2_P1,   &$1);
	    ptApplication->require (eSQL92_LR_5_2_P2_B, &$1);
	    $$ = $1;
	  }
	;

%%


//
//  Y A C C   F U N C T I O N S
//

void yyerror (const char* pccMESSAGE)
{

  ptApplication->handleCompilerError (pccMESSAGE);  
  
}  // yyerror()
